﻿using System;
using System.Collections.Generic;
using GestioService;
using XiquiparcsReunits_DAO;
using System.ServiceModel;
using System.Data.Services;

namespace GestioImplementation
{
    public class GestorXiquiparc : IGestorXiquiparc
    {
        /// <summary>
        /// This method is called only once to initialize service-wide policies.
        /// </summary>
        /// <param name="config"></param>
        public static void InitializeService(IDataServiceConfiguration config)
        {
            // Make certain entity sets writable.
            config.SetEntitySetAccessRule ("Cobrament", EntitySetRights.All);
            config.SetEntitySetAccessRule("Empresa", EntitySetRights.All);
            config.SetEntitySetAccessRule("Factura", EntitySetRights.All);
            config.SetEntitySetAccessRule("Oferta_facturada", EntitySetRights.All);
            config.SetEntitySetAccessRule("Recurs_facturat", EntitySetRights.All);
            config.SetEntitySetAccessRule("TipusIVA", EntitySetRights.All);
            config.SetEntitySetAccessRule("Recurs", EntitySetRights.All);
            config.SetEntitySetAccessRule("Oferta", EntitySetRights.All);
            config.SetEntitySetAccessRule("Seccio", EntitySetRights.All);
            config.SetEntitySetAccessRule("Dret", EntitySetRights.All);
            config.SetEntitySetAccessRule("Rol", EntitySetRights.All);
            config.SetEntitySetAccessRule("Usuari", EntitySetRights.All);
            config.SetEntitySetAccessRule("Continent", EntitySetRights.All);
            config.SetEntitySetAccessRule("Contingut", EntitySetRights.All);
            config.SetEntitySetAccessRule("Pagina", EntitySetRights.All);

            // Make the remaining entity sets read-only.
            //config.SetEntitySetAccessRule("*", EntitySetRights.AllRead);
        }

        #region IGestioServiceMembers
        
        
        /// <summary>
        /// Obté el llistat de totes les instancies del sistema
        /// </summary>
        /// <returns>Llista amb totes les ofertes</returns>
        public List<Oferta> GetOfertes() { return Oferta.GetInstanciesActives(); }

        
        /// <summary>
        /// Obté el llistat de tots els noms de les instancies del sistema
        /// </summary>
        /// <returns>Llista amb tots els noms de les ofertes</returns>
        public List<string> GetNomOfertes() { return Oferta.GetIdInstanciesActives(); }

        /// <summary>
        /// Obté el llistat de totes les instacies del sistema d'un recurs
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <returns>Llista amb totes les ofertes</returns>
        public List<Oferta> GetOfertes(string id_recurs) { return Oferta.GetInstanciesActivesByRecurs(id_recurs); }

        /// <summary>
        /// Obté el llistat de tots els noms de les instacies del sistema d'un recurs
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <returns>Llista amb tots els noms de les ofertes</returns>
        public List<string> GetNomOfertes(string id_recurs) { return Oferta.GetIdInstanciesActivesByRecurs(id_recurs); }

        /// <summary>
        /// Obté una oferta a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de la oferta</param>
        /// <returns>Oferta buscada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Oferta GetOferta(String id) {
            Oferta result = Oferta.GetInstancia(id);
            if (result !=null)
                return result; 
            else {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra (desactiva) una oferta del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id">Identificador de la oferta a esborrar</param>
        /// <param name="usuari_esborra">Usuari que esborra la oferta</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteOferta(String id, String usuari_esborra)
        {
            Oferta oferta = Oferta.GetInstancia(id);         
            if (oferta == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).oferta[2].Equals('1'))
            {
                //oferta.Borrar();
                oferta.actiu_oferta = false;
               // oferta.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar una oferta"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Modifica una oferta del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id_oferta">Identificador de l'oferta a modificar</param>
        /// <param name="import_unitari">Nou import unitari del recurs ofertat</param>
        /// <param name="id_recurs">Identificador del recurs ofertat</param>
        /// <param name="quantitat_ofertada">Nova quantitat de recurs ofertat</param>
        /// <param name="data_inici_oferta">Nova data i hora d'inici de l'oferta</param>
        /// <param name="data_fi_oferta">Nova data i hora de fi de l'oferta</param>
        /// <param name="usuari_mod_oferta">Usuari que modifica l'oferta</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateOferta(String id_oferta, Decimal import_unitari, String id_recurs, int quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, String usuari_mod_oferta)
        {
            Oferta oferta = Oferta.GetInstancia(id_oferta);
            if (oferta == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(oferta.usuari_mod_oferta).nom_rol).oferta[1].Equals('1'))
            {
                oferta.import_unitari = import_unitari;
                oferta.id_recurs = id_recurs;
                oferta.quantitat_ofertada = quantitat_ofertada;
                oferta.data_inici_oferta = data_inici_oferta;
                oferta.data_fi_oferta = data_fi_oferta;
                oferta.usuari_mod_oferta = usuari_mod_oferta;
                oferta.data_mod_oferta = DateTime.Now;
                //oferta.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar una oferta"
            }, new FaultReason("Error"));
        }
        
        /// <summary>
        /// Crea una nova oferta i ho registra a la persistència
        /// </summary>
        /// <param name="import_unitari">Import unitari del recurs ofertat</param>
        /// <param name="id_recurs">Identificador del recurs ofertat</param>
        /// <param name="quantitat_ofertada">Quantitat de recurs ofertat</param>
        /// <param name="data_inici_oferta">Data d'inici de l'oferta</param>
        /// <param name="data_fi_oferta">Data de fi de l'oferta</param>
        /// <param name="usuari_crea_oferta">Usuari que crea l'oferta</param>
        /// <returns>Oferta creada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Oferta CreateOferta(Decimal import_unitari, String id_recurs, int quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, String usuari_crea_oferta)
        {
            Oferta oferta;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_oferta).nom_rol).oferta[0].Equals('1'))
            {
                oferta = Oferta.CreateOferta(import_unitari, id_recurs, quantitat_ofertada, data_inici_oferta, data_fi_oferta, usuari_crea_oferta);
                oferta.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear una oferta"
            }, new FaultReason("Error"));
            return oferta;
        }

        /// <summary>
        /// Obté el llisat de tots els recursos no esborrats del sistema
        /// </summary>
        /// <returns>Llistat amb tots els recursos del sistema</returns>
        public List<Recurs> GetRecursos() { return Recurs.GetInstanciesActives(); }


        /// <summary>
        /// Obté el llisat de tots els noms de recursos no esborrats
        /// </summary>
        /// <returns>Llistat amb tots els noms de recursos</returns>
        public List<string> GetNomRecursos() { return Recurs.GetIdInstanciesActives(); }

        /// <summary>
        /// Obté el llisat de tots els recursos no esborrats d'una secció
        /// </summary>
        /// <param name="id.seccio">Identificador de la secció dels recursos</param>
        /// <returns>Llistat amb tots els recursos d'una secció</returns>
        public List<Recurs> GetRecursos(string id_seccio) { return Recurs.GetInstanciesActivesBySeccio(id_seccio); }

        /// <summary>
        /// Obté el llisat de tots els noms de recursos no esborrats d'una secció
        /// </summary>
        /// <param name="id_seccio">Identificador de secció dels recursos</param>
        /// <returns>Llistat amb tots els noms de recursos d'una secció</returns>
        public List<string> GetNomRecursos(string id_seccio) { return Recurs.GetIdInstanciesActivesBySeccio(id_seccio); }

        /// <summary>
        /// Obté un recurs a partir del seu nom identificador
        /// </summary>
        /// <param name="nom">Nom identificador del recurs buscat</param>
        /// <returns>Recurs buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Recurs GetRecurs(String nom) {
            Recurs result = Recurs.GetInstancia(nom);
            if (result !=null)
                return result; 
            else {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra un recurs (el desactiva), també esborra totes les ofertes del recurs, i ho registra a la persistència
        /// </summary>
        /// <param name="nom">Nom identificador del recurs a esborrar</param>
        /// <param name="usuari_esborra">Usuari que esborra el recurs</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteRecurs(String nom, String usuari_esborra)
        {
            Recurs recurs = Recurs.GetInstancia(nom);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).recurs[2].Equals('1'))
            {
                //recurs.Borrar();
                recurs.actiu_recurs = false;
                //recurs.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el recurs"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Modifica un recurs del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <param name="id_seccio">Identificador de la secció del recurs</param>
        /// <param name="preu">Preu normal del recurs</param>
        /// <param name="nom_IVA">Tipus d'IVA a aplicar al recurs</param>
        /// <param name="quantitat">Quantitat disponible del recurs</param>
        /// <param name="disponibilitat_controlable">booleà que representa si s'ha de controlar les unitat utilitzades de recurs</param>
        /// <param name="descripcio">Nova descripció del recurs</param>
        /// <param name="usuari_mod_recurs">Usuari que modifica el recurs</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateRecurs(String id_recurs, String id_seccio, Decimal preu, String nom_IVA, int quantitat, bool disponibilitat_controlable, String descripcio, String usuari_mod_recurs)
        {
            Recurs recurs = Recurs.GetInstancia(id_recurs);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(recurs.usuari_mod_recurs).nom_rol).recurs[1].Equals('1'))
            {
                recurs.id_seccio = id_seccio;
                recurs.preu = preu;
                recurs.nom_IVA = nom_IVA;
                recurs.quantitat = quantitat;
                recurs.disponibilitat_controlable = disponibilitat_controlable;
                recurs.descripcio = descripcio;
                recurs.usuari_mod_recurs = usuari_mod_recurs;
                recurs.data_mod_recurs = DateTime.Now;
                //recurs.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el recurs"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Crea un recurs del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <param name="id_seccio">Identificador de la secció del recurs</param>
        /// <param name="preu">Preu nomal del nou recurs</param>
        /// <param name="nom_IVA">Tipus d'IVA del nou recurs</param>
        /// <param name="quantitat">Quantitat del nou recurs</param>
        /// <param name="disponibilitat_controlable">Evalua si la quantitat de recurs disposada s'ha de controlar</param>
        /// <param name="descripcio">Descripcio del recurs</param>
        /// <param name="usuari_crea_recurs">Usuari que crea el recurs</param>
        /// <returns>Recurs creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Recurs CreateRecurs(String id_recurs, String id_seccio, Decimal preu, String nom_IVA, int quantitat, bool disponibilitat_controlable, string descripcio, String usuari_crea_recurs)
        {
            Recurs recurs;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_recurs).nom_rol).recurs[0].Equals('1'))
            {
                recurs = Recurs.CreateRecurs(id_recurs, id_seccio, preu, nom_IVA, disponibilitat_controlable, quantitat, descripcio, usuari_crea_recurs);
                //recurs.Refrescar();
                recurs.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el recurs"
            }, new FaultReason("Error"));
            return recurs;
        }

        /// <summary>
        /// Obté un llistat de totes les seccions del sistema
        /// </summary>
        /// <returns>Llistat amb totes les seccions</returns>
        public List<Seccio> GetSeccions() { return Seccio.GetInstanciesActives(); }

        /// <summary>
        /// Obté un llistat dels noms de totes les seccions del sistema
        /// </summary>
        /// <returns>Llistat amb els noms de totes les seccions</returns>
        public List<string> GetNomSeccions() { return Seccio.GetIdInstanciesActives(); }

        /// <summary>
        /// Obté una secció a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de la secció buscada</param>
        /// <returns>Secció buscada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Seccio GetSeccio(String id) { 
            Seccio result = Seccio.GetInstancia(id);
            if (result !=null)
                return result; 
            else {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la secció"
                }, new FaultReason("Error"));
            }
        }
        /// <summary>
        /// Elimina una secció i tots els recursos d'aquesta del sistema, i ho registra a la persistència
        /// </summary>
        /// <param name="id">Identificador de la secció</param>
        /// <param name="usuari_esborra">Usuari que esborra la secció</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteSeccio(String id, String usuari_esborra)
        {
            Seccio seccio = Seccio.GetInstancia(id);
            if (seccio == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la secció"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).seccio[2].Equals('1'))
            {
                //seccio.Borrar();
                seccio.actiu_seccio = false;
                //seccio.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar la secció"
            }, new FaultReason("Error")); ;
        }

        /// <summary>
        /// Modifica una secció del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id_seccio">Identificador de la secció</param>
        /// <param name="descripcio">Descripció de la secció</param>
        /// <param name="usuari_mod_seccio">Usuari que modifica la secció</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateSeccio(String id_seccio, string descripcio, String usuari_mod_seccio)
        {
            Seccio seccio = Seccio.GetInstancia(id_seccio);
            if (seccio == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la secció"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(seccio.usuari_mod_seccio).nom_rol).seccio[1].Equals('1'))
            {
                seccio.descripcio_seccio = descripcio;
                seccio.usuari_mod_seccio = usuari_mod_seccio;
                seccio.data_mod_seccio = DateTime.Now;
                //seccio.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar la secció"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Crea una nova secció i ho registra a la persistència
        /// </summary>
        /// <param name="id_seccio">Identificador que ha de ser únic de la secció</param>
        /// <param name="descripcio">Descripcio de la nova secció</param>
        /// <param name="usuari_crea_seccio">Usuari que crea la secció</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Seccio CreateSeccio(String id_seccio, string descripcio, String usuari_crea_seccio)
        {
            Seccio seccio;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_seccio).nom_rol).seccio[0].Equals('1'))
            {
                seccio = Seccio.CreateSeccio(id_seccio, usuari_crea_seccio);
                //seccio.Refrescar();
                seccio.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear la secció"
            }, new FaultReason("Error"));
            return seccio;
        }
	
     
        /// <summary>
        /// Obté un llistat de tots els usuaris actius del sistema
        /// </summary>
        /// <param name="usuari_consulta">Usuari que consulta el llistat</param>
        /// <returns>Llistat amb tots els usuaris</returns>
        [SecurityOperationBehavior]
        public List<Usuari> GetUsuaris(string usuari_consulta) { return Usuari.GetInstanciesActives(); }

        /// <summary>
        /// Obté un usuari a partir del seu id
        /// </summary>
        /// <param name="id">Id del usuari buscat</param>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Usuari buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        [SecurityOperationBehavior]
        public Usuari GetUsuari(String id, String usuari_consulta)
        {
            Usuari result = Usuari.GetInstancia(id);
            if (result != null)
            {
                //ContextDAO.Registrar();
                return result;
            }
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Funció que a partir d'un nom d'usuari retorna el seu id a la persistència
        /// </summary>
        /// <param name="nom_usuari">Nom de l'usuari que es necessita el identificador</param>
        /// <returns></returns>
        public string GetIdUsuari(String nom_usuari)
        {
            return Usuari.GetInstanciaByNom(nom_usuari).id_usuari;
        }

        /// <summary>
        /// Obté un usuari a partir del seu nom d'usuari
        /// </summary>
        /// <param name="nom">Nom del usuari buscat</param>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Usuari buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        [SecurityOperationBehavior]
        public Usuari GetUsuariByNom(String nom, String usuari_consulta)
        {
            Usuari result = Usuari.GetInstanciaByNom(nom);
            if (result != null)
            {
                //ContextDAO.Registrar();
                return result;
            }
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Obté un llistat de tots els usuaris actius d'un rol
        /// </summary>
        /// <param name="id_rol">Identificador del rol dels usuaris</param>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Llistat de tots els usuaris del rol</returns>
        [SecurityOperationBehavior]
        public List<Usuari> GetUsuaris(string id_rol, String usuari_consulta) { return Usuari.GetInstanciesActives(id_rol); }

        /// <summary>
        /// Obté un llistat dels noms de tots els usuaris actius
        /// </summary>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Llistat de tots els noms dels usuaris</returns>
        [SecurityOperationBehavior]
        public List<string> GetNomUsuaris(String usuari_consulta) { return Usuari.GetNomInstanciesActives(); }

        /// <summary>
        /// Obté un llistat dels noms de tots els usuaris actius d'un rol
        /// </summary>
        /// <param name="rol">Identificaddor del rol dels usuaris</param>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Llistat del nom de tots els usuaris del rol</returns>
        [SecurityOperationBehavior]
        public List<string> GetNomUsuaris(string id_rol, string usuari_consulta) { return Usuari.GetNomInstanciesActives(id_rol); }

        /// <summary>
        /// Esborra un usuari del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id">Identificador de l'usuari</param>
        /// <param name="usuari_esborra">Usuari que esborra</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteUsuari(String id, String usuari_esborra)
        {
            Usuari usuari = Usuari.GetInstancia(id);
            if (usuari == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).usuari[2].Equals('1'))
            {
            //usuari.Borrar();
            usuari.actiu_usuari = false;
            //usuari.Refrescar();
            ContextDAO.Registrar();
                 }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar l'usuari"
            }, new FaultReason("Error"));
        
        }

        /// <summary>
        /// Modifica un usuari del sistema i ho registra a la persistència
        /// </summary>
        /// <param name="id">Identificador de l'usuari a modificar</param>
        /// <param name="nom">Nou nom de l'usuari</param>
        /// <param name="nif">Nou NIF de l'usuari</param>
        /// <param name="nomFiscal">Nou nom fiscal de l'usuari</param>
        /// <param name="tel1">Nou telèfon 1 de l'usuari</param>
        /// <param name="tel2">Nou telèfon 2 de l'usuari</param>
        /// <param name="movil">Nou movil de l'usuari</param>
        /// <param name="direccio">Nova direcció de l'usuari</param>
        /// <param name="poblacio">Nova població de l'usuari</param>
        /// <param name="codpos">Nou codi postal de l'usuari</param>
        /// <param name="provincia">Nova província de l'usuari</param>
        /// <param name="estat">Nou estat de l'usuari</param>
        /// <param name="pwd">Nou codi d'accés de l'usuari en format MD5</param>
        /// <param name="observacions">Noves observacions de l'usuari</param>
        /// <param name="nom_rol">Nom del nou rol de l'usuari</param>
        /// <param name="id_usuari_modifica">Identificador de l'usuari que modificad l'usuari</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateUsuari(String id, String nom, String nif, String nomFiscal, String tel1, String tel2, String movil, String direccio, String poblacio, int codpos, String provincia, String estat, String pwd, String observacions, string nom_rol, string id_usuari_modifica)
        {
            Usuari usuari_mod = Usuari.GetInstancia(id_usuari_modifica);
            if (usuari_mod == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            }
            bool teDret = Rol.GetInstancia(usuari_mod.nom_rol).usuari[1].Equals('1');
            
            Usuari usuari = Usuari.GetInstancia(id);
            if (usuari == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            } 
            if (teDret)
            {

                usuari.nom_usuari = nom;
                usuari.NIF_usuari = nif;
                usuari.telefon1 = tel1;
                usuari.telefon2 = tel2;
                usuari.movil = movil;
                usuari.direccio = direccio;
                usuari.poblacio = poblacio;
                usuari.codpos = codpos;
                usuari.provincia = provincia;
                usuari.estat = estat;
                usuari.pwd = pwd;
                usuari.nom_rol = nom_rol;
                usuari.observacions = observacions;
                usuari.usuari_mod_usuari = id_usuari_modifica;
                usuari.data_mod_usuari = System.DateTime.Now;
                //usuari.Refrescar();
                
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar l'usuari"
            }, new FaultReason("Error")); ;
          }

        /// <summary>
        /// Crea un nou usuari i ho registra a la persistència
        /// </summary>
        /// <param name="nom">Identificador únic de l'usuari</param>
        /// <param name="nif">NIF de l'usuari</param>
        /// <param name="nomFiscal">Nom fiscal de l'usuari</param>
        /// <param name="tel1">Telèfon 1 de l'usuari</param>
        /// <param name="tel2">Telèfon 2 de l'usuari</param>
        /// <param name="movil">Movil de l'usuari</param>
        /// <param name="direccio">Direcció de l'usuari</param>
        /// <param name="poblacio">Població de l'usuari</param>
        /// <param name="codpos">Codi postal de l'usuari</param>
        /// <param name="provincia">Provincia de l'usuari</param>
        /// <param name="estat">Estat de l'usuari</param>
        /// <param name="pwd">Codi d'accés de l'usuari en format MD5</param>
        /// <param name="observacions">Observacions sobre l'usuari</param>
        /// <param name="nom_rol">Nom del rol de l'usuari</param>
        /// <param name="id_usuari_crea">Identificador de l'usuari que crea l'usuari</param>
        /// <returns>Usuari creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Usuari CreateUsuari(String nom, String nif, String nomFiscal, String tel1, String tel2, String movil, String direccio, String poblacio, int codpos, String provincia, String estat, String pwd, String observacions, string nom_rol, string id_usuari_crea)
        {
            Usuari usuariCrea = Usuari.GetInstancia(id_usuari_crea);
            if (usuariCrea == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'usuari"
                }, new FaultReason("Error"));
            } 
            Usuari usuari;
            if (Rol.GetInstancia(usuariCrea.nom_rol).usuari[0].Equals('1'))
            {
                
                usuari= Usuari.CreateUsuari(nom, nif, nomFiscal, tel1, tel2, movil, direccio, poblacio, codpos, provincia, estat, pwd, observacions, nom_rol, id_usuari_crea);
                //usuari.Refrescar();
                usuari.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear l'usuari"
            }, new FaultReason("Error"));
            return usuari;                 
        }

        /// <summary>
        /// Obté un llistat de tots els rols actius del sistema
        /// </summary>
        /// <returns>Llistat de tots els rols del sistema</returns>
        public List<Rol> GetRols()
        {
            return Rol.GetInstanciesActives();
        }

        /// <summary>
        /// Obté un llistat del nom de tots els rols actius del sistema
        /// </summary>
        /// <returns>Llistat de noms de tots els rols del sistema</returns>
        public List<string> GetNomRols()
        {
            return Rol.GetNomInstanciesActives();
        }

        /// <summary>
        /// Obté un rol a partir del seu nom
        /// </summary>
        /// <param name="nom">Nom del rol buscat</param>
        /// <returns>Rol buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Rol GetRol(String nom)
        {
            Rol result = Rol.GetInstancia(nom);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del rol"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra un rol del sistema i tots els usuaris amb aquest rol, i ho registra a la persistència
        /// </summary>
        /// <param name="nom">Nom del rol a esborrar</param>
        /// <param name="usuari_esborra">Usuari que esborra el rol</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteRol(String nom, String usuari_esborra)
        {
            Rol rol = Rol.GetInstancia(nom);
            if (rol == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del rol"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).rol1[2].Equals('1'))
            {
                //rol.Borrar();
                rol.actiu_rol = false;
                //rol.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el rol"
            }, new FaultReason("Error")); ;
        }

                /// <summary>
        /// Modifica un rol del sistem i ho registra a la persistència
        /// </summary>
        /// <param name="nom">Nom del rol a modificar</param>
        /// <param name="acces_escriptori">Permet als usuaris accedir a la aplicació d'escriptori (mode administració)</param>
        /// <param name="modifica_empresa">Permet als usuaris modificar les dades generals de l'empresa</param>
        /// <param name="dretRol">Defineix els drets sobre crear, modificar i esborrar rols</param>
        /// <param name="dretUsuari">Defineix els drets sobre crear, modificar i esborrar usuaris</param>
        /// <param name="dretReservaForana">Defineix els drets sobre crear, modificar i esborrar reserves no propies</param>
        /// <param name="dretReserva">Defineix els drets sobre crear, modificar i esborrar reserves propies</param>
        /// <param name="dretRecurs">Defineix els drets sobre crear, modificar i esborrar recursos</param>
        /// <param name="dretSeccio">Defineix els drets sobre crear, modificar i esborrar seccions</param>
        /// <param name="dretOferta">Defineix els drets sobre crear, modificar i esborrar ofertes</param>
        /// <param name="dretFactura">Defineix els drets sobre crear, modificar i esborrar factures propies</param>
        /// <param name="dretFacturaForana">Defineix els drets sobre crear, modificar i esborrar factures no propies</param>
        /// <param name="dretCobrament">Defineix els drets sobre crear, modificar i esborrar tipus de cobrament</param>
        /// <param name="dretIVA">Defineix els drets sobre crear, modificar i esborrar tipus d'IVA</param>
        /// <param name="dretPagina">Defineix els drets sobre crear, modificar i esborrar pàgines</param>
        /// <param name="dretContinent">Defineix els drets sobre crear, modificar i esborrar continents</param>
        /// <param name="dretContingut">Defineix els drets sobre crear, modificar i esborrar continguts</param>
        /// <param name="usuari_modifica">Usuari que modifica el rol</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateRol(String nom, bool acces_escriptori, bool modifica_empresa, string dretRol, string dretUsuari, string dretReservaForana, string dretReserva, string dretRecurs, string dretSeccio, string dretOferta, string dretFactura, string dretFacturaForana, string dretCobrament, string dretIVA, string dretPagina, string dretContinent, string dretContingut, string usuari_modifica)
        {
            Rol rol = Rol.GetInstancia(nom);
            if (rol == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del rol"
                }, new FaultReason("Error"));
            }
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_modifica).nom_rol).rol1[1].Equals('1'))
            {
                rol.modifica_empresa = modifica_empresa;
                rol.usuari = dretUsuari;
                rol.reserva_forana = dretReservaForana;
                rol.reserva = dretReserva;
                rol.recurs = dretRecurs;
                rol.seccio = dretSeccio;
                rol.oferta = dretOferta;
                rol.factura = dretFactura;
                rol.factura_forana = dretFacturaForana;
                rol.cobrament = dretCobrament;
                rol.tipus_IVA = dretIVA;
                rol.pagina = dretPagina;
                rol.continent = dretContinent;
                rol.contingut = dretContingut;
                rol.acces_escriptori = acces_escriptori;
                rol.rol1 = dretRol;
                rol.usuari_mod_rol = usuari_modifica;
                rol.data_mod_rol = DateTime.Now;
                //rol.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el rol"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Crea un nou rol i el escriu a la persistencia
        /// </summary>
        /// <param name="nom">Nom del rol</param>
        /// <param name="acces_escriptori">Marca si els usuaris poden accedir a l'aplicació</param>
        /// <param name="modifica_empresa">Marca si els usuaris poden modificar les dades de l'empresa</param>
        /// <param name="dretRol">Drets sobre els rols</param>
        /// <param name="dretUsuari">Drets sobre els usuaris</param>
        /// <param name="dretReservaForana">Drets sobre reserves que no son del usuari identificat</param>
        /// <param name="dretReserva">Dret sobre les reserves de l'usuari identificat</param>
        /// <param name="dretRecurs">Drets sobre els recursos</param>
        /// <param name="dretSeccio">Drets sobre les seccions</param>
        /// <param name="dretOferta">Drets sobre les ofertes</param>
        /// <param name="dretFactura">Drets sobre les factures de l'usuari identificat</param>
        /// <param name="dretFacturaForana">Drets sobre les factures que no son de l'usuari identificat</param>
        /// <param name="dretCobrament">Drets sobre els cobraments</param>
        /// <param name="dretIVA">Drets sobre els tipus IVA</param>
        /// <param name="dretPagina">Drets sobre les pàgines</param>
        /// <param name="dretContinent">Drets sobre els continents</param>
        /// <param name="dretContingut">Drets sobre els continguts</param>
        /// <param name="usuari_crea">Usuari que crea el rol</param>
        /// <returns>Rol creat</returns>
        [SecurityOperationBehavior]
        public Rol CreateRol(
                string nom, 
                bool acces_escriptori, 
                bool modifica_empresa, 
                string dretRol, 
                string dretUsuari, 
                string dretReservaForana, 
                string dretReserva, 
                string dretRecurs, 
                string dretSeccio, 
                string dretOferta, 
                string dretFactura, 
                string dretFacturaForana, 
                string dretCobrament, 
                string dretIVA, 
                string dretPagina, 
                string dretContinent, 
                string dretContingut, 
                string usuari_crea)
             {
            Rol rol;
            if (Rol.GetInstancia(usuari_crea).rol1[0].Equals('1'))
            {
                rol=Rol.CreateRol(nom, acces_escriptori,modifica_empresa,dretRol, dretUsuari, dretReservaForana, dretReserva, dretRecurs
                    , dretSeccio, dretOferta, dretFactura, dretFacturaForana, dretCobrament, dretIVA, dretPagina, dretContinent, dretContingut, usuari_crea);
                //rol.Refrescar();
                rol.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el rol"
            }, new FaultReason("Error"));
            return rol;
        }
    
        /// <summary>
        /// Obté un llistat de tots els tipus de continents del sistema
        /// </summary>
        /// <returns></returns>
        public List<Continent> GetContinents() { return Continent.GetInstancies(); }

        /// <summary>
        /// Obté un continent a partir del seu id
        /// </summary>
        /// <param name="id">Id del continent buscat</param>
        /// <returns>Continent buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Continent GetContinent(String id)
            {
                Continent result = Continent.GetInstancia(id);
                if (result != null)
                    return result;
                else
                {
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "No es troba identificador del continent"
                    }, new FaultReason("Error"));
                }
         }
       
        /// <summary>
        /// Esborra un contient
        /// </summary>
        /// <param name="id">Identificador del continent</param>
        /// <param name="usuari_esborra">Usuari que esborra</param>
        [SecurityOperationBehavior]
        public void DeleteContinent(String id, String usuari_esborra)
        {
            Continent continent = Continent.GetInstancia(id);
            if (continent == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del continent"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(usuari_esborra).continent[2].Equals('1'))
            {
                continent.Borrar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el continent"
            }, new FaultReason("Error")); ;
        }

        /// <summary>
        /// Modifica un continent
        /// </summary>
        /// <param name="id_continent">Identificador del continent a modificar</param>
        /// <param name="amplada">Amplada en píxels del continent</param>
        /// <param name="alcada">Alçada en píxels del continent</param>
        /// <param name="xaml">Codi XAML (UserControl) del continent</param>
        /// <param name="usuari_mod_continent">Usuari que modifica el continent</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateContinent(String id_continent, int amplada, int alcada, String xaml, String usuari_mod_continent)
        {
            Continent continent = Continent.GetInstancia(id_continent);

            if (continent == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del continent"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_continent).nom_rol).continent[1].Equals('1'))
            {
            continent.amplada = amplada;
            continent.alcada = alcada;
            continent.xaml = xaml;
            continent.usuari_mod_continent = usuari_mod_continent;
            continent.data_mod_continent = DateTime.Now;
            //continent.Refrescar();
            ContextDAO.Registrar();
         }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el continent"
            }, new FaultReason("Error"));;}

        /// <summary>
        /// Crea un nou continent a partir de tots els paràmetres
        /// </summary>
        /// <param name="id_continent">Identificador del continent</param>
        /// <param name="amplada">Amplada en píxels del continent</param>
        /// <param name="alcada">Alçada en píxels del continent</param>
        /// <param name="xaml">Codi xaml (UserControl) del continent</param>
        /// <param name="observacions">Observacions sobre el continent</param>
        /// <param name="usuari_crea_continent">Usuari que crea el continent</param>
        /// <returns>Continent creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Continent CreateContinent(String id_continent, int amplada, int alcada, String xaml, String observacions, String usuari_crea_continent)
        {
            Continent continent;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_continent).nom_rol).continent[0].Equals('1'))
            {
                continent=Continent.CreateContinent(id_continent, amplada, alcada, xaml, observacions, usuari_crea_continent);
                //continent.Refrescar();
                continent.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el continent"
            }, new FaultReason("Error"));
            return continent;
        }



        /// <summary>
        /// Obté un llistat de continguts del sistema
        /// </summary>
        /// <returns>Llistat de Continguts del sistema</returns>
        public List<Contingut> GetContinguts() { return Contingut.GetInstancies(); }


        /// <summary>
        /// Obté un llistat de continguts del sistema
        /// </summary>
        /// <param name="pagina">Identificador de la pàgina a retornar els continguts</param>
        /// <returns>Llistat de Continguts del sistema</returns>
        public List<Contingut> GetContingutsByPagina(int pagina) { return Contingut.GetInstancies(Pagina.GetInstancia(pagina)); }

        /// <summary>
        /// Obté un contingut a partir de la seva posició i la pàgina
        /// </summary>
        /// <param name="posicio">Posicio del contingut a la pàgina</param>
        /// <param name="posicio_pagina">Posicio de la pàgina que conté el contingut</param>
        /// <returns>Contingut buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Contingut GetContingut(int posicio, int posicio_pagina)
        {
            Contingut result = Contingut.GetInstancia(posicio, posicio_pagina);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del contingut"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra un contingut del sistema i ho registra a la permanència
        /// </summary>
        /// <param name="posicio">Posicio del contingut dins la pàgina</param>
        /// <param name="pagina">Posicio de la pàgina que conté el contingut</param>
        /// <param name="usuari_esborra">Usuari que l'esborra</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteContingut(int posicio, int pagina, String usuari_esborra)
        {
            Contingut contingut = Contingut.GetInstancia(posicio, pagina);
            if (contingut == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del contingut"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).contingut[2].Equals('1'))
            {
            contingut.Borrar();
            //ContextDAO.Registrar();
         }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el contingut"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Modifica un contingut i ho registra a la permanència
        /// </summary>
        /// <param name="posicio_contingut">Posicio del contingut a modificar</param>
        /// <param name="posicio_pagina">Posicio de la pàgina que conté el contingut</param>
        /// <param name="id_continent">Nou identificador del tipus de continent</param>
        /// <param name="xaml">Nou codi xaml</param>
        /// <param name="observacions">Observacions sobre el contingut</param>
        /// <param name="usuari_mod_contingut">Usuari que modifica el contingut</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateContingut(int posicio_contingut, int posicio_pagina, String id_continent, String xaml, String observacions, String usuari_mod_contingut)
        {
            Contingut contingut = Contingut.GetInstancia(posicio_contingut, posicio_pagina);

            if (contingut == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del contingut"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_contingut).nom_rol).contingut[1].Equals('1'))
            {
            contingut.id_continent = id_continent;
            contingut.xaml = xaml;
            contingut.usuari_mod_contingut = usuari_mod_contingut;
            contingut.observacions = observacions;
            contingut.data_mod_contingut = DateTime.Now;
            //contingut.Refrescar();
            ContextDAO.Registrar();
         }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el contingut"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Crea un nou contingut i el situa a la pàgina en darrera posició
        /// </summary>
        /// <param name="pos_pagina">Posició de la pàgina que conté el contingut</param>
        /// <param name="id_continent">Identificador del tipus de continent</param>
        /// <param name="xaml">Codi xaml</param>
        /// <param name="observacions">Observacions sobre el contingut</param>
        /// <param name="usuari_crea_contingut">Usuari que crea el contingut</param>
        /// <returns>Contingut creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Contingut CreateContingut(int pos_pagina, String id_continent, String xaml, String observacions, String usuari_crea_contingut)
        {
            Contingut contingut;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_contingut).nom_rol).contingut[0].Equals('1'))
            {
                contingut=Contingut.CreateContingut(pos_pagina, id_continent, xaml, observacions, usuari_crea_contingut);
                //contingut.Refrescar();
                contingut.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el contingut"
            }, new FaultReason("Error"));
            return contingut;
        }

        /// <summary>
        /// Obté un llistat de les pàgines del sistema
        /// </summary>
        /// <returns>Llistat de les pàgines del sistema</returns>
        public List<Pagina> GetPagines() { 
            List<Pagina> pagines = Pagina.GetInstancies();
            if (pagines == null)            
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "Error en la lectura de les pàgines"
                }, new FaultReason("Error"));
            }
            return pagines;
        }

        /// <summary>
        /// Obté una pàgina a partir de la seva posició
        /// </summary>
        /// <param name="posicio">Posicio de la pàgina</param>
        /// <returns>Pàgina buscada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Pagina GetPagina(int posicio)
        {
            Pagina result = Pagina.GetInstancia(posicio);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la pàgina"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra una pàgina del sistema i ho registra a la permanència
        /// </summary>
        /// <param name="posicio">Posicio de la pàgina buscada</param>
        /// <param name="usuari_esborra">Usuari que esborra la pàgina</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeletePagina(int posicio, String usuari_esborra)
        {
            Pagina pagina = Pagina.GetInstancia(posicio);
            if (pagina == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la pàgina"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).pagina[2].Equals('1'))
            {
            pagina.Borrar();
            //ContextDAO.Registrar();
         }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar la pàgina"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Modifica una pàgina al sistema i ho registra a la permanència
        /// </summary>
        /// <param name="pos_pagina">Posició de la pàgina al sistema</param>
        /// <param name="nom_pagina">Nou nom de la pàgina</param>
        /// <param name="text_menu">Nou text al menú de la pàgina</param>
        /// <param name="observacions">Observacions sobre la pàgina</param>
        /// <param name="usuari_mod_pagina">Usuari que modifica la pàgina</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdatePagina(int pos_pagina, String nom_pagina, String text_menu, String observacions, String usuari_mod_pagina)
        {
            Pagina pagina = Pagina.GetInstancia(pos_pagina);
            if (pagina == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'objecte"
                }, new FaultReason("Error"));
            } 
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_pagina).nom_rol).pagina[1].Equals('1'))
            {
            pagina.nom_pagina = nom_pagina;
            pagina.text_menu = text_menu;
            pagina.usuari_mod_pagina = usuari_mod_pagina;
            pagina.data_mod_pagina = DateTime.Now;
            pagina.observacions = observacions;
            //pagina.Refrescar();
            ContextDAO.Registrar();
         }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar la pàgina"
            }, new FaultReason("Error")); ;
        }

        /// <summary>
        /// Crea una nova pàgina i la situa la darrera i ho registra a la persistència
        /// </summary>
        /// <param name="titol_pagina">Titol de la pàgina</param>
        /// <param name="text_menu">Text al menú de la pàgina</param>
        /// <param name="observacions">Observacions sobre la pàgina</param>
        /// <param name="usuari_crea_pagina">Usuari que crea la pàgina</param>
        /// <returns>Pagina creada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Pagina CreatePagina(String titol_pagina, String text_menu, String observacions, String usuari_crea_pagina)
        {
            Pagina pagina;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_pagina).nom_rol).pagina[0].Equals('1'))
            {
                pagina=Pagina.CreatePagina(titol_pagina, text_menu, observacions, usuari_crea_pagina);
                pagina.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear la pàgina"
            }, new FaultReason("Error"));
            return pagina;
        }

        //private ResourceManager resourceManager;

        /// <summary>
        /// Retorna el llistat de tots els tipus de cobrament del sistema
        /// </summary>
        /// <returns>Llistat amb els tipus de cobrament del sistema</returns>
        public List<Cobrament> GetCobraments() { return Cobrament.GetInstanciesActives(); }

        /// <summary>
        /// Retorna el llistat de noms de tots els tipus de cobrament del sistema
        /// </summary>
        /// <returns>Llistat amb els noms de tipus de cobrament del sistema</returns>
        public List<string> GetNomCobraments() { return Cobrament.GetIdInstanciesActives(); }

        /// <summary>
        /// Obté un tipus de Cobrament a partir del seu Id
        /// </summary>
        /// <param name="id">Identificador del tipus de Cobrament</param>
        /// <returns>Cobrament buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Cobrament GetCobrament(String id)
        {
            Cobrament cobrament = Cobrament.GetInstancia(id);
            if (cobrament != null)
                return cobrament;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del cobrament"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Esborra un tipus de Cobrament del sistema i totes les factures que estan en aquest estat i ho registra a la persistència
        /// </summary>
        /// <param name="id">Identificador del tipus de Cobrament</param>
        /// <param name="usuari_esborra">Usuari que esborra l'element</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteCobrament(String id, String usuari_esborra)
        {
            Cobrament cobrament = Cobrament.GetInstancia(id);
            if (cobrament == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del cobrament"
                }, new FaultReason("Error"));
            }
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).cobrament[2].Equals('1'))
            {
                cobrament.actiu_cobrament = false;
                //cobrament.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el cobrament"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Actualitza un tipus de Cobrament i ho modifica a la persistència
        /// </summary>
        /// <param name="estat_cobrament">Nom de l'estat a modificar </param>
        /// <param name="descripcio">Nova descripció de l'estat de cobrament</param>
        /// <param name="usuari_mod_cobrament">Usuari que modifica el tipus de cobrament</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateCobrament(String estat_cobrament, String descripcio, String usuari_mod_cobrament)
        {
            Cobrament cobrament = Cobrament.GetInstancia(estat_cobrament);
            if (cobrament == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del cobrament"
                }, new FaultReason("Error"));
            }
            if (Rol.GetInstancia(Usuari.GetInstancia(cobrament.usuari_mod_cobrament).nom_rol).cobrament[1].Equals('1'))
            {
                cobrament.estat_cobrament = estat_cobrament;
                cobrament.descripcio = descripcio;

                cobrament.usuari_mod_cobrament = usuari_mod_cobrament;
                cobrament.data_mod_cobrament = System.DateTime.Now;
                //cobrament.Refrescar();
                //ContextDades.ApplyCurrentValues<Cobrament>(estat_cobrament, cobrament);
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el cobrament"
            }, new FaultReason("Error")); ;
        }

        /// <summary>
        /// Crea un tipus de Cobrament i ho modifica a la persistència
        /// </summary>
        /// <param name="estat_cobrament">Nom de l'estat a crear </param>
        /// <param name="descripcio">Nova descripció de l'estat de cobrament</param>
        /// <param name="usuari_mod_cobrament">Usuari que crea el tipus de cobrament</param>
        /// <returns>Cobrament retornat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Cobrament CreateCobrament(String id, String estat_cobrament, String usuari_crea_cobrament)
        {
            Cobrament cobrament;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_cobrament).nom_rol).cobrament[0].Equals('1'))
            {
                cobrament = Cobrament.CreateCobrament(id, true, usuari_crea_cobrament, usuari_crea_cobrament, System.DateTime.Now, System.DateTime.Now);
                cobrament.Insertar();
                //ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el cobrament"
            }, new FaultReason("Error"));
            return cobrament;
        }

        /// <summary>
        /// Obté l'objecte de la empresa
        /// </summary>
        /// <returns>Empresa del sistema</returns>
        public Empresa GetEmpresa() { return Empresa.GetInstancia(); }

        /// <summary>
        /// Modifica les dades de la empresa i ho registra a la persistència
        /// </summary>
        /// <param name="nIF">Nou NIF de la empresa</param>
        /// <param name="nom_fiscal">Nou nom fiscal de la empresa</param>
        /// <param name="nom_comercial">Nou nom comercial de la empresa</param>
        /// <param name="telefon1">Nou telèfon 1 de la empresa</param>
        /// <param name="telefon2">Nou telèfon 2</param>
        /// <param name="telefon3">Nou telèfon 3</param>
        /// <param name="fax">Nou fax de l'empresa</param>
        /// <param name="correu_electronic">Nou correu electrònic de la empresa</param>
        /// <param name="direccio_web">Nova direcció web de la empresa</param>
        /// <param name="direccio_empresa">Nova direcció postal de la empresa</param>
        /// <param name="codpos_empresa">Nou codi postal de la empresa</param>
        /// <param name="poblacio">Nova població de la empresa</param>
        /// <param name="provincia">Nova província de la empresa</param>
        /// <param name="nom_gerent">Nom del gerent de l'empresa</param>
        /// <param name="comentaris">Comentaris sobre l'empresa</param>
        /// <param name="altra_info">Altra informació sobre l'empresa</param>
        /// <param name="dades_mercantils">Dades mercantils de l'empresa</param>
        /// <param name="estat">Estat (pais) de l'empresa</param>
        /// <param name="usuari_mod_empresa">Usuari que modifica les dades</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateEmpresa(String nIF, String nom_fiscal, String nom_comercial, String telefon1, string telefon2, string telefon3, string fax, String correu_electronic, String direccio_web, String direccio_empresa, Int32 codpos_empresa, String poblacio, String provincia, string nom_gerent, string comentaris, string altra_info, string dades_mercantils, String estat, string usuari_mod_empresa)
        {
            Empresa empresa = Empresa.GetInstancia();
            if (Rol.GetInstancia(Usuari.GetInstancia(empresa.usuari_mod_empresa).nom_rol).modifica_empresa)
            {
                empresa.NIF = nIF;
                empresa.nom_fiscal = nom_fiscal;
                empresa.nom_comercial = nom_comercial;
                empresa.telefon1 = telefon1;
                empresa.telefon2 = telefon2;
                empresa.telefon3 = telefon3;
                empresa.fax = fax;
                empresa.correu_electronic = correu_electronic;
                empresa.direccio_web = direccio_web;
                empresa.direccio_empresa = direccio_empresa;
                empresa.codpos_empresa = codpos_empresa;
                empresa.poblacio = poblacio;
                empresa.provincia = provincia;
                empresa.estat = estat;
                empresa.altra_info = altra_info;
                empresa.comentaris = comentaris;
                empresa.nom_gerent_empresa = nom_gerent;
                empresa.dades_mercantils = dades_mercantils;
                empresa.usuari_mod_empresa = usuari_mod_empresa;
                empresa.data_mod_empresa = DateTime.Now;
                //empresa.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar la empresa"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Obté un llistat de les factures del sistema, en qualsevol estat i de qualsevol usuari
        /// </summary>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Llista amb totes les factures del sistema</returns>
        [SecurityOperationBehavior]
        public List<Factura> GetFactures(string usuari_consulta) { return Factura.GetInstanciesActives(); }

        /// <summary>
        /// Obté un llistat dels noms de les factures del sistema, en qualsevol estat i de qualsevol usuari
        /// </summary>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns>Llista amb tots els noms de les factures del sistema</returns>
        [SecurityOperationBehavior]
        public List<string> GetNomFactures(string usuari_consulta) { return Factura.GetIdInstanciesActives(); }

        /// <summary>
        /// Retorna una factura a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de la factura</param>
        /// <returns>Factura buscada</returns>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        [SecurityOperationBehavior]
        public Factura GetFactura(String id, string usuari_consulta)
        {
            Factura result = Factura.GetInstancia(id);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Obté un llistat de les factures d'un usuari, en qualsevol estat
        /// </summary>
        /// <param name="id_usuari>Identificiador de l'usuari</param>
        /// <param name="usuari_consulta">Usuari que consulta el llistat</param>
        /// <returns>Llista amb totes les factures del usuari</returns>
        [SecurityOperationBehavior]
        public List<Factura> GetFactures(string id_usuari, string usuari_consulta) { return Factura.GetInstanciesActivesByUsuari(id_usuari); }

        /// <summary>
        /// Obté un llistat de les factures d'un usuari, en l'estat definit
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari de les factures</param>
        /// <param name="id_cobrament">Identificador del Tipus de cobrament de les factures</param>
        /// <param name="usuari_consulta">Usuari que consulta el llistat</param>
        /// <returns>Llista amb totes les factures del usuari en l'estat dessitjat</returns>
        [SecurityOperationBehavior]
        public List<Factura> GetFactures(string id_usuari, string id_cobrament, string usuari_consulta) { return Factura.GetInstanciesActivesByUsuariAndCobrament(id_usuari, id_cobrament); }

        /// <summary>
        /// Obté un llistat de les factures en l'estat definit
        /// </summary>
        /// <param name="id_cobrament">Identificador del Tipus de cobrament de les factures</param>
        /// <param name="usuari_consulta">Usuari que consulta el llistat</param>
        /// <returns>Llista amb totes les factures del usuari en l'estat dessitjat</returns>
        [SecurityOperationBehavior]
        public List<Factura> GetFacturesByCobrament(string id_cobrament, string usuari_consulta) { return Factura.GetInstanciesActivesByCobrament(id_cobrament); }

        /// <summary>
        /// Esborra una factura del sistema  i ho registra a la persistència
        /// </summary>
        /// <param name="id"></param>
        /// <param name="usuari_esborra"></param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteFactura(String id, String usuari_esborra)
        {
            Factura factura = Factura.GetInstancia(id);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_esborra.Equals(factura.usuari_factura))
                //comprovo si el usuari té autorització per esborrar una factura (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per esborrar la factura"
                    }, new FaultReason("Error"));
                else
                    //comprovo si el usuari té autorització per esborrar una factura (forana)
                    if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura_forana[2].Equals('1'))
                    {
                        throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                        {
                            Detail = "L'usuari no té els drets suficients per esborrar la factura"
                        }, new FaultReason("Error"));
                    }
            //factura.Borrar();
            factura.actiu_factura = false;
            //factura.Refrescar();
            ContextDAO.Registrar();
        }

        /// <summary>
        /// Actualitza les dades d'una factura i les modifica a la persistència
        /// </summary>
        /// <param name="id_factura">Identificador de la factura a modificar</param>
        /// <param name="modificable">Booleà que representa si la factura es modificable (tancada) o no</param>
        /// <param name="estat_cobrament">Nou estat de cobrament de la factura</param>
        /// <param name="data_cobrament">Data en que s'ha realitzat el cobrament o NULL</param>
        /// <param name="a_compte">Import ja pagat de la factura</param>
        /// <param name="usuari_mod_factura">Usuari que realitza la modificació</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateFactura(String id_factura, bool modificable, String estat_cobrament, DateTime data_cobrament, Decimal a_compte, String descripcio, String usuari_mod_factura)
        {
            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }
            if (usuari_mod_factura.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una factura (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura[1].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modificar la factura"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per esborrar una factura (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura_forana[1].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modificar la factura"
                    }, new FaultReason("Error"));

            factura.estat_cobrament = estat_cobrament;
            factura.modificable = modificable;
            factura.usuari_mod_factura = usuari_mod_factura;
            factura.data_mod_factura = DateTime.Now;
            factura.a_compte = a_compte;
            factura.descripcio = descripcio;
            factura.data_cobrament = data_cobrament;
            //factura.Refrescar();
            ContextDAO.Registrar();
        }

        /// <summary>
        /// Crea una nova factura al sistema  i ho registra a la persistència
        /// </summary>
        /// <param name="client">Identificador del client de la factura</param>
        /// <param name="estat_cobrament">Estat de cobrament de la factura</param>
        /// <param name="usuari_crea_factura">Usuari que crea la factura</param>
        /// <returns>Factura creada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Factura CreateFactura(String client, String estat_cobrament, DateTime data_cobrament, String descripcio, Decimal a_compte, bool modificable, String usuari_crea_factura)
        {
            Factura factura;
            if (usuari_crea_factura.Equals(client))
            {
                //comprovo si el usuari té autorització per crear una factura (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura[0].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear la factura propia"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per crear una factura (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura_forana[0].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear la factura externa"
                    }, new FaultReason("Error"));
            factura = Factura.CreateFactura(client, estat_cobrament, data_cobrament, descripcio, a_compte, modificable, usuari_crea_factura);
            factura.Insertar();
            return factura;
        }

        /// <summary>
        /// Retorna un llistat de totes les ofertes facturades al sistema, d'ofertes no esborrades
        /// </summary>
        /// <returns>Llistat amb totes les ofertes facturades</returns>
        public List<Oferta_facturada> GetOfertesFacturades() { return Oferta_facturada.GetInstanciesActives(); }

        /// <summary>
        /// Retorna un llistat de totes les ofertes facturades d'una factura
        /// </summary>
        /// <param name="id_factura">Identificador de factura</param>
        /// <returns>Llistat amb totes les ofertes facturades</returns>
        public List<Oferta_facturada> GetOfertesFacturadesByFactura(string id_factura) { return Oferta_facturada.GetInstanciesByFactura(id_factura); }

        /// <summary>
        /// Retorna un llistat de totes les ofertes facturades d'una oferta, d'ofertes no esborrades
        /// </summary>
        /// <param name="id_oferta">Identificador de l'oferta</param>        /// <returns>Llistat amb totes les ofertes facturades</returns>
        public List<Oferta_facturada> GetOfertesFacturadesByOferta(string id_oferta) { return Oferta_facturada.GetInstanciesByOferta(id_oferta); }

        /// <summary>
        /// Retorna un llistat de noms de totes les ofertes facturades al sistema, d'ofertes no esborrades
        /// </summary>
        /// <returns>Llistat amb noms de totes les ofertes facturades</returns>
        public List<string> GetNomOfertesFacturades() { return Oferta_facturada.GetNomInstanciesActives(); }

        /// <summary>
        /// Retorna una oferta facturada a partir d'una factura i un recurs
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <returns>Oferta facturada buscada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Oferta_facturada GetOfertaFacturada(String id_factura, String id_oferta)
        {
            Oferta_facturada result = Oferta_facturada.GetInstancia(id_factura, id_oferta);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura o la oferta"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Elimina una oferta facturada i ho registra a la persistència
        /// </summary>
        /// <param name="id_factura">Identificador de la factura on eliminar la utilització</param>
        /// <param name="id_oferta">Identificador de la oferta que s'ha d'eliminar</param>
        /// <param name="usuari_esborra">Usuari que esborra l'element</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteOfertaFacturada(String id_factura, String id_oferta, String usuari_esborra)
        {
            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }
            Oferta_facturada oferta = Oferta_facturada.GetInstancia(id_factura, id_oferta);
            if (oferta == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de l'oferta"
                }, new FaultReason("Error"));
            }

            if (usuari_esborra.Equals(factura.usuari_factura))
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per esborrar la oferta facturada"
                    }, new FaultReason("Error"));
                else
                    //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                    if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura_forana[2].Equals('1'))
                        throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                        {
                            Detail = "L'usuari no té els drets suficients per esborrar la oferta facturada"
                        }, new FaultReason("Error"));

            oferta.Borrar();
            //oferta.Refrescar();
            //ContextDAO.Registrar();
        }

        /// <summary>
        /// Modifica una oferta facturada del sistema i actualitza el preu i ho registra a la persistència
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_oferta">Identificador de la oferta</param>
        /// <param name="quantitat">Nova quantitat a facturar</param>
        /// <param name="data_inici_servei">Nova data i hora d'inici de la utilització</param>
        /// <param name="data_fi_servei">Nova data i hora de fi de la utilització</param>
        /// <param name="usuari_mod_factura">Usuari que modifica l'element</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        [SecurityOperationBehavior]
        public void UpdateOfertaFacturada(String id_factura, String id_oferta, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_mod_factura)
        {
            Oferta_facturada oferta_facturada = Oferta_facturada.GetInstancia(id_factura, id_oferta);
            if (oferta_facturada == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta facturada"
                }, new FaultReason("Error")); 
            }

            Oferta oferta = Oferta.GetInstancia(id_oferta);
            if (oferta == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta"
                }, new FaultReason("Error"));
            }

            Recurs recurs = Recurs.GetInstancia(oferta.id_recurs);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            }

            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_mod_factura.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modifica la oferta facturada"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura_forana[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modificar la oferta facturada"
                    }, new FaultReason("Error"));

            oferta_facturada.id_factura = id_factura;
            oferta_facturada.id_oferta = id_oferta;
            oferta_facturada.import = oferta.import_unitari;
            oferta_facturada.quantitat = quantitat;
            oferta_facturada.percentatge_IVA = recurs.TipusIVA.percentatge;
            oferta_facturada.data_inici_servei = data_inici_servei;
            oferta_facturada.data_fi_servei = data_fi_servei;
            oferta_facturada.usuari_mod_factura = usuari_mod_factura;
            oferta_facturada.data_mod_factura = DateTime.Now;
            //oferta_facturada.Refrescar();
            ContextDAO.Registrar();

        }

        /// <summary>
        /// Crea una nova oferta facturada, i ho registra al sistema
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_oferta">Identificador de la oferta a afegir</param>
        /// <param name="quantitat">Quantitat de recursos</param>
        /// <param name="data_inici_servei">Data i hora d'inici de utilització de la oferta</param>
        /// <param name="data_fi_servei">Data i hora de fi de la utilització de la oferta</param>
        /// <param name="usuari_crea_factura">Usuari que crea la factura</param>
        /// <returns>Oferta facturada creada</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Oferta_facturada CreateOfertaFacturada(String id_factura, String id_oferta, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_crea_factura)
        {
            Oferta_facturada oferta_facturada;
            Oferta oferta = Oferta.GetInstancia(id_oferta);
            if (oferta == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la oferta"
                }, new FaultReason("Error"));
            }

            Recurs recurs = Recurs.GetInstancia(oferta.id_recurs);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            }

            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_crea_factura.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear la oferta facturada"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura_forana[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear la oferta facturada"
                    }, new FaultReason("Error"));


            oferta_facturada = Oferta_facturada.CreateOferta_facturada(id_factura, id_oferta, oferta.import_unitari, quantitat, recurs.TipusIVA.percentatge, data_inici_servei, data_fi_servei, usuari_crea_factura, usuari_crea_factura, DateTime.Now, DateTime.Now);
            //oferta_facturada.Refrescar();
            oferta_facturada.Insertar();
            //ContextDAO.Registrar();
            return oferta_facturada;
        }

        /// <summary>
        /// Retorna un llistat de tots els recursos facturats d'una factura
        /// </summary>
        /// <param name="id_factura">Identificador de la factura que conté els recursos facturats</param>
        /// <returns>Llistat complert de tots els recursos facturats en una factura</returns>
        public List<Recurs_facturat> GetRecursosFacturatsByFactura(string id_factura) { return Recurs_facturat.GetInstanciesByFactura(id_factura); }


        /// <summary>
        /// Retorna un llistat de tots els recursos facturats d'un recurs
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs que conté els recursos facturats</param>
        /// <returns>Llistat complert de tots els recursos facturats d'un recurs</returns>
        public List<Recurs_facturat> GetRecursosFacturatsByRecurs(string id_recurs) { return Recurs_facturat.GetInstanciesByRecurs(id_recurs); }


        /// <summary>
        /// Retorna un llistat de tots els recursos facturats del sistema de factures no esborrades
        /// </summary>
        /// <returns>Llistat complert de tots els recursos facturats al sistema de factures no esborrades</returns>
        public List<Recurs_facturat> GetRecursosFacturats() { return Recurs_facturat.GetInstanciesActives(); }

        /// <summary>
        /// Obté un recurs facturat 
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <returns>Recurs facturat buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public Recurs_facturat GetRecursFacturat(String id_factura, String id_recurs)
        {
            Recurs_facturat result = Recurs_facturat.GetInstancia(id_factura, id_recurs);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs facturat"
                }, new FaultReason("Error"));
            }
        }


        /// <summary>
        /// Elimina un recurs utilitzat en una factura
        /// </summary>
        /// <param name="id_factura">Identificador de la factura on eliminar la utilització</param>
        /// <param name="id_recurs">Identificador del recurs que s'ha d'eliminar</param>
        /// <param name="usuari_esborra">Usuari que esborra l'element</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteRecursFacturat(String id_factura, String id_recurs, String usuari_esborra)
        {
            Recurs_facturat recurs = Recurs_facturat.GetInstancia(id_factura, id_recurs);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            }

            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_esborra.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per esborrar el recurs facturat"
                    }, new FaultReason("Error"));
            }
             else
                    //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                    if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).factura_forana[2].Equals('1'))
                        throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                        {
                            Detail = "L'usuari no té els drets suficients per esborrar el recurs facturat"
                        }, new FaultReason("Error"));
            //Els recursos facturats no es desactivaran, si no que es borraran definitivament
            recurs.Borrar();
        }


        /// <summary>
        /// Modifica un recurs facturat, actualitza el preu, i ho registra al sistema
        /// </summary>
        /// <param name="id_factura">Identificador de la factura on modificar l'ús</param>
        /// <param name="id_recurs">Identificador del recurs que s'ha de facturar</param>
        /// <param name="quantitat">Quantitat de recurs a facturar</param>
        /// <param name="data_inici_servei">Data i hora d'inici de la utilització</param>
        /// <param name="data_fi_servei">Data i hora de fi de la utilització</param>
        /// <param name="usuari_crea_factura">Usuari que afegeix el recurs a la factura</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateRecursFacturat(String id_factura, String id_recurs, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_mod_factura)
        {
            Recurs_facturat recurs_facturat = Recurs_facturat.GetInstancia(id_factura, id_recurs);
            if (recurs_facturat == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs facturat"
                }, new FaultReason("Error"));
            }

            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_mod_factura.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modificar el recurs facturat"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_mod_factura).nom_rol).factura_forana[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per modificar el recurs facturat"
                    }, new FaultReason("Error"));

            recurs_facturat.import = Recurs.GetInstancia(id_recurs).preu;
            recurs_facturat.quantitat = quantitat;
            recurs_facturat.percentatge_IVA = Recurs.GetInstancia(id_recurs).TipusIVA.percentatge;
            recurs_facturat.data_inici_servei = data_inici_servei;
            recurs_facturat.data_fi_servei = data_fi_servei;
            recurs_facturat.usuari_mod_factura = usuari_mod_factura;
            recurs_facturat.data_mod_factura = DateTime.Now;
            //recurs_facturat.Refrescar();
            ContextDAO.Registrar();
        }

        /// <summary>
        /// Crea un nou recurs facturat, i ho registra al sistema
        /// </summary>
        /// <param name="id_factura">Identificador de la factura on afegir l'ús</param>
        /// <param name="id_recurs">Identificador del recurs que s'ha de facturar</param>
        /// <param name="quantitat">Quantitat de recurs a facturar</param>
        /// <param name="data_inici_servei">Data i hora d'inici de la utilització</param>
        /// <param name="data_fi_servei">Data i hora de fi de la utilització</param>
        /// <param name="usuari_crea_factura">Usuari que afegeix el recurs a la factura</param>
        /// <returns>Recurs facturat creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public Recurs_facturat CreateRecursFacturat(String id_factura, String id_recurs, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_crea_factura)
        {
            Recurs_facturat recurs_facturat;
            Recurs recurs = Recurs.GetInstancia(id_recurs);
            if (recurs == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del recurs"
                }, new FaultReason("Error"));
            }

            Factura factura = Factura.GetInstancia(id_factura);
            if (factura == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador de la factura"
                }, new FaultReason("Error"));
            }

            if (usuari_crea_factura.Equals(factura.usuari_factura))
            {
                //comprovo si el usuari té autorització per esborrar una oferta facturada (propia)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear el recurs facturat"
                    }, new FaultReason("Error"));
            }
            else
                //comprovo si el usuari té autorització per esborrar una oferta facturada (forana)
                if (!Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_factura).nom_rol).factura_forana[2].Equals('1'))
                    throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                    {
                        Detail = "L'usuari no té els drets suficients per crear el recurs facturat"
                    }, new FaultReason("Error"));


            recurs_facturat = Recurs_facturat.CreateRecurs_facturat(id_factura, id_recurs, recurs.preu, quantitat, recurs.TipusIVA.percentatge, data_inici_servei, data_fi_servei, usuari_crea_factura, usuari_crea_factura, DateTime.Now, DateTime.Now);
            //recurs_facturat.Refrescar();
            recurs_facturat.Insertar();
            //ContextDAO.Registrar();
            return recurs_facturat;
        }

        /// <summary>
        /// Obté un tipus d'IVA del sistema
        /// </summary>
        /// <param name="id">Identificador de l'objecte a retornar</param>
        /// <returns>Objecte buscat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        public TipusIVA GetTipusIVA(String id)
        {
            TipusIVA result = TipusIVA.GetInstancia(id);
            if (result != null)
                return result;
            else
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del tipus d'IVA"
                }, new FaultReason("Error"));
            }
        }

        /// <summary>
        /// Obté un llistat de tots els tipus d'IVA actius al sistema
        /// </summary>
        /// <returns>Llistat amb els tipus d'IVA</returns>
        public List<TipusIVA> GetTipusIVA() { return TipusIVA.GetInstanciesActives(); }

        /// <summary>
        /// Obté un llistat de noms de tots els tipus d'IVA actius al sistema
        /// </summary>
        /// <returns>Llistat amb noms dels tipus d'IVA</returns>
        public List<string> GetNomTipusIVA() { return TipusIVA.GetIdInstanciesActives(); }

        /// <summary>
        /// Esborra un Tipus d'IVA i ho registra a la persistència. A més, canvia tots els recursos que tenen assignat aquest IVA al tipus d'IVA més car
        /// </summary>
        /// <param name="id">Nom del tipus d'IVA</param>
        /// <param name="usuari_esborra">Usuari que esborra l'element</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void DeleteTipusIVA(String id, String usuari_esborra)
        {
            TipusIVA tipus = TipusIVA.GetInstancia(id);
            if (tipus == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del tipus d'IVA"
                }, new FaultReason("Error"));
            }

            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_esborra).nom_rol).tipus_IVA[2].Equals('1'))
            {
                //tipus.Borrar();
                tipus.actiu_IVA = false;
                //tipus.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per esborrar el tipus d'IVA"
            }, new FaultReason("Error"));
        }

        /// <summary>
        /// Classe que modifica un tipus d'IVA i el modifica a la persistència
        /// </summary>
        /// <param name="nom_IVA">Nou nom del tipus d'IVA</param>
        /// <param name="percentatge">Nou percentatge del tipus</param>
        /// <param name="usuari_crea_IVA">Usuari que modifica el tipus</param>
        /// <exception cref="FaultReason">Es llença una excepció en cas que l'identificador de l'objecte no existeixi</exception>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public void UpdateTipusIVA(String nom_IVA, Decimal percentatge, string descripcio, String usuari_mod_IVA)
        {
            TipusIVA tipusIVA = TipusIVA.GetInstancia(nom_IVA);
            if (tipusIVA == null)
            {
                throw new FaultException<ErrorDataContract>(new ErrorDataContract()
                {
                    Detail = "No es troba identificador del tipus d'IVA"
                }, new FaultReason("Error"));
            }

            if (Rol.GetInstancia(Usuari.GetInstancia(tipusIVA.usuari_mod_IVA).nom_rol).tipus_IVA[1].Equals('1'))
            {
                tipusIVA.percentatge = percentatge;
                tipusIVA.actiu_IVA = true;
                tipusIVA.observacions = descripcio;
                tipusIVA.usuari_mod_IVA = usuari_mod_IVA;
                tipusIVA.data_mod_IVA = DateTime.Now;
                //tipusIVA.Refrescar();
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per modificar el tipus d'IVA"
            }, new FaultReason("Error"));

        }

        /// <summary>
        /// Classe que crea un nou tipus d'IVA i l'afegeix a la persistència
        /// </summary>
        /// <param name="nom_IVA">Nou nom del tipus d'IVA</param>
        /// <param name="percentatge">Percentatge del nou tipus</param>
        /// <param name="usuari_crea_IVA">Usuari que crea el nou tipus</param>
        /// <returns>Tipus d'IVA creat</returns>
        /// <exception cref="FaultReason">Es llença una excepció en cas que no es tinguin els drets suficients</exception>
        [SecurityOperationBehavior]
        public TipusIVA CreateTipusIVA(String nom_IVA, Decimal percentatge, String descripcio, String usuari_crea_IVA)
        {
            TipusIVA iva;
            if (Rol.GetInstancia(Usuari.GetInstancia(usuari_crea_IVA).nom_rol).tipus_IVA[0].Equals('1'))
            {
                iva = TipusIVA.CreateTipusIVA(nom_IVA, percentatge, descripcio, usuari_crea_IVA);
                ContextDAO.Registrar();
            }
            else throw new FaultException<ErrorDataContract>(new ErrorDataContract()
            {
                Detail = "L'usuari no té els drets suficients per crear el tipus d'IVA"
            }, new FaultReason("Error"));
            iva.Insertar();
            return iva;
        }

 
        /// <summary>
        /// Funció que si s'arriba a executar, es que ha superat el test de Login, o sigui que el codi 
        /// d'usuri i password son correctes, si no, s'haurà llençat una excepció
        /// </summary>
        /// <param name="usuari_consulta">Usuari que fa la consulta</param>
        /// <returns></returns>
        [SecurityOperationBehavior]
        public bool TestLogin(string usuari_consulta)
        {
            return true;
        }
        #endregion

    }
}