﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Dispatcher;
using XiquiparcsReunits_DAO;
using System.ServiceModel;
using System.ServiceModel.Channels;
using GestioImplementation;
using System.Windows;

namespace GestioService
{
    /// <summary>
    /// Funció que es cridada en cada petició, que comprova les dades de l'usuari i el codi en la capcelera SOAP
    /// </summary>
    class SecurityOperationInvoker : IOperationInvoker
    {
        public SecurityOperationInvoker(IOperationInvoker operationInvoker)
        {
            this.InnerOperationInvoker = operationInvoker;
        }

        private IOperationInvoker InnerOperationInvoker { get; set; }

        #region IOperationInvoker Members

        public object[] AllocateInputs()
        {
            return InnerOperationInvoker.AllocateInputs();
        }

        public object Invoke(object instance, object[] inputs, out object[] outputs)
        {
            MessageHeaders messageHeadersElement = OperationContext.Current.IncomingMessageHeaders;

            if (messageHeadersElement.FindHeader("username", "") < 0 || messageHeadersElement.FindHeader("password", "") < 0)
                throw new FaultException<InvalidOperationException>(new InvalidOperationException("No s'ha presentat dades d'identificació"), new FaultReason("No s'ha presentat dades d'identificació"));

            string username = messageHeadersElement.GetHeader<String>("username", "");
            string password = messageHeadersElement.GetHeader<String>("password", "");
            Usuari user = Usuari.GetInstancia(username);
            if (user == null || !user.pwd.Equals(password))
                throw new FaultException<InvalidOperationException>(new InvalidOperationException("Identificació incorrecta"), new FaultReason("Identificació incorrecta"));

            //Com totes les funcions que requereixen seguretat, inclouen el usuari que modifica, crea o esborra,
            //per seguretat, comprovo que aquest usuari coincideixi amb el de la capcelera
            if (!((string)inputs[inputs.Length-1]).Equals(username))
                throw new FaultException<InvalidOperationException>(new InvalidOperationException("No coincideix l'usuari registrat i l'usuari que registra la funció"), new FaultReason("No coincideix l'usuari registrat i l'usuari que registra la funció"));

            return InnerOperationInvoker.Invoke(instance, inputs, out outputs);
        }

        public IAsyncResult InvokeBegin(object instance, object[] inputs, AsyncCallback callback, object state)
        {
            return InnerOperationInvoker.InvokeBegin(instance, inputs, callback, state);
        }

        public object InvokeEnd(object instance, out object[] outputs, IAsyncResult result)
        {
            return InnerOperationInvoker.InvokeEnd(instance, out outputs, result);
        }

        public bool IsSynchronous
        {
            get { return InnerOperationInvoker.IsSynchronous; }
        }

        #endregion
    }
}
