﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using XiquiparcsReunits_DAO;

namespace GestioService
{
    [ServiceContract]
    public interface IGestorXiquiparc
    {
        [OperationContract(Name = "GetOfertes")]
        List<Oferta> GetOfertes();

        [OperationContract(Name = "GetNomOfertes")]
        List<string> GetNomOfertes();

        [OperationContract(Name = "GetOfertesByRecurs")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Oferta> GetOfertes(string id_recurs);

        [OperationContract(Name = "GetNomOfertesByRecurs")]
        [FaultContract(typeof(ErrorDataContract))]
        List<string> GetNomOfertes(string id_recurs);

        [OperationContract]
        Oferta GetOferta(String id);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteOferta(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateOferta(String id_oferta, Decimal import_unitari, String id_recurs, int quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, String usuari_mod_oferta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Oferta CreateOferta(Decimal import_unitari, String id_recurs, int quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, String usuari_crea_oferta);

        [OperationContract(Name = "GetRecursosBySeccio")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Recurs> GetRecursos(string id_seccio);

        [OperationContract(Name = "GetRecursos")]
        List<Recurs> GetRecursos();

        [OperationContract(Name = "GetNomRecursos")]
        List<string> GetNomRecursos();

        [OperationContract(Name = "GetNomRecursosBySeccio")]
        [FaultContract(typeof(ErrorDataContract))]
        List<string> GetNomRecursos(string id_seccio);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Recurs GetRecurs(String nom);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteRecurs(String nom, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateRecurs(String id_recurs, String id_seccio, Decimal preu, String nom_IVA, int quantitat, bool disponibilitat_controlable, String descripcio, String usuari_mod_recurs);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Recurs CreateRecurs(String id_recurs, String id_seccio, Decimal preu, String nom_IVA, int quantitat, bool disponibilitat_controlable, string descripcio, String usuari_crea_recurs);

        [OperationContract]
        List<Seccio> GetSeccions();

        [OperationContract]
        List<string> GetNomSeccions();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Seccio GetSeccio(String id);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteSeccio(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateSeccio(String id_seccio, string descripcio, String usuari_mod_seccio);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Seccio CreateSeccio(String id_seccio, string descripcio, String usuari_crea_seccio);

        [OperationContract(Name = "GetUsuaris")]
        List<Usuari> GetUsuaris(string usuari_consulta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Usuari GetUsuari(String id, string usuari_consulta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        string GetIdUsuari(String nom_usuari);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Usuari GetUsuariByNom(String nom, string usuari_consulta);

        [OperationContract(Name = "GetUsuarisByRol")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Usuari> GetUsuaris(string id_rol, string usuari_consulta);

        [OperationContract(Name = "GetNomUsuaris")]
        List<string> GetNomUsuaris(string usuari_consulta);

        [OperationContract(Name = "GetNomUsuarisByRol")]
        [FaultContract(typeof(ErrorDataContract))]
        List<string> GetNomUsuaris(string id_rol, string usuari_consulta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteUsuari(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateUsuari(String id, String nom, String nif, String nomFiscal, String tel1, String tel2, String movil, String direccio, String poblacio, int codpos, String provincia, String estat, String pwd, String observacions, string nom_rol, string id_usuari_modifica);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Usuari CreateUsuari(String nom, String nif, String nomFiscal, String tel1, String tel2, String movil, String direccio, String poblacio, int codpos, String provincia, String estat, String pwd, String observacions, string nom_rol, string id_usuari_crea);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        List<Rol> GetRols();

        [OperationContract]
        List<string> GetNomRols();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Rol GetRol(String nom);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteRol(String nom, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateRol(String nom, bool acces_escriptori, bool modifica_empresa, string dretRol, string dretUsuari, string dretReservaForana, string dretReserva, string dretRecurs, string dretSeccio, string dretOferta, string dretFactura, string dretFacturaForana, string dretCobrament, string dretIVA, string dretPagina, string dretContinent, string dretContingut, string usuari_modifica);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Rol CreateRol(
                string nom,
                bool acces_escriptori,
                bool modifica_empresa,
                string dretRol,
                string dretUsuari,
                string dretReservaForana,
                string dretReserva,
                string dretRecurs,
                string dretSeccio,
                string dretOferta,
                string dretFactura,
                string dretFacturaForana,
                string dretCobrament,
                string dretIVA,
                string dretPagina,
                string dretContinent,
                string dretContingut,
                string usuari_modifica);

        [OperationContract]
        List<Continent> GetContinents();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Continent GetContinent(String id);

        [OperationContract]
        List<Contingut> GetContinguts();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        List<Contingut> GetContingutsByPagina(int pagina);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Contingut GetContingut(int posicio, int posicio_pagina);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteContingut(int posicio, int pagina, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateContingut(int posicio_contingut, int posicio_pagina, String id_continent, String xaml, String observacions, String usuari_mod_contingut);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Contingut CreateContingut(int pos_pagina, String id_continent, String xaml, String observacions, String usuari_crea_contingut);


        [OperationContract]
        List<Pagina> GetPagines();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Pagina GetPagina(int posicio);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeletePagina(int posicio, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdatePagina(int pos_pagina, String nom_pagina, String text_menu, String observacions, String usuari_mod_pagina);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Pagina CreatePagina(String nom_pagina, String text_menu, String observacions, String usuari_crea_pagina);
	
        [OperationContract]
        List<Cobrament> GetCobraments();

        [OperationContract]
        List<string> GetNomCobraments();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Cobrament GetCobrament(String id);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteCobrament(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateCobrament(String estat_cobrament, String descripcio, String usuari_mod_cobrament);

        [OperationContract]
        Cobrament CreateCobrament(String id, String estat_cobrament, String usuari_crea_cobrament);

        [OperationContract]
        Empresa GetEmpresa();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateEmpresa(String nIF, String nom_fiscal, String nom_comercial, String telefon1, string telefon2, string telefon3, string fax, String correu_electronic, String direccio_web, String direccio_empresa, Int32 codpos_empresa, String poblacio, String provincia, string nom_gerent, string comentaris, string altra_info, string dades_mercantils, String estat, string usuari_mod_empresa);
       
        [OperationContract(Name = "GetFactures")]
        List<Factura> GetFactures(string usuari_consulta);

        [OperationContract(Name = "GetFacturesByUsuari")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Factura> GetFactures(string id_usuari, string usuari_consulta);

        [OperationContract(Name = "GetFacturesByUsuariAndCobrament")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Factura> GetFactures(string id_usuari, string nom_cobrament, string usuari_consulta);

        [OperationContract(Name = "GetFacturesByCobrament")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Factura> GetFacturesByCobrament(string id_cobrament, string usuari_consulta);

        [OperationContract(Name = "GetNomFactures")]
        List<string> GetNomFactures(string usuari_consulta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Factura GetFactura(String id, string usuari_consulta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteFactura(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateFactura(String id_factura, bool modificable, String estat_cobrament, DateTime data_cobrament, Decimal a_compte, String descripcio, String usuari_mod_factura);
       
        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Factura CreateFactura(String client, String estat_cobrament, DateTime data_cobrament, String descripcio, Decimal a_compte, bool modificable, String usuari_crea_factura);

        [OperationContract]
        List<Oferta_facturada> GetOfertesFacturades();

        [OperationContract]
        List<string> GetNomOfertesFacturades();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        List<Oferta_facturada> GetOfertesFacturadesByFactura(string id_factura);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        List<Oferta_facturada> GetOfertesFacturadesByOferta(string id_oferta);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Oferta_facturada GetOfertaFacturada(String id_factura, String id_recurs);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteOfertaFacturada(String id_factura, String id_oferta, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateOfertaFacturada(String id_factura, String id_oferta, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_mod_factura);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Oferta_facturada CreateOfertaFacturada(String id_factura, String id_oferta, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_crea_factura);

        [OperationContract(Name = "GetRecursosFacturatsByFactura")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Recurs_facturat> GetRecursosFacturatsByFactura(string id_factura);

        [OperationContract(Name = "GetRecursosFacturatsByRecurs")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Recurs_facturat> GetRecursosFacturatsByRecurs(string id_recurs);

        [OperationContract(Name = "GetRecursosFacturats")]
        [FaultContract(typeof(ErrorDataContract))]
        List<Recurs_facturat> GetRecursosFacturats();

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Recurs_facturat GetRecursFacturat(String id_factura, String id_recurs);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteRecursFacturat(String id_factura, String id_recurs, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateRecursFacturat(String id_factura, String id_recurs, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_mod_factura);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        Recurs_facturat CreateRecursFacturat(String id_factura, String id_recurs, int quantitat, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_crea_factura);

        [OperationContract(Name = "GetTipusIVA")]
        List<TipusIVA> GetTipusIVA();

        [OperationContract(Name = "GetNomTipusIVA")]
        List<string> GetNomTipusIVA();

        [OperationContract(Name = "GetOneTipusIVA")]
        [FaultContract(typeof(ErrorDataContract))]
        TipusIVA GetTipusIVA(String id);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void DeleteTipusIVA(String id, String usuari_esborra);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        void UpdateTipusIVA(String nom_IVA, Decimal percentatge, String descripcio, String usuari_mod_IVA);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        TipusIVA CreateTipusIVA(String nom_IVA, Decimal percentatge, String descripcio, String usuari_crea_IVA);

        [OperationContract]
        [FaultContract(typeof(ErrorDataContract))]
        bool TestLogin(string usuari_consulta);
    }
}
