﻿using System.Collections.Generic;
using System.Configuration;
using System.Web;
using System.Linq;
using System.Data.Objects;
using System;

/**
 * Classe que defineix i controla els contextos d'acces a dades
 * 
 * Els contextos son uns intermediaris entre la base de dades i les diferents aplicacions
 * 
 * En aquest cas, es crea una llista de contextes, 1 per cada connexió web, i 1 per totes les connexions d'escriptori
 * 
 **/
namespace XiquiparcsReunits_DAO
{
    public partial class ContextDAO
    {
        private static Dictionary<string, xiquiparcEntities> _LListaContextDades;
        private static Dictionary<string, xiquiparcEntities> LListaContextDades
        {
            get { return ContextDAO._LListaContextDades; }
            set { ContextDAO._LListaContextDades = value; }
        }

        /**
         * Constructor estatic que inicialitza el diccionari de entitats
         **/
        static ContextDAO()
        {
            ContextDAO.LListaContextDades = new Dictionary<string, xiquiparcEntities>();
        }

        /**
         * Funció que crea un nou context de dades, es definirà com a identificador de la connexió "xiquiparcEntities"
         * 
         **/
        private static xiquiparcEntities CrearContextDades()
        {
            
            //Creo un nou context de dadesd 
            xiquiparcEntities oContextDades = new xiquiparcEntities(ConfigurationManager.ConnectionStrings["xiquiparcEntities"].ConnectionString);

            // Definició de les regles d'actualització de cada entitat, per defecte es AppendOnly
            //oContextDades.Rol.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.Seccio.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.TipusIVA.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.Empresa.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.Cobrament.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.Continent.MergeOption = MergeOption.OverwriteChanges;
            //oContextDades.Usuari.MergeOption = MergeOption.OverwriteChanges;

            return oContextDades;
        }

        /**
         * Funció que comprova si ja hi ha un context creat per a aquest usuari (web) o en crea un de nou i el retorna.
         * Pel cas de l'escriptori, retorna sempre el mateix context
         * 
         **/
        internal static xiquiparcEntities ContextDades
        {
            get
            {
                xiquiparcEntities oContextDades = null;
                if (HttpContext.Current != null && HttpContext.Current.Session != null)
                {
                    if (LListaContextDades.ContainsKey(HttpContext.Current.Session.SessionID))
                    {
                        oContextDades = LListaContextDades[HttpContext.Current.Session.SessionID];
                    }
                    else
                    {
                        oContextDades = ContextDAO.CrearContextDades();
                        LListaContextDades.Add(HttpContext.Current.Session.SessionID, oContextDades);
                    }
                }
                else
                {
                    if (LListaContextDades.ContainsKey("AppliWin"))
                    {
                        oContextDades = LListaContextDades["AppliWin"];
                    }
                    else
                    {
                        oContextDades = ContextDAO.CrearContextDades();
                        LListaContextDades.Add("AppliWin", oContextDades);
                    }
                }
                return oContextDades;
            }
        }

        /**
         * Funció que esborrar un context en el moment en que no sigui necessari
         * 
         **/
        public static void BorrarContext(string aIdContextoDatos)
        {
            if (LListaContextDades.ContainsKey(aIdContextoDatos))
            {
                LListaContextDades.Remove(aIdContextoDatos);
            }
        }

        /**
         * Funció que s'encarregarà de registrar els canvis a la bbdd (fer commit)
         * 
         **/
        public static void Registrar()
        {
            ContextDAO.ContextDades.SaveChanges();
            
        }

    }

}
