﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects.DataClasses;

namespace XiquiparcsReunits_DAO
{
    /// <summary>
    /// Objecte que registra els bens o serveis oferts a un client en un periode determinat. Es tanca la facturació, un cop
    /// el client ha pagat la totalitat de l'import debengat. En aquest estat no es podrà modificar.
    /// També registra reserves futures, ja que es factures, i passen a estat de cobrament pendent.
    /// </summary>
    public partial class Factura : EntityObject
    {
        public TipusIVA TipusIVA
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        public Empresa Empresa
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        public Cobrament Cobrament1
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }
        /// <summary>
        /// Varia l'estat de la factura a no modificable
        /// </summary>
        /// <param name="usuari_mod_factura">Usuari que realitza el canvi</param>
        public void inmodificable(String usuari_mod_factura)
        {
            this.modificable = false;
            this.usuari_mod_factura = usuari_mod_factura;
            this.data_mod_factura = DateTime.Now;
        }

        /// <summary>
        /// Crea una nova factura amb els paràmetres principals
        /// </summary>
        /// <param name="client">Identificador del client</param>
        /// <param name="estat_cobrament">Estat de cobrament de la factura</param>
        /// <param name="data_cobrament">Data en que es realitza l'últim cobrament</param>
        /// <param name="a_compte">Import a compte que acumula l'usuari</param>
        /// <param name="descripcio">Descripcio de la factura</param>
        /// <param name="modificable">Estat de la factura, mostra si es modificable</param>
        /// <param name="usuari_crea_factura">Usuari que crea la factura</param>
        /// <returns>Objecte creat</returns>
        public static Factura CreateFactura(String client, String estat_cobrament, DateTime data_cobrament, String descripcio, Decimal a_compte, bool modificable, String usuari_crea_factura)
        {
            Factura factura = new Factura();
            factura.id_factura = "f"+(Factura.GetInstancies().Count+1).ToString("D8");
            factura.usuari_factura = client;
            factura.estat_cobrament = estat_cobrament;
            factura.modificable = modificable;
            factura.data_cobrament = data_cobrament;
            factura.a_compte = a_compte;
            factura.actiu_factura = true;
            factura.usuari_crea_factura = usuari_crea_factura;
            factura.usuari_mod_factura = usuari_crea_factura;
            factura.data_crea_factura = DateTime.Now;
            factura.data_mod_factura = DateTime.Now;
            return factura;
        }

        /// <summary>
        /// Obté un llistat de totes les factures del sistema
        /// </summary>
        /// <returns>Llista de factures</returns>
        public static List<Factura> GetInstancies()
        {
            return (from oFactura in ContextDAO.ContextDades.Factura select oFactura).ToList();
        }

        /// <summary>
        /// Obté un objecte concret a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de l'objecte</param>
        /// <returns>Objecte buscat</returns>
        public static Factura GetInstancia(string id)
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.id_factura == id
                    select oFactura).FirstOrDefault();
        }

        /// <summary>
        /// Obté un llistat de tots els identificadors de les factures actives del sistema
        /// </summary>
        /// <returns>Llista d'identificador de les factures actives del sistema</returns>
        public static List<String> GetIdInstanciesActives()
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true
                    select oFactura.id_factura).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les factures actives del sistema
        /// </summary>
        /// <returns>LLista de totes les factures actives del sistema</returns>
        public static List<Factura> GetInstanciesActives()
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true
                    select oFactura).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les factures actives i en un estat de modificabilitat
        /// </summary>
        /// <param name="modificable">Estat de modificabilitat buscat</param>
        /// <returns>Llistat de factures buscat</returns>
        public static List<Factura> GetInstanciesActives(Boolean modificable)
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true && oFactura.modificable == modificable
                    select oFactura).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les factures actives i en un estat de cobrament concret
        /// </summary>
        /// <param name="nom_cobrament">Identificador del tipus de cobrament de les factures</param>
        /// <returns>Llista d'objectes buscat</returns>
        public static List<Factura> GetInstanciesActivesByCobrament(string nom_cobrament)
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true && oFactura.estat_cobrament==nom_cobrament
                    select oFactura).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les factures actives d'un usuari
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari</param>
        /// <returns>Llista de factures</returns>
        public static List<Factura> GetInstanciesActivesByUsuari(string id_usuari)
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true && oFactura.usuari_factura==id_usuari
                    select oFactura).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les factures actives d'un usuari en en estat de cobrament
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari</param>
        /// <param name="nom_cobrament">Identificador de l'estat de cobrament</param>
        /// <returns>Llisat buscat</returns>
        public static List<Factura> GetInstanciesActivesByUsuariAndCobrament(string id_usuari, string nom_cobrament)
        {
            return (from oFactura in ContextDAO.ContextDades.Factura
                    where oFactura.actiu_factura == true && oFactura.usuari_factura == id_usuari && oFactura.estat_cobrament == nom_cobrament
                    select oFactura).ToList();
        }

	}
}

