﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects.DataClasses;

namespace XiquiparcsReunits_DAO
{
    /// <summary>
    /// Objecte que representa la utilització d'un recurs en una factura però en un preu promocional
    /// </summary>
    public partial class Oferta_facturada : EntityObject
	{
        /// <summary>
        /// Obté el Recurs ofertat en aquest apunt
        /// </summary>
        /// <returns>Recurs ofertat en aquest apunt</returns>
        public Recurs RecursOfertat()
        {
           return Recurs.GetInstancia((from oOferta in ContextDAO.ContextDades.Oferta
                    where  oOferta.id_oferta == this.id_oferta
                    select oOferta.id_recurs).FirstOrDefault());
        }

        /// <summary>
        /// Constructor amb els paràmetres principals inicialitzats
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_oferta">Identificador de la oferta</param>
        /// <param name="import">Import de la oferta del producte</param>
        /// <param name="quantitat">Quantitat facturada</param>
        /// <param name="percentatge_IVA">Percentatge d'IVA del producte facturat</param>
        /// <param name="data_inici_servei">Data i hora d'inici d'utilització</param>
        /// <param name="data_fi_servei">Data i hora de fi d'utilització</param>
        /// <param name="usuari_crea_factura">Usuari que modifica la factura</param>
        /// <returns>Nou recursos ofertat facturat</returns>
        public static Oferta_facturada CreateOferta_facturada(String id_factura, String id_oferta, Decimal import, Int32 quantitat, Decimal percentatge_IVA, DateTime data_inici_servei, DateTime data_fi_servei, String usuari_crea_factura)
        {
            Oferta_facturada oferta_facturada = new Oferta_facturada();
            oferta_facturada.id_factura = id_factura;
            oferta_facturada.id_oferta = id_oferta;
            oferta_facturada.import = import;
            oferta_facturada.quantitat = quantitat;
            oferta_facturada.percentatge_IVA = percentatge_IVA;
            oferta_facturada.data_inici_servei = data_inici_servei;
            oferta_facturada.data_fi_servei = data_fi_servei;
            oferta_facturada.usuari_crea_factura = usuari_crea_factura;
            oferta_facturada.usuari_mod_factura = usuari_crea_factura;
            oferta_facturada.data_crea_factura = DateTime.Now;
            oferta_facturada.data_mod_factura = DateTime.Now;
            return oferta_facturada;
        }
        
        /// <summary>
        /// Obté un llistat de totes les ofertes facturades d'una factura
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <returns>Llista amb les ofertes facturades</returns>
        public static List<Oferta_facturada> GetInstanciesByFactura(string id_factura)
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada
                    where oOferta_facturada.id_factura==id_factura
                    select oOferta_facturada).ToList();
        }

        /// <summary>
        /// Obté un llistat de totes les ofertes facturades del sistema 
        /// </summary>
        /// <returns>Llista amb les ofertes facturades</returns>
        public static List<Oferta_facturada> GetInstancies()
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada select oOferta_facturada).ToList();
        }

        /// <summary>
        /// Obté una oferta facturada a partir dels seus identificadors
        /// </summary>
        /// <param name="id_factura">Identificador de la factura origen</param>
        /// <param name="id_oferta">Identificador de la oferta del producte facturat</param>
        /// <returns>Objecte buscat</returns>
        public static Oferta_facturada GetInstancia(String id_factura, String id_oferta)
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada
                    where oOferta_facturada.id_factura == id_factura && oOferta_facturada.id_oferta == id_oferta
                    select oOferta_facturada).FirstOrDefault();
        }

        /// <summary>
        /// Llista de totes les ofertes de productes facturades actives al sistema
        /// </summary>
        /// <returns>Llista amb les ofertes facturades actives al sistema</returns>
        public static List<Oferta_facturada> GetInstanciesActives()
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada
                    join oFactura in ContextDAO.ContextDades.Factura
                    on oOferta_facturada.id_factura equals oFactura.id_factura
                    where oFactura.actiu_factura == true
                    select oOferta_facturada).ToList();
        }

        /// <summary>
        /// Llista de noms de totes les ofertes de productes facturades actives al sistema
        /// </summary>
        /// <returns>Llista amb noms de les ofertes facturades actives al sistema</returns>
        public static List<string> GetNomInstanciesActives()
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada
                    join oFactura in ContextDAO.ContextDades.Factura
                    on oOferta_facturada.id_factura equals oFactura.id_factura
                    where oFactura.actiu_factura == true
                    select oOferta_facturada.id_oferta).ToList();
        }

        /// <summary>
        /// Obté una llista amb les ofertes facturades d'una oferta de producte concret
        /// </summary>
        /// <param name="id_oferta">Identificador de la oferta del recurs</param>
        /// <returns>Llista amb les ofertes facturades</returns>
        public static List<Oferta_facturada> GetInstanciesByOferta(string id_oferta)
        {
            return (from oOferta_facturada in ContextDAO.ContextDades.Oferta_facturada
                    where oOferta_facturada.id_oferta == id_oferta
                    select oOferta_facturada).ToList();
        }

	}
}

