﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Objects.DataClasses;

namespace XiquiparcsReunits_DAO
{

    public partial class Continent : EntityObject
    {
        /// <summary>
        /// Obté un llistat dels continents del sistema
        /// </summary>
        /// <returns>Llistat amb els continents del sistema</returns>
        public static List<Continent> GetInstancies()
        {
            List<Continent> continents = new List<Continent>();
           
                    continents = (from oContinent in ContextDAO.ContextDades.Continent select oContinent).ToList();
           
            return continents;
        }

        /// <summary>
        /// Constructor sense paràmetres inicialitzats
        /// </summary>
        public Continent()
        {
        }

        /// <summary>
        /// Constructor amb tots els paràmetres inicialitzats
        /// </summary>
        /// <param name="id_continent">Identificador del continent</param>
        /// <param name="amplada">Amplada del continent</param>
        /// <param name="alcada">Alçada del continent</param>
        /// <param name="xaml">Descripció (UserControl) del contingut</param>
        /// <param name="observacions">Observacions sobre el continent</param>
        /// <param name="usuari_crea_continent">Usuari que crea el continent</param>
        public Continent(string id_continent, int amplada, int alcada, string xaml, string observacions, string usuari_crea_continent){
            this.id_continent = id_continent;
            this.amplada = amplada;
            this.alcada = alcada;
            this.xaml = xaml;
            this.observacions = observacions;
            this.usuari_crea_continent = usuari_crea_continent;
            this.usuari_mod_continent = usuari_crea_continent;
            this.data_crea_continent = DateTime.Now;
            this.data_mod_continent = DateTime.Now;
        }

        /// <summary>
        /// Obté un continent a partir del seu id
        /// </summary>
        /// <param name="id">Identificador del continent</param>
        /// <returns>Continent buscat</returns>
        public static Continent GetInstancia(string id)
        {
            return (from oContinent in ContextDAO.ContextDades.Continent
                    where oContinent.id_continent == id
                    select oContinent).FirstOrDefault();
        }

        /// <summary>
        /// Obté un llistat dels noms dels continents del sistema
        /// </summary>
        /// <returns>Llistat dels noms dels continents del sistema</returns>
        public static List<String> GetNomInstancies()
        {
            return (from oContinent in ContextDAO.ContextDades.Continent
                    select oContinent.id_continent).ToList();
        }

        /// <summary>
        /// Fàbrica amb tots els paràmetres inicialitzats
        /// </summary>
        /// <param name="id_continent">Identificador del continent</param>
        /// <param name="amplada">Amplada en píxels del continent</param>
        /// <param name="alcada">Alçada en píxels del continent</param>
        /// <param name="xaml">Codi xaml (UserControl) del continent</param>
        /// <param name="observacions">Observacions sobre el continent</param>
        /// <param name="usuari_crea_continent">Usuari que crea el continent</param>
        /// <returns>Continent creat</returns>
        public static Continent CreateContinent(string id_continent, int amplada, int alcada, string xaml, string observacions, string usuari_crea_continent){
            return new Continent(id_continent, amplada, alcada, xaml, observacions, usuari_crea_continent);
        }

        public Pagina Pagina
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        public Contingut Contingut1
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }
    }
}
