﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Data.Objects.DataClasses;

namespace XiquiparcsReunits_DAO
{

    public partial class Contingut : EntityObject
    {
        /// <summary>
        /// Obté un llistat dels Continguts del sistema
        /// </summary>
        /// <returns>Llistat amb els Continguts del sistema</returns>
        public static List<Contingut> GetInstancies()
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut select oContingut).ToList();
        }

        /// <summary>
        /// Obté un llistat dels Continguts de d'un tipus de continent
        /// </summary>
        /// <returns>Llistat amb els Continguts d'un tipus de continent</returns>
        public static List<Contingut> GetInstancies(Continent continent)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_continent == continent.id_continent
                    select oContingut).ToList();
        }

        /// <summary>
        /// Obté un llistat dels Continguts d'una pàgina
        /// </summary>
        /// <returns>Llistat amb els Continguts d'una pàgina</returns>
        public static List<Contingut> GetInstancies(Pagina pagina)
        {
            List<Contingut> result = new List<Contingut>();
                    result= (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_pagina == pagina.id_pagina
                             select oContingut).ToList();
            return result;
        }


        /// <summary>
        /// Obté un llistat dels Continguts d'una pàgina d'un tipus de continent
        /// </summary>
        /// <returns>Llistat amb els Continguts d'una pàgina del tipus de continent seleccionat</returns>
        public static List<Contingut> GetInstancies(Pagina pagina, Continent continent)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_pagina == pagina.id_pagina && oContingut.id_continent==continent.id_continent
                    select oContingut).ToList();
        }


        /// <summary>
        /// Obté un Contingut a partir de la seva posició
        /// </summary>
        /// <param name="id">Identificador del Contingut</param>
        /// <returns>Contingut buscat</returns>
        public static Contingut GetInstancia(int posicio, int posicio_pagina)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_contingut==posicio && oContingut.id_pagina==posicio_pagina
                    select oContingut).FirstOrDefault();
        }

        /// <summary>
        /// Obté un llistat dels noms dels Continguts del sistema
        /// </summary>
        /// <returns>Llistat dels noms dels Continguts del sistema</returns>
        public static List<String> GetNomInstancies()
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    select oContingut.id_contingut.ToString()).ToList();
        }

        /// <summary>
        /// Obté un llistat dels noms dels Continguts de d'un tipus de continent
        /// </summary>
        /// <returns>Llistat amb els noms dels Continguts d'un tipus de continent</returns>
        public static List<string> GetNomInstancies(Continent continent)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_continent == continent.id_continent
                    select oContingut.id_contingut.ToString()).ToList();
        }

        /// <summary>
        /// Obté un llistat dels noms dels Continguts d'una pàgina
        /// </summary>
        /// <returns>Llistat amb els noms dels Continguts d'una pàgina</returns>
        public static List<string> GetNomInstancies(Pagina pagina)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_pagina == pagina.id_pagina
                    select oContingut.id_contingut.ToString()).ToList();
        }


        /// <summary>
        /// Obté un llistat dels noms dels Continguts d'una pàgina d'un tipus de continent
        /// </summary>
        /// <returns>Llistat amb els noms dels Continguts d'una pàgina del tipus de continent seleccionat</returns>
        public static List<string> GetNomInstancies(Pagina pagina, Continent continent)
        {
            return (from oContingut in ContextDAO.ContextDades.Contingut
                    where oContingut.id_pagina == pagina.id_pagina && oContingut.id_continent == continent.id_continent
                    select oContingut.id_contingut.ToString()).ToList();
        }

        /// <summary>
        /// Crea un nou contingut que es situarà darrer en la pàgina
        /// </summary>
        /// <param name="id_pagina">Posició de la pàgina que el contindrà</param>
        /// <param name="id_continent">Identificador del tipus de continent</param>
        /// <param name="titol">Titol del contingut</param>
        /// <param name="xaml">Codi xaml</param>
        /// <param name="observacions">Observacions sobre el contingut</param>
        /// <param name="usuari_crea">Usuari que crea el recurs</param>
        /// <returns>Nou Contingut creat</returns>
        public static Contingut CreateContingut(int id_pagina, string id_continent, string xaml, string observacions, string usuari_crea)
        {
            Contingut contingut = new Contingut();
            List<Contingut> continguts = Contingut.GetInstancies();
            if (continguts.Count > 0)
                contingut.id_contingut = continguts.Max(m => m.id_contingut) + 1;
            else contingut.id_contingut = 1;
            contingut.id_pagina = id_pagina;
            contingut.id_continent = id_continent;
            contingut.xaml = xaml;
            contingut.observacions = observacions;
            contingut.usuari_crea_contingut = usuari_crea;
            contingut.usuari_mod_contingut = usuari_crea;
            contingut.data_crea_contingut = DateTime.Now;
            contingut.data_mod_contingut= DateTime.Now;
            return contingut;
        }
    }
}
