﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using XiquiparcsReunits_RDA.Tools;
using System.ServiceModel.Channels;
using System.ServiceModel;


namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Objecte que representa un tipus d'estat de cobrament d'una factura, per exemple, al comptat, pendent de cobrament...
    /// </summary>
    public class Cobrament : INotifyPropertyChanged
    {

        public string Estat_cobrament { get; set; }
        public string Descripcio { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private Cobrament() { }

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        private static ObservableCollection<Cobrament> initInstancies()
        {
            ObservableCollection<Cobrament> llista = new ObservableCollection<Cobrament>();

            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Cobrament it in client.GetCobraments())
            {
                llista.Add(Cobrament.CreateCobrament(it.estat_cobrament, it.descripcio));
            }
            return llista;
        }


        public static ObservableCollection<string> GetNomInstancies()
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient(); 
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            return new ObservableCollection<string>(client.GetNomCobraments());
        }

        public static ObservableCollection<Cobrament> GetInstancies()
        {
            return initInstancies();
        }

        public static Cobrament GetInstancia(string id)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.Cobrament it = client.GetCobrament(id);
            return Cobrament.CreateCobrament(it.estat_cobrament, it.descripcio);
        }

        public static void Insert(Cobrament cobrament)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.CreateCobrament(cobrament.Estat_cobrament, cobrament.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        public static void Remove(string estat_cobrament)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.DeleteCobrament(estat_cobrament, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        public static void Update(Cobrament cobrament)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateCobrament(cobrament.Estat_cobrament, cobrament.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        public static Cobrament CreateCobrament(string estat_cobrament, string descripcio)
        {
            Cobrament cobrament = new Cobrament();
            cobrament.Estat_cobrament = estat_cobrament;
            cobrament.Descripcio = descripcio;
            //cobrament.facturable = facturable;
            return cobrament;
        }

        public override string ToString()
        {
            return this.Estat_cobrament;
        }
    }
}