﻿using System.ComponentModel;
//using XiquiparcsReunits_RDA.XiquiparcsReunits_DAO;
using XiquiparcsReunits_RDA.Tools;

using System;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Objecte que integra totes les dades generals de la empresa: dades fiscals, postals, Internet...
    /// </summary>
	public class Empresa: INotifyPropertyChanged
	{
		public String NIF{get;set;}
		public String Nom_fiscal{get;set;}
		public String Nom_comercial{get;set;}
		public String Telefon1{get;set;}
        public string Telefon2 { get; set; }
        public string Telefon3 { get; set; }
        public string Fax { get; set; }
		public String Correu_electronic{get;set;}
		public  String Direccio_web{get;set;}
		public  String Direccio_empresa{get;set;}
		public  Int32 Codpos_empresa{get;set;}
		public  String Poblacio{get;set;}

        private string provincia;
        public String Provincia { get { return provincia; } set { provincia = value; OnPropertyChanged("Provincia"); } }
        public string Dades_mercantils { get; set; }
        public string Comentaris { get; set; }
        public string Altra_informacio { get; set; }
        public string Nom_gerent { get; set; }
		public  String Estat{get;set;}


        private GestorXiquiparc.GestorXiquiparcClient client;
		
		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

        public Empresa()
        {
            //Aquest client no necessita seguretat
            client = new GestorXiquiparc.GestorXiquiparcClient();
           
            GestorXiquiparc.Empresa empresa= client.GetEmpresa();
            this.NIF = empresa.NIF;
            this.Nom_fiscal = empresa.nom_fiscal;
            this.Nom_comercial = empresa.nom_comercial;
            this.Telefon1 = empresa.telefon1;
            this.Telefon2 = empresa.telefon2;
            this.Telefon3 = empresa.telefon3;
            this.Fax = empresa.fax;
            this.Correu_electronic = empresa.correu_electronic;
            this.Direccio_web = empresa.direccio_web;
            this.Direccio_empresa = empresa.direccio_empresa;
            this.Codpos_empresa = empresa.codpos_empresa;
            this.Poblacio = empresa.poblacio;
            this.Provincia = empresa.poblacio;
            this.Dades_mercantils = empresa.dades_mercantils;
            this.Altra_informacio = empresa.altra_info;
            this.Comentaris = empresa.comentaris;
            this.Nom_gerent = empresa.nom_gerent_empresa;
            this.Estat = empresa.estat;
        }

        public static Empresa GetEmpresa()
        {
            return new Empresa();
        }

        /// <summary>
        /// Funció que retorna un nou objecte amb tots els paràmetres
        /// </summary>
        /// <param name="nIF">Nou NIF de l'empresa</param>
        /// <param name="nom_fiscal">Nou nom fiscal de l'empresa</param>
        /// <param name="nom_comercial">Nou nom comercial de l'empresa</param>
        /// <param name="telefon1">Nou telèfon 1</param>
        /// <param name="telefon2">Nou telèfon 2</param>
        /// <param name="telefon3">Nou telèfon 3</param>
        /// <param name="fax">Nou fax</param>
        /// <param name="correu_electronic">Nou correu electrònic</param>
        /// <param name="direccio_web">Nova direcció web</param>
        /// <param name="direccio_empresa">Direcció postal de l'empresa</param>
        /// <param name="codpos_empresa">Codi postal de l'empresa</param>
        /// <param name="poblacio">Població postal de l'empresa</param>
        /// <param name="provincia">Provincia postal de l'empresa</param>
        /// <param name="nom_gerent">Nom del gerent o representant de l'empresa</param>
        /// <param name="comentaris">Comentaris sobre l'empresa</param>
        /// <param name="altra_info">Altra informació de l'empresa</param>
        /// <param name="dades_mercantils">Dades mercantils de l'empresa</param>
        /// <param name="estat">Estat (pais) de l'empresa</param>
        /// <returns>Objecte creat a partir dels paràmetres</returns>
		public static Empresa CreateEmpresa(String nIF, String nom_fiscal, String nom_comercial, String telefon1, string telefon2, string telefon3, string fax, String correu_electronic, String direccio_web, String direccio_empresa, Int32 codpos_empresa, String poblacio, String provincia, string nom_gerent, string comentaris, string altra_info, string dades_mercantils, String estat)
        {
            Empresa empresa = new Empresa();
            empresa.NIF = nIF;
            empresa.Nom_fiscal = nom_fiscal;
            empresa.Nom_comercial = nom_comercial;
            empresa.Telefon1 = telefon1;
            empresa.Telefon2 = telefon2;
            empresa.Telefon3 = telefon3;
            empresa.Fax = fax;
            empresa.Correu_electronic = correu_electronic;
            empresa.Direccio_web = direccio_web;
            empresa.Direccio_empresa = direccio_empresa;
            empresa.Codpos_empresa = codpos_empresa;
            empresa.Poblacio = poblacio;
            empresa.Provincia = provincia;
            empresa.Dades_mercantils = dades_mercantils;
            empresa.Comentaris = comentaris;
            empresa.Nom_gerent = nom_gerent;
            empresa.Altra_informacio = altra_info;
            empresa.Estat = estat;
            return empresa;
        }

        public static void Update(String nIF, String nom_fiscal, String nom_comercial, String telefon1, string telefon2, string telefon3, string fax, String correu_electronic, String direccio_web, String direccio_empresa, Int32 codpos_empresa, String poblacio, String provincia, string nom_gerent, string comentaris, string altra_info, string dades_mercantils, String estat)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateEmpresa(nIF, nom_fiscal, nom_comercial, telefon1, telefon2, telefon3, fax, correu_electronic, direccio_web, direccio_empresa, codpos_empresa, poblacio, provincia, nom_gerent,comentaris,altra_info,dades_mercantils, estat, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

		public override  string ToString(){
			return this.Nom_comercial;
		}
	}
	
}