﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
//using XiquiparcsReunits_RDA.XiquiparcsReunits_DAO;
using XiquiparcsReunits_RDA.Tools;

using System.Collections.Generic;
using System.Windows;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.Model
{   
    /// <summary>
    /// Objecte que registra els bens o serveis oferts a un client en un periode determinat. Es tanca la facturació, un cop
    /// el client ha pagat la totalitat de l'import debengat. En aquest estat no es podrà modificar.
    /// També registra reserves futures, ja que es factures, i passen a estat de cobrament pendent.
    /// </summary>
	public class Factura: INotifyPropertyChanged
	{
		private string _usuari_factura;
		private string _id_factura;
		private string _estat_cobrament;
		private DateTime _data_cobrament;
		private string _descripcio;
        private Decimal _a_compte;
		private bool _modificable;

		public string Id_usuari_factura  {
			get {return _usuari_factura;}
			set {
				_usuari_factura=value;
				OnPropertyChanged("Usuari_factura");
				}
		}

        public string Nom_usuari
        {
            get
            {
                if (client == null) return "";
                else
                    return Usuari.GetInstancia(Id_usuari_factura).Nom_usuari;
            }
        }

        public Usuari Usuari_factura
        {
            get {
                return Usuari.GetInstancia(Id_usuari_factura);
                }
            set
            {
                Id_usuari_factura = value.Id_usuari;
                OnPropertyChanged("Usuari_factura");
            }
        }

        //Si assignem un id a la factura, podem llegir de la persistència les seus recursos facturats
		public string Id_factura {
			get {return _id_factura;}
			set {
				_id_factura=value;
                recursos = Recurs_facturat.GetInstanciesByFactura(Id_factura);
                OnPropertyChanged("Recursos_facturats");
                OnPropertyChanged("Id_factura");
                OnPropertyChanged("FacturaCreada");
				}
		}

		public string Estat_cobrament {
			get {return _estat_cobrament;}
			set {
				_estat_cobrament=value;
				OnPropertyChanged("Estat_cobrament");
				}
		}

		public DateTime Data_cobrament {
			get {return _data_cobrament;}
			set {
				_data_cobrament=value;
				OnPropertyChanged("Data_cobrament");
				}
		}

		public string Descripcio {
			get {return _descripcio;}
			set {
				_descripcio=value;
				OnPropertyChanged("Descripcio");
				}
		}
		public bool Modificable {
			get {return _modificable;}
			set {
				_modificable=value;
				OnPropertyChanged("Modificable");
                OnPropertyChanged("FacturaCreada");
				}
		}

        private GestorXiquiparc.GestorXiquiparcClient client;

        public int contador_recursos
        {
            get
            {
                return Recursos_facturats.Count;
            }
        }

        private ObservableCollection<Recurs_facturat> recursos;
        /// <summary>
        /// Variable que conté tots els recursos facturats d'aquesta factura
        /// </summary>
        public ObservableCollection<Recurs_facturat> Recursos_facturats
        {
            get
            {
                return recursos;
            }
            set
            {
                recursos = value;
                OnPropertyChanged("Recursos_facturats");
            }
        }

        public Decimal A_compte
        {
            get { return _a_compte; }
            set
            {
                _a_compte = value;
                OnPropertyChanged("A_compte");
                OnPropertyChanged("Pendent");
            }
        }

        public Decimal Pendent
        {
            get
            {
                return Import_total - A_compte;
            }
        }

        public Decimal Base_imponible
        {
            get
            {
                Decimal import = 0;
                foreach (Recurs_facturat recurs_facturat in this.Recursos_facturats)
                    import = import + (recurs_facturat.Import * recurs_facturat.Quantitat);
                return import;
            }
        }

        public Decimal Impostos
        {
            get { return Import_total - Base_imponible; }
        }

        public Decimal Import_total
        {
            get
            {
                Decimal import = 0;
                foreach (Recurs_facturat recurs_facturat in this.Recursos_facturats)
                    import = import + (recurs_facturat.Import * recurs_facturat.Quantitat * (1 + recurs_facturat.IVA_recurs));
                return import;
            }
        }
		
        /// <summary>
        /// Propietat que representa si una factura es modificable i té identificador,
        /// o sigui, si se li poden afegir recursos facturats
        /// </summary>
        #region ViewModelProperty: FacturaCreada
        public bool FacturaCreada
        {
            get
            {
                return Modificable && Id_factura != null;
            }
        }
        #endregion

        private Factura()
        {
        }

        public Factura(string id_factura){
            recursos = new ObservableCollection<Recurs_facturat>();
            client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.GetFactura(id_factura, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
       }

				
		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
		
		/// <summary>
		/// Funció que retorna un llistat de totes les factures
		/// </summary>
		/// <returns>Llista de totes les factures</returns>
		public static ObservableCollection<Factura> GetInstancies(){
            ObservableCollection<Factura> llista = new ObservableCollection<Factura>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Factura it in client.GetFactures(SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
            {
                llista.Add(Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio));
            }
            return llista;
		}

        /// <summary>
        /// Funció que retorna un llistat de les factures d'un usuari
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari de les factures</param>
        /// <returns>Llista de factures buscada</returns>
        public static ObservableCollection<Factura> GestInstanciesByUsuari(string id_usuari)
        {
            ObservableCollection<Factura> llista = new ObservableCollection<Factura>(); 
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Factura it in client.GetFacturesByUsuari(id_usuari, SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
            {
                llista.Add(Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio ));
            }
            return llista;
        }

        /// <summary>
        /// Funció que retorna un llistat de les factures d'un usuari en un estat de cobrament concret
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari de les factures</param>
        /// <param name="estat_cobrament">Estat de cobrament de les factures</param>
        /// <returns>Llista de factures buscada</returns>
        public static ObservableCollection<Factura> GestInstanciesByUsuariAndCobrament(string id_usuari, string estat_cobrament)
        {
            ObservableCollection<Factura> llista = new ObservableCollection<Factura>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Factura it in client.GetFacturesByUsuariAndCobrament(id_usuari, estat_cobrament, SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
            {
                llista.Add(Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio));
            }
            return llista;
        }

        /// <summary>
        /// Funció que retorna un llistat de les factures en un estat de cobrament concret
        /// </summary>
        /// <param name="estat_cobrament">Estat de cobrament de les factures</param>
        /// <returns>Llista de factures buscada</returns>
        public static ObservableCollection<Factura> GestInstanciesByCobrament(string estat_cobrament)
        {
            ObservableCollection<Factura> llista = new ObservableCollection<Factura>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Factura it in client.GetFacturesByCobrament(estat_cobrament, SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
            {
                llista.Add(Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio));
            }
            return llista;
        }

        /// <summary>
        /// Funció que insereix a la BBDD una nova factura, i retorna el identificador automàtic assignat a la BBDD
        /// </summary>
        /// <param name="factura">Factura amb els paràmetres inicialitzats</param>
        /// <returns>Identificador automàtic de la factura a la BBDD</returns>
        public static string Insert(Factura factura)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            return client.CreateFactura(factura.Id_usuari_factura, factura.Estat_cobrament, factura.Data_cobrament, factura.Descripcio, factura.A_compte, factura.Modificable, SingletonLoggedUser.Instance.LoggedUser.Id_usuari).id_factura;
		}
		
        /// <summary>
        /// Esborra una factura de la BBDD a partir del seu identificador
        /// </summary>
        /// <param name="id_factura">Identificador de la factura a esborrar</param>
		public static void Remove(string id_factura){
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.DeleteFactura(id_factura, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}

        /// <summary>
        /// Actualitza una factura ja existent a la BBDD
        /// </summary>
        /// <param name="factura">Factura a actualitzar amb els seus nous paràmetres</param>
        public static void Update(Factura factura)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateFactura(factura.Id_factura, factura.Modificable, factura.Estat_cobrament, factura.Data_cobrament, factura.A_compte, factura.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        /// <summary>
        /// Obé una factura a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de la factura buscada</param>
        /// <returns>Factura buscada</returns>
		public static Factura GetInstancia(string id)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.Factura it = client.GetFactura(id, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            return Factura.CreateFactura(it.id_factura,it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio);
        }
				
        /// <summary>
        /// Funció que retorna un usuari inicialitzant els mínims paràmetres
        /// </summary>
        /// <param name="id_factura">Identificador de la factura (no es comprova)</param>
        /// <param name="id_usuari_factura">Identificador de l'usuari (no es comprova)</param>
        /// <param name="estat_cobrament">Identifciador de l'estat de cobrament (no es comprova)</param>
        /// <param name="modificable">Estat de la factura, que permet modificar-la</param>
        /// <param name="a_compte">Import que l'usuari a donat a compte</param>
        /// <param name="data_cobrament">Data en que s'ha produït el darrer cobrament</param>
        /// <param name="descripcio">Descripcio de la factura</param>
        /// <returns>Factura creada</returns>
	    public static Factura CreateFactura(String id_factura, String id_usuari_factura, String estat_cobrament, bool modificable, decimal a_compte, DateTime data_cobrament, String descripcio)
        {
            Factura factura = new Factura();
            factura.Id_factura = id_factura;
			factura.Id_usuari_factura=id_usuari_factura;
            factura.Estat_cobrament = estat_cobrament;
			factura.Modificable=modificable;
            factura.A_compte = a_compte;
            factura.Descripcio = descripcio;
            factura.Data_cobrament = data_cobrament;
            return factura;
        }
		
		/// <summary>
		/// Funció que retorna un identificador de la factura
		/// </summary>
		/// <returns>Cadena que representa a una factura</returns>
		public  override string ToString(){
			return this.Id_factura;
		}
	}
}