﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_RDA.Tools;
using System.ServiceModel;
using System.ServiceModel.Channels;


namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Objecte que representa una promoció sobre un recurs. S'oferirà un determinat recurs en un tram de temps a un preu
    /// determinat, i en una quantitat limitada.
    /// </summary>
	public class Oferta: INotifyPropertyChanged
	{
		public string Id_oferta {get;set;}
		public string Id_recurs {get;set;}
		public Decimal Import_unitari {get;set;}
        public String Descripcio { get; set; }
		public int Quantitat_ofertada {get;set;}
		public DateTime Data_inici_oferta{get;set;}
		public DateTime Data_fi_oferta{get;set;}
        			
		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
		
        /// <summary>
        /// Obté un llistat de totes les ofertes del sistema
        /// </summary>
        /// <returns>Llistat buscat</returns>
        public static ObservableCollection<Oferta> GetInstancies(){
            ObservableCollection<Oferta> llista = new ObservableCollection<Oferta>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Oferta it in client.GetOfertes())
            {
                llista.Add(Oferta.CreateOferta(it.id_oferta, it.import_unitari, it.id_recurs, it.quantitat_ofertada, it.data_inici_oferta, it.data_fi_oferta, it.descripcio));
            }
            return llista;
		}

        /// <summary>
        /// Obté un llistat de totes les ofertes del sistema d'un determinat recurs
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <returns>Llista de ofertes del recurs</returns>
        public static ObservableCollection<Oferta> GetInstancies(string id_recurs)
        {
            ObservableCollection<Oferta> llista = new ObservableCollection<Oferta>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Oferta it in client.GetOfertesByRecurs(id_recurs))
            {
                llista.Add(Oferta.CreateOferta(it.id_oferta, it.import_unitari, it.id_recurs, it.quantitat_ofertada, it.data_inici_oferta, it.data_fi_oferta, it.descripcio));
            }
            return llista;
        }
		
        /// <summary>
        /// Obté una instancia a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de l'oferta</param>
        /// <returns>Oferta buscada</returns>
		public static Oferta GetInstancia(string id)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.Oferta it = client.GetOferta(id);
            return Oferta.CreateOferta(it.id_oferta,it.import_unitari, it.id_recurs, it.quantitat_ofertada, it.data_inici_oferta, it.data_fi_oferta, it.descripcio);
        }
        		
		/// <summary>
		/// Funció que insereix una nova oferta a la BBDD i retorna el identificador automàtic que se li ha assignat
		/// </summary>
		/// <param name="oferta">Oferta a inserir</param>
		/// <returns>Identificador automàtic de l'oferta a la BBDD</returns>
		public static string Insert(Oferta oferta){
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            return client.CreateOferta(oferta.Import_unitari, oferta.Id_recurs, oferta.Quantitat_ofertada, oferta.Data_inici_oferta, oferta.Data_fi_oferta, SingletonLoggedUser.Instance.LoggedUser.Id_usuari).id_oferta;
        }
		
        /// <summary>
        /// Elimina una oferta de la BBDD a partir del seu identificador
        /// </summary>
        /// <param name="id_oferta">Identificador de l'oferta a esborrar</param>
        public static void Remove(string id_oferta)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.DeleteOferta(id_oferta, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        /// <summary>
        /// Actualitza una oferta a la BBDD
        /// </summary>
        /// <param name="oferta">Oferta a actualitzar amb els paràmetres nous</param>
        public static void Update(Oferta oferta)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateOferta(oferta.Id_oferta, oferta.Import_unitari, oferta.Id_recurs, oferta.Quantitat_ofertada, oferta.Data_inici_oferta, oferta.Data_fi_oferta, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        /// <summary>
        /// Crea una oferta (fàbrica) a partir de tots els paràmetres. No la inserta a la BBDD.
        /// </summary>
        /// <param name="id_oferta">Identificador de la oferta</param>
        /// <param name="import_unitari">Import unitari del recurs en oferta</param>
        /// <param name="id_recurs">Identificador del recurs en oferta</param>
        /// <param name="quantitat_ofertada">Quantitat de recurs ofertat</param>
        /// <param name="data_inici_oferta">Data i hora d'inici de l'oferta</param>
        /// <param name="data_fi_oferta">Data i hora de fi de l'oferta</param>
        /// <param name="descripcio">Descripció de l'oferta</param>
        /// <returns>Oferta creada</returns>
		public static Oferta CreateOferta(String id_oferta, Decimal import_unitari, String id_recurs, Int32 quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, string descripcio)
        {
            Oferta oferta = new Oferta();
            oferta.Id_oferta = id_oferta;
            oferta.Descripcio = descripcio;
            oferta.Import_unitari = import_unitari;
            oferta.Id_recurs = id_recurs;
            oferta.Quantitat_ofertada = quantitat_ofertada;
            oferta.Data_inici_oferta = data_inici_oferta;
            oferta.Data_fi_oferta = data_fi_oferta;
            return oferta;
        }
		
		/// <summary>
		/// Funció que retorna una cadena identificador de l'oferta
		/// </summary>
		/// <returns>Cadena identificadora de l'oferta</returns>
		public override  string ToString(){
			return this.Id_oferta;
		}
	}
}