﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_RDA.Tools;

using System.Windows;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Objecte que representa un bé o servei que un client pot contractar o consumir
    /// </summary>
	public class Recurs: INotifyPropertyChanged
	{
		private string IVA {get; set;}
		public string Id_recurs {get;set;}
		public string Id_seccio {get;set;}
		public Decimal Preu {get;set;}
		public string Nom_IVA 
        {
            get{ return IVA;}
            set{ IVA = (value);}
        }

		public int Quantitat {get;set;}
        public string Descripcio { get; set; }
		public bool Disponibilitat_controlable {get;set;}
        
		public Decimal Percentatge_IVA { get {return TipusIVA.GetInstancia(IVA).Percentatge;}}

		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}	
		
		/// <summary>
		/// Funció que retorna tots els recursos del sistema
		/// </summary>
		/// <returns>Llistat de tots els recursos del sistema</returns>
		public static ObservableCollection<Recurs> GetInstancies(){
            ObservableCollection<Recurs> llista = new ObservableCollection<Recurs>(); GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Recurs it in client.GetRecursos())
            {
                llista.Add(Recurs.CreateRecurs(it.id_recurs, it.id_seccio, it.preu, it.nom_IVA, it.quantitat, it.disponibilitat_controlable, it.descripcio));
            }
            return llista;
		
		}

        /// <summary>
        /// Funció que retorna tots els recursos del sistema d'una secció
        /// </summary>
        /// <returns>Llistat de tots els recursos d'una secció</returns>
        public static ObservableCollection<Recurs> GetInstancies(string id_seccio)
        {
            try
            {
                ObservableCollection<Recurs> llista = new ObservableCollection<Recurs>(); 
                GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
                OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
                MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

                messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
                messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

                foreach (GestorXiquiparc.Recurs it in client.GetRecursosBySeccio(id_seccio))
                {
                    llista.Add(Recurs.CreateRecurs(it.id_recurs, it.id_seccio, it.preu, it.nom_IVA, it.quantitat, it.disponibilitat_controlable, it.descripcio));
                }
                return llista;
            }
            catch (Exception e)
            {
                MessageBox.Show("Error al crear la llista de recursos", "Error de cerca de recursos", MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.ToString());
                return null;
            }

        }

        /// <summary>
        /// Funció que retorna un llistat del noms de tots els recursos
        /// </summary>
        /// <returns>Llistat de noms de tots els recursos</returns>
        public static ObservableCollection<string> GetNomInstancies()
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            return new ObservableCollection<string>(client.GetNomRecursos());
        }

        /// <summary>
        /// Funció que retorna un recurs a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador del recurs buscat</param>
        /// <returns>Recurs buscat</returns>
		public static Recurs GetInstancia(string id)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.Recurs recurs = client.GetRecurs(id);
            return Recurs.CreateRecurs(recurs.id_recurs, recurs.id_seccio, recurs.preu, recurs.nom_IVA, recurs.quantitat, recurs.disponibilitat_controlable, recurs.descripcio);
        }

	    /// <summary>
	    /// Funció que crea un nou recurs a partir de tots els paràmetres
	    /// </summary>
	    /// <param name="id_recurs">Identificador del recurs</param>
	    /// <param name="id_seccio">Identificador de la secció</param>
	    /// <param name="preu">Preu actual del recurs</param>
	    /// <param name="nom_IVA">Nom de l'IVA actual a aplicar</param>
	    /// <param name="quantitat">Quantitat de recurs total</param>
	    /// <param name="disponibilitat_controlable">Booleà que mostra si es controlable la disponibilitat</param>
	    /// <param name="descripcio">Descripció del recurs</param>
	    /// <returns>Recurs creat</returns>
		public static Recurs CreateRecurs(String id_recurs, String id_seccio, Decimal preu, String nom_IVA, Int32 quantitat, Boolean disponibilitat_controlable, string descripcio)
        {
            Recurs recurs = new Recurs();
            recurs.Id_recurs = id_recurs;
            recurs.Id_seccio = id_seccio;
            recurs.Preu = preu;
            recurs.Nom_IVA = nom_IVA;
            recurs.Quantitat = quantitat;
            recurs.Descripcio = descripcio;
            recurs.Disponibilitat_controlable = disponibilitat_controlable;
            return recurs;
        }
		
        /// <summary>
        /// Funció que insereix un recurs al sistema
        /// </summary>
        /// <param name="recurs">Recurs a inserir</param>
        public static void Insert(Recurs recurs)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.CreateRecurs(recurs.Id_recurs, recurs.Id_seccio, recurs.Preu, recurs.Nom_IVA, recurs.Quantitat, recurs.Disponibilitat_controlable, recurs.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}

        /// <summary>
        /// Funció que actualitza un recurs ja existent
        /// </summary>
        /// <param name="recurs">Recurs a actualitzar</param>
        public static void Update(Recurs recurs)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateRecurs(recurs.Id_recurs, recurs.Id_seccio, recurs.Preu, recurs.Nom_IVA, recurs.Quantitat, recurs.Disponibilitat_controlable, recurs.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }
		
        /// <summary>
        /// Funció que elimina un recurs del sistema
        /// </summary>
        /// <param name="id_recurs">Identificador del recurs a eliminar</param>
        public static void Remove(string id_recurs)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.DeleteRecurs(id_recurs, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}
		
		public override  string ToString(){
			return this.Id_recurs;
		}
		
	}
}