﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_RDA.Tools;

using System.Windows;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Objecte que representa la utilització d'un recurs en una factura però en un preu promocional
    /// </summary>
	public class Recurs_facturat: INotifyPropertyChanged
	{

		public String Id_factura {get;set;}

        private Recurs recurs { get; set; }
        public String Id_recurs
        {
            get
            {
                if (recurs == null)
                    return null;
                else
                    return recurs.Id_recurs;
            }
            set
            {
                if (value != null)
                {
                    recurs = Recurs.GetInstancia(value);
                    Import = recurs.Preu;
                    Percentatge_IVA = recurs.Percentatge_IVA;
                }
                else
                    recurs = null;
                OnPropertyChanged("Id_recurs");
                OnPropertyChanged("Descripcio_recurs");
                OnPropertyChanged("Recurs_controlable");
                OnPropertyChanged("Percentatge_IVA");
                OnPropertyChanged("Import");
            }
        }

        private Decimal import;
        public Decimal Import 
        { 
            get 
            { 
                return import; 
            }
            set 
            { 
                import=value; OnPropertyChanged("Import");
            } 
        }

        private int quantitat=0;
        public Int32 Quantitat
        {
            get
            {
                return quantitat;
            }
            set
            {
                quantitat = value; OnPropertyChanged("Quantitat"); OnPropertyChanged("Import_total");
            }
        }

        private Decimal percentatge_IVA=0;
        public Decimal Percentatge_IVA
        {
            get
            {
                return percentatge_IVA;
            }
            set
            {
                percentatge_IVA = value; OnPropertyChanged("Percentatge_IVA");
            }
        }

        private DateTime data_inici_servei = DateTime.Now;
        public DateTime Data_inici_servei { get { return data_inici_servei; } set { data_inici_servei = value; OnPropertyChanged("Data_inici_servei"); OnPropertyChanged("Import_total"); } }

        private DateTime data_fi_servei = DateTime.Now.AddHours(1);
        public DateTime Data_fi_servei { get { return data_fi_servei; } set { data_fi_servei = value; OnPropertyChanged("Data_fi_servei"); OnPropertyChanged("Import_total"); } }

        public string Descripcio_recurs { get { return recurs.Descripcio; } }

        public Decimal IVA_recurs { get { return recurs.Percentatge_IVA; } }

        //L'import total depen de si es un recurs amb disponibilitat controlable, es a dir, si consumim un refresc
        //el import total es del recurs, però si consumim una sala (o 2 o 3), l'import serà el preu per el nombre d'hores utilitzada
        public Decimal Import_total
        { 
            get 
            {
                //Si es tracta d'un recurs de disponibilitat controlable
                if (this.Recurs_controlable)
                    return Import * (Data_fi_servei - data_inici_servei).Hours  * Quantitat * (1 + Percentatge_IVA); 
                else
                    return Import * Quantitat * (1 + Percentatge_IVA); 
            } 
        }

        public bool Recurs_controlable { get { if (recurs == null) return false; else return recurs.Disponibilitat_controlable; } }

		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
		
		public static ObservableCollection<Recurs_facturat> GetInstancies()
        {
            ObservableCollection<Recurs_facturat> list = new ObservableCollection<Recurs_facturat>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Recurs_facturat it in client.GetRecursosFacturats())
            {
                list.Add(Recurs_facturat.CreateRecurs_facturat(it.id_factura, it.id_recurs, it.import, it.quantitat, it.percentatge_IVA, it.data_inici_servei, it.data_fi_servei));
            }
            return list;
		}
		
		public static Recurs_facturat GetInstancia(string id_factura, string id_recurs)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

                GestorXiquiparc.Recurs_facturat it = client.GetRecursFacturat(id_factura, id_recurs);
                return Recurs_facturat.CreateRecurs_facturat(it.id_factura, it.id_recurs, it.import, it.quantitat, it.percentatge_IVA, it.data_inici_servei, it.data_fi_servei);
        }
		
		public static ObservableCollection<Recurs_facturat> GetInstanciesByFactura(string id_factura)
        {
            ObservableCollection<Recurs_facturat> list = new ObservableCollection<Recurs_facturat>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Recurs_facturat it in client.GetRecursosFacturatsByFactura(id_factura))
            {
                list.Add(Recurs_facturat.CreateRecurs_facturat(it.id_factura, it.id_recurs, it.import, it.quantitat, it.percentatge_IVA, it.data_inici_servei, it.data_fi_servei));
            }
            return list;
        }
		
		public static ObservableCollection<Recurs_facturat> GetInstanciesByRecurs(string id_recurs)
        {
            ObservableCollection<Recurs_facturat> list = new ObservableCollection<Recurs_facturat>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            foreach (GestorXiquiparc.Recurs_facturat it in client.GetRecursosFacturatsByRecurs(id_recurs))
            {
                list.Add(Recurs_facturat.CreateRecurs_facturat(it.id_factura, it.id_recurs, it.import, it.quantitat, it.percentatge_IVA, it.data_inici_servei, it.data_fi_servei));
            }
            return list;
        }
		
		public static Recurs_facturat CreateRecurs_facturat(String id_factura, String id_recurs, Decimal import, Int32 quantitat, Decimal percentatge_IVA, DateTime data_inici_servei, DateTime data_fi_servei)
        {
            Recurs_facturat recurs_facturat = new Recurs_facturat();
            recurs_facturat.Id_factura = id_factura;
            recurs_facturat.Id_recurs = id_recurs;
            recurs_facturat.Import = import;
            recurs_facturat.Quantitat = quantitat;
            recurs_facturat.Percentatge_IVA = percentatge_IVA;
            recurs_facturat.Data_inici_servei = data_inici_servei;
            recurs_facturat.Data_fi_servei = data_fi_servei;
            return recurs_facturat;
        }
		
		public static void Insert(Recurs_facturat recurs){
            ObservableCollection<Recurs_facturat> list = new ObservableCollection<Recurs_facturat>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.CreateRecursFacturat(recurs.Id_factura, recurs.Id_recurs, recurs.Quantitat, recurs.Data_inici_servei, recurs.Data_fi_servei, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}
		
		public static void Remove(string id_factura, string id_recurs){
            ObservableCollection<Recurs_facturat> list = new ObservableCollection<Recurs_facturat>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
            client.DeleteRecursFacturat(id_factura, id_recurs, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}
		
		
		public override  string ToString(){
			return this.Id_recurs + " a " + this.Id_factura;
		}
		
	}
}