﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
//using XiquiparcsReunits_DAO;
using XiquiparcsReunits_RDA.Tools;
using System.ServiceModel.Channels;
using System.ServiceModel;


namespace XiquiparcsReunits_RDA.Model
{
    /// <summary>
    /// Classe que defineix un tipus d'iva que s'aplica a les factures
    /// </summary>
	public class TipusIVA: INotifyPropertyChanged
	{
		public String Nom_IVA{get;set;}
		public Decimal Percentatge{get;set;}
        public string Descripcio { get; set; }
        			
		public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Constructor privat per evitar la seva crida
        /// </summary>
        private TipusIVA() { }

		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

        /// <summary>
        /// Funció que inicialitza la acumulació de tipus d'IVA
        /// </summary>
        /// <returns>Llistat amb tots els tips d'IVA</returns>
		private static ObservableCollection<TipusIVA> initInstancies(){
			ObservableCollection<TipusIVA> listIVA=new ObservableCollection<TipusIVA>();
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

			foreach (GestorXiquiparc.TipusIVA it in client.GetTipusIVA()){
            		listIVA.Add (TipusIVA.CreateTipusIVA(it.nom_IVA, it.percentatge, it.observacions));
			}
			return listIVA;
		}
		
        /// <summary>
        /// Llegeix un llistat de totes les instacies de la persistència
        /// </summary>
        /// <returns>LListat d'objectes cercat</returns>
        public static ObservableCollection<TipusIVA> GetInstancies(){
            return initInstancies();
		}

        /// <summary>
        /// Funció que llegeix un objecte de la persistència a partir del seu id
        /// </summary>
        /// <param name="id">Identificador de la instacia cercada</param>
        /// <returns>Objecte cercat</returns>
		public static TipusIVA GetInstancia(string id)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.TipusIVA it = client.GetOneTipusIVA(id);
            return TipusIVA.CreateTipusIVA(it.nom_IVA, it.percentatge, it.observacions);
        }

        /// <summary>
        /// Crea un nou objecte sense registar-lo a la persistència
        /// </summary>
        /// <param name="nom_IVA">Nom del nou tipus d'IVA</param>
        /// <param name="percentatge">Percentatge d'impost del nou tipus d'IVA</param>
        /// <param name="descripcio">Descripció del tipus d'IVA</param>
        /// <returns>Objecte creat a partir dels paràmetres</returns>
	    public static TipusIVA CreateTipusIVA(String nom_IVA, Decimal percentatge, string descripcio)
        {
            TipusIVA tipusIVA = new TipusIVA();
            tipusIVA.Nom_IVA = nom_IVA;
            tipusIVA.Percentatge = percentatge;
            tipusIVA.Descripcio = descripcio;
            return tipusIVA;
        }

        /// <summary>
        /// Insereix un nou objecte a la persistencia
        /// </summary>
        /// <param name="iva">Tipus d'objecte a inserir</param>
		public static void Insert(TipusIVA iva){
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.CreateTipusIVA(iva.Nom_IVA, iva.Percentatge, iva.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
		}

        /// <summary>
        /// Modifica un objecte de la persistencia
        /// </summary>
        /// <param name="iva">Objecte a modificar</param>
        public static void Update(TipusIVA iva)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.UpdateTipusIVA(iva.Nom_IVA, iva.Percentatge, iva.Descripcio, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }
		
        /// <summary>
        /// Esborra un objecte de la persistencia
        /// </summary>
        /// <param name="nom_iva">Identificador de l'objecte a esborrar</param>
        public static void Remove(string nom_iva)
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            client.DeleteTipusIVA(nom_iva, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }
		
        /// <summary>
        /// Retorna una frase indentificador de l'objecte
        /// </summary>
        /// <returns></returns>
		public override string ToString(){
			return this.Nom_IVA;
		}
	}
}