﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Generic;

using System.Windows;
using XiquiparcsReunits_RDA.Tools;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.Model
{
	public class Usuari: INotifyPropertyChanged
	{
        private static GestorXiquiparc.GestorXiquiparcClient client;

        private string id_usuari;
        public string Id_usuari
        {
            get
            { return id_usuari; }
            set
            { id_usuari = value; OnPropertyChanged("Id_usuari"); }
        }

        private string nom_usuari;
        public string Nom_usuari
        {
            get
            { return nom_usuari; }
            set
            { nom_usuari = value; OnPropertyChanged("Nom_usuari"); }
        }

        private string nif_usuari;
        public string NIF_usuari
        {
            get
            { return nif_usuari; }
            set
            { nif_usuari = value; OnPropertyChanged("NIF_usuari"); }
        }

        private string nom_fiscal_usuari;
        public string Nom_fiscal_usuari
        {
            get
            { return nom_fiscal_usuari; }
            set
            { nom_fiscal_usuari = value; OnPropertyChanged("Nom_fiscal_usuari"); }
        }
        
        public string Telefon1 { get; set; }
		public string Telefon2 {get;set;}
		public string Movil {get;set;}
		public string Fax {get;set;}
		public string Direccio {get;set;}

        private string poblacio;
        public string Poblacio
        {
            get
            { return poblacio; }
            set
            { poblacio = value; OnPropertyChanged("Poblacio"); }
        }

		public int Codpos {get;set;}

        private string provincia;
        public string Provincia
        {
            get
            { return provincia; }
            set
            { provincia = value; OnPropertyChanged("Provincia"); }
        }

		public string Estat {get;set;}
        
        private string MD5Pwd;
        /// <summary>
        /// Propietat que registra el password de l'usuari, sempre es guarda en format MD5
        /// </summary>
        public string Pwd { get { return MD5Pwd; } set { MD5Pwd = value; OnPropertyChanged("Pwd"); } }
		
        private string observacions;
        public string Observacions
        {
            get
            { return observacions; }
            set
            { observacions = value; OnPropertyChanged("Observacions"); }
        }

        private string nom_rol;
        public string Nom_rol
        {
            get
            { return nom_rol; }
            set
            { nom_rol = value; OnPropertyChanged("Nom_rol"); }
        }
        			
		public event PropertyChangedEventHandler PropertyChanged;
        
        /// <summary>
        /// Event captat en la modificació d'una propietat de l'usuari
        /// </summary>
        /// <param name="name">Nom de la propietat modificada</param>
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
       		
        /// <summary>
        /// Funció que retorna un llistat de tots els usuaris actius del sistema
        /// </summary>
        /// <returns>Llistat amb tots els usuaris</returns>
		public static ObservableCollection<Usuari> GetInstancies(){
            client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            ObservableCollection<Usuari> list = new ObservableCollection<Usuari>();
            foreach (GestorXiquiparc.Usuari it in client.GetUsuaris(SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
            {
                list.Add(Usuari.CreateUsuari(it.id_usuari, it.nom_usuari, it.NIF_usuari, it.nom_fiscal_usuari, it.telefon1, it.telefon2, it.movil, it.direccio, it.poblacio, it.codpos, it.provincia, it.estat, it.pwd, it.observacions, it.nom_rol));
            }
            return list;
        }

        /// <summary>
        /// Funció que retorna un llistat d'usuaris filtrats per un rol
        /// </summary>
        /// <param name="rol">Rol a filtrar el llistat d'usuaris</param>
        /// <returns>Llistat d'usuaris amb el rol filtrat</returns>
        public static ObservableCollection<Usuari> GetInstancies(Rol rol)
        {
            client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            ObservableCollection<Usuari> list = new ObservableCollection<Usuari>();
            List<GestorXiquiparc.Usuari> usuaris = new List<GestorXiquiparc.Usuari>(client.GetUsuarisByRol(rol.Nom_rol, SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            foreach (GestorXiquiparc.Usuari it in usuaris)
            {
                list.Add(Usuari.CreateUsuari(it.id_usuari, it.nom_usuari, it.NIF_usuari, it.nom_fiscal_usuari, it.telefon1, it.telefon2, it.movil, it.direccio, it.poblacio, it.codpos, it.provincia, it.estat, it.pwd, it.observacions, it.nom_rol));
            }
            return list;
        }

        /// <summary>
        /// Funció que a partir d'un nom d'usuari retorna el identificador de la persistència
        /// </summary>
        /// <param name="nom">Nom del usuari</param>
        /// <returns>Identificador de l'usuari</returns>
        public static string GetIdUsuari(string nom)
        {
            //Aquesta consulta no necessita seguretat
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            return client.GetIdUsuari(nom);
        }

        /// <summary>
        /// Funció que obté de la BBDD un usuari a partir del seu identificador
        /// </summary>
        /// <param name="id">Identificador de l'usuari a cercar</param>
        /// <returns>Usuari buscat</returns>
        public static Usuari GetInstancia(string id)
        {
            client = new GestorXiquiparc.GestorXiquiparcClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));

            GestorXiquiparc.Usuari it = client.GetUsuari(id, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            return Usuari.CreateUsuari(it.id_usuari, it.nom_usuari, it.NIF_usuari, it.nom_fiscal_usuari, it.telefon1, it.telefon2, it.movil, it.direccio, it.poblacio, it.codpos, it.provincia, it.estat, it.pwd, it.observacions, it.nom_rol);
        }

        /// <summary>
        /// Funció que obté de la BBDD un usuari a partir del seu nom d'usuari
        /// </summary>
        /// <param name="nom">Nom de l'usuari a cercar</param>
        /// <returns>Usuari buscat</returns>
        public static Usuari GetInstanciaByNom(string nom)
        {
            client = new GestorXiquiparc.GestorXiquiparcClient();
   
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            GestorXiquiparc.Usuari it = client.GetUsuariByNom(nom, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            return Usuari.CreateUsuari(it.id_usuari, it.nom_usuari, it.NIF_usuari, it.nom_fiscal_usuari, it.telefon1, it.telefon2, it.movil, it.direccio, it.poblacio, it.codpos, it.provincia, it.estat, it.pwd, it.observacions, it.nom_rol);
        }

        /// <summary>
        /// Constructor (fàbrica) d'usuaris amb tots els paràmetres. No l'inscriu a la BBDD
        /// </summary>
        /// <param name="nom_usuari">Nom d'usuari</param>
        /// <param name="nIF_usuari">NIF de l'usuari</param>
        /// <param name="nom_fiscal">Nom fiscal (real) de l'usuari</param>
        /// <param name="tel1">Telèfon 1 de l'usuari</param>
        /// <param name="tel2">Telèfon 2 de l'usuari</param>
        /// <param name="movil">Movil de l'usuari</param>
        /// <param name="direccio">Direcció postal de l'usuari</param>
        /// <param name="poblacio">Població postal de l'usuari</param>
        /// <param name="codpos">Codi postal de l'usuari</param>
        /// <param name="provincia">Provincia postal de l'usuari</param>
        /// <param name="estat">Estat postal de l'usuari</param>
        /// <param name="pwd">Codi d'accés de l'usuari</param>
        /// <param name="observacions">Observacions sobre l'usuari</param>
        /// <param name="nom_rol">Nom del rol de l'usuari</param>
        /// <returns>Usuari creat a partir dels paràmetres</returns>
		public static Usuari CreateUsuari(String nom_usuari, String nIF_usuari, String nom_fiscal, string tel1, string tel2, String movil, String direccio, String poblacio, Int32 codpos, String provincia, String estat, String pwd, string observacions, String nom_rol)
        {
            Usuari usuari = new Usuari();
            usuari.Nom_usuari = nom_usuari;
            usuari.NIF_usuari = nIF_usuari;
            usuari.Nom_fiscal_usuari = nom_fiscal;
            usuari.Movil = movil;
            usuari.Telefon1 = tel1;
            usuari.Telefon2 = tel2;
            usuari.Direccio = direccio;
            usuari.Poblacio = poblacio;
            usuari.Codpos = codpos;
            usuari.Provincia = provincia;
            usuari.Estat = estat;
            usuari.Pwd = pwd;
            usuari.Observacions = observacions;
            usuari.Nom_rol = nom_rol;
            return usuari;
        }

        /// <summary>
        /// Funció constructora de usuaris a partir de tots els paràmetres (fàbrica d'usuaris). No l'inscriu a la BBDD
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari</param>
        /// <param name="nom_usuari">Nom de l'usuari</param>
        /// <param name="nIF_usuari">NIF de l'usuari</param>
        /// <param name="nom_fiscal">Nom fiscal (real) de l'usuari</param>
        /// <param name="tel1">Telèfon 1 del client</param>
        /// <param name="tel2">Telèfon 2 del client</param>
        /// <param name="movil">Telèfon movil del client</param>
        /// <param name="direccio">Direcció postal del client</param>
        /// <param name="poblacio">Població postal del client</param>
        /// <param name="codpos">Codi postal del client</param>
        /// <param name="provincia">Provincia postal del client</param>
        /// <param name="estat">Estat postal del client</param>
        /// <param name="pwd">Codi d'accés del client</param>
        /// <param name="observacions">Observacions sobre el client</param>
        /// <param name="nom_rol">Rol del client</param>
        /// <returns>Usuari creat a partir dels paràmetres</returns>
        public static Usuari CreateUsuari(String id_usuari, String nom_usuari, String nIF_usuari, String nom_fiscal, String tel1, String tel2, String movil, String direccio, String poblacio, Int32 codpos, String provincia, String estat, String pwd, string observacions, String nom_rol)
        {
            Usuari usuari = CreateUsuari(nom_usuari, nIF_usuari, nom_fiscal, tel1, tel2, movil, direccio, poblacio, codpos, provincia, estat, pwd, observacions, nom_rol);
            usuari.Id_usuari = id_usuari;
            return usuari;
        }

		/// <summary>
		/// Funció que inserta a la BBDD un nou usuari i retorna l'identificador automàtic creat
		/// </summary>
		/// <param name="usuari">Usuari a inserir</param>
		/// <returns>Identificador automàtic de l'usuari a la bbdd</returns>
		public static string Insert(Usuari usuari){
            client = new GestorXiquiparc.GestorXiquiparcClient();

            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            return client.CreateUsuari(usuari.Nom_usuari, usuari.NIF_usuari, usuari.Nom_fiscal_usuari, usuari.Telefon1, usuari.Telefon2, usuari.Movil, usuari.Direccio, usuari.Poblacio, usuari.Codpos, usuari.Provincia, usuari.Estat, usuari.Pwd, usuari.Observacions, usuari.Nom_rol, SingletonLoggedUser.Instance.LoggedUser.Id_usuari).id_usuari;
        }
		
        /// <summary>
        /// Funció que elimina un usuari de la BBDD a partir del seu identificador
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari a esborrar</param>
		public static void  Remove(string id_usuari){
            client = new GestorXiquiparc.GestorXiquiparcClient();

            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            client.DeleteUsuari(id_usuari, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }

        /// <summary>
        /// Funció que actualitza les dades de la BBDD sobre un usuari ja existent
        /// </summary>
        /// <param name="usuari">Usuari amb els paràmetres a modificar</param>
        public static void Update(Usuari usuari)
        {
            client = new GestorXiquiparc.GestorXiquiparcClient();

            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
           
            client.UpdateUsuari(usuari.Id_usuari, usuari.Nom_usuari, usuari.NIF_usuari, usuari.Nom_fiscal_usuari, usuari.Telefon1, usuari.Telefon2, usuari.Movil, usuari.Direccio, usuari.Poblacio, usuari.Codpos, usuari.Provincia, usuari.Estat, usuari.Pwd, usuari.Observacions, usuari.Nom_rol, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
        }
		
        /// <summary>
        /// Funció que retorna el nom d'usuari de l'usuari
        /// </summary>
        /// <returns>Text amb el nom de l'usuari</returns>
		public override  string ToString(){
			return this.Nom_usuari;
		}

        /// <summary>
        /// Assigna un nou codi d'accés a l'usuari
        /// </summary>
        /// <param name="password">Nou codi d'accés a assignar</param>
        public void SetPwd(string password)
        {
            this.Pwd=password;
        }

        /// <summary>
        /// Funció que comprova un codi d'accés comparant-lo amb el del usuari actual
        /// </summary>
        /// <param name="password">Codi a comparar</param>
        /// <returns>Cert si ambdós codis coincideixen</returns>
        public bool TestPwd(string password)
        {
            return this.Pwd.Equals(password);
        }
	}
}