﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Model;
using XiquiparcsReunits_RDA.Tools;
using System;
using System.Windows;
using System.Windows.Controls;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarCobramentsViewModel : ViewModel
    {

        public ObservableCollection<Cobrament> Cobraments{get;set;}

        /// <summary>
        /// Propietat que registra el cobrament que s'ha de modificar
        /// </summary>
        #region ViewModelProperty: CobramentAModificar
        private Cobrament cobramentAModificar;
        public Cobrament CobramentAModificar
        {
            get
            {
                return cobramentAModificar;
            }
            set
            {
                cobramentAModificar = value;
                OnPropertyChanged("CobramentAModificar");
            }
        }
        #endregion

        public GestionarCobramentsViewModel(IEventAggregator messageBus)
        {
            try
            {
                Cobraments = Cobrament.GetInstancies();
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_cobraments, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                Cobraments = new ObservableCollection<Cobrament>();
                //System.Console.WriteLine(e.InnerException);
            }

            ElimCobramentCommand = new DelegateCommand<Cobrament>(ElimCobrament);
            RetornLlistaCobramentsCommand = new DelegateCommand<Cobrament>(RetornLlistaCobraments);
            RetornModificaCobramentCommand = new DelegateCommand<Cobrament>(RetornModificaCobrament);
            RetornCreaCobramentCommand = new DelegateCommand<Cobrament>(RetornCreaCobrament);
            this.messageBus = messageBus;

        }



        /// <summary>
        /// Funció que s'executa al triar un cobrament i voler modificar-lo
        /// </summary>
        #region DelegateCommand: RetornModificaCobrament
        public ICommand RetornModificaCobramentCommand { get; set; }
        private void RetornModificaCobrament(Cobrament cobrament)
        {
            cobramentAModificar = cobrament;
            Uri newUri = new Uri("/Views/Modifica_Tipus_Cobrament.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un cobrament accepta la finestra de confirmació de modificació
        /// </summary>
        #region DelegateCommand: ModifyCobrament
        private DelegateCommand modifyCobramentCommand;

        public ICommand ModifyCobramentCommand
        {
            get
            {
                if (modifyCobramentCommand == null)
                {
                    if (CobramentAModificar.Estat_cobrament == null)
                        modifyCobramentCommand = new DelegateCommand(ModifyCobrament, CanCreateCobrament);
                    else
                        modifyCobramentCommand = new DelegateCommand(ModifyCobrament, CanModifyCobrament);
                    
                }
                return modifyCobramentCommand;
            }
        }

        private bool isNew;

        private void ModifyCobrament()
        {
            if (isNew)
            {
                try
                {
                    Cobrament.Insert(CobramentAModificar);
                    Cobraments.Add(CobramentAModificar);
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_inserir_cobrament + CobramentAModificar.Estat_cobrament, Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                    //System.Console.WriteLine(e.InnerException);
                }
            }
            else
            {
                try
                {
                    Cobrament.Update(CobramentAModificar);
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_modificar_cobrament + CobramentAModificar.Estat_cobrament, Strings.Error_de_modificacio, MessageBoxButton.OK, MessageBoxImage.Error);
                    //System.Console.WriteLine(e.InnerException);
                }
            }
            modifyCobramentCommand = null;
            OnPropertyChanged("Cobraments"); 
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifyCobrament()
        {
            bool modifica=false;
            isNew = false;
            try
            {
                modifica= Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Cobrament.modifica;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return modifica;
        }

        private bool CanCreateCobrament()
        {
            bool modifica = false;
            isNew = true;
            try {
                modifica = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Cobrament.crea;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return modifica;
        }
        #endregion
        
        /// <summary>
        /// Funció que s'executa al voler crear un nou cobrament
        /// </summary>
        #region DelegateCommand: RetornCreaCobrament
        public ICommand RetornCreaCobramentCommand { get; set; }
        private void RetornCreaCobrament(Cobrament cobrament)
        {
            cobramentAModificar = Cobrament.CreateCobrament(null, "");
            Uri newUri = new Uri("/Views/Modifica_Tipus_Cobrament.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion
        
        /// <summary>
        /// Funció que elimina un cobrament, confirmant-ho previament
        /// </summary>
        #region DelegateCommand: ElimCobrament
        public ICommand ElimCobramentCommand { get; private set; }

        private void ElimCobrament(Cobrament obj)
        {
            CobramentAModificar = obj;
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarCobrament();
            finestra.Show();
        }

        public ICommand ConfirmaElimCobramentCommand
        {
            get
            {
                if (modifyCobramentCommand == null)
                {
                    modifyCobramentCommand = new DelegateCommand(ConfirmaElimCobrament, CanElimCobrament);
                }
                return modifyCobramentCommand;
            }
        }
        private void ConfirmaElimCobrament()
        {
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (CobramentAModificar != null)
            {
                try
                {
                    Cobrament.Remove(CobramentAModificar.Estat_cobrament);
                    Cobraments.Remove(cobramentAModificar);
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_esborrar_cobrament + CobramentAModificar.Estat_cobrament, Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                    //System.Console.WriteLine(e.InnerException);
                }
                
                cobramentAModificar = null;
                modifyCobramentCommand = null;
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimCobrament()
        {
            bool modifica = false;
            try{
                return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Cobrament.esborra;
             }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return modifica;
        }
        #endregion
        
        
    }
}