﻿using System.Windows.Input;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Tools;
using XiquiparcsReunits_RDA.Model;
using System;
using System.Collections.ObjectModel;

using System.Windows;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarEmpresaViewModel : ViewModel
    {
        public string Provincia 
        {
            get
            {
                return EmpresaAModificar.Provincia;
            }
            set 
            {

                EmpresaAModificar.Provincia = value;
                OnPropertyChanged("Provincia");
            }
        }

        /// <summary>
        /// Propietat que registra la empresa que s'ha de modificar
        /// </summary>
        #region ViewModelProperty: EmpresaAModificar
        private Empresa empresaAModificar = new Empresa();
        public Empresa EmpresaAModificar
        {
            get
            {

                return empresaAModificar;
            }
            set
            {
                empresaAModificar = value;
                OnPropertyChanged("EmpresaAModificar"); 
                OnPropertyChanged("Provincia");
            }
        }
        #endregion

        /// <summary>
        /// Constructor del ViewModel que getiona la empresa
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarEmpresaViewModel(IEventAggregator messageBus)
        {

            RetornModificaEmpresaCommand = new DelegateCommand<Empresa>(RetornModificaEmpresa);
            this.messageBus = messageBus;
            
        }


        /// <summary>
        /// Funció que s'executa al voler modifica l'empresa. Assigna a partir de la BBDD la empresa
        /// </summary>
        #region DelegateCommand: RetornModificaEmpresa
        public ICommand RetornModificaEmpresaCommand { get; set; }
        private void RetornModificaEmpresa(Empresa empresa)
        {
           
            Uri newUri = new Uri("/Views/Modifica_Empresa.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            try
            {
                empresaAModificar = new Empresa();
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_empresa, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan accepta la finestra de confirmació de modificació 
        /// </summary>
        #region DelegateCommand: ModifyEmpresa
        private DelegateCommand modifyEmpresaCommand;

        public ICommand ModifyEmpresaCommand
        {
            get
            {
                if (modifyEmpresaCommand == null)
                {
                    modifyEmpresaCommand = new DelegateCommand(ModifyEmpresa, CanModifyEmpresa);
                }
                return modifyEmpresaCommand;
            }
        }
        
        private void ModifyEmpresa()
        {
            try
            {
                Empresa.Update(EmpresaAModificar.NIF, EmpresaAModificar.Nom_fiscal, EmpresaAModificar.Nom_comercial, EmpresaAModificar.Telefon1,
                    EmpresaAModificar.Telefon2, EmpresaAModificar.Telefon3, EmpresaAModificar.Fax, EmpresaAModificar.Correu_electronic,
                    EmpresaAModificar.Direccio_web, EmpresaAModificar.Direccio_empresa, EmpresaAModificar.Codpos_empresa,
                    EmpresaAModificar.Poblacio, EmpresaAModificar.Provincia, EmpresaAModificar.Nom_gerent, EmpresaAModificar.Comentaris,
                    EmpresaAModificar.Altra_informacio, EmpresaAModificar.Dades_mercantils, EmpresaAModificar.Estat);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_empresa, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifyEmpresa()
        {
            bool modifica = false;
            try
            {
                modifica = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Modifica_empresa;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return modifica;
        }
        #endregion
     }
}