﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Model;
using System;
using System.Drawing.Printing;
using System.Collections;
using System.Windows.Controls;
using System.Printing;
using System.Windows;
using System.ServiceModel;
using XiquiparcsReunits_RDA.Tools;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarFacturesViewModel : ViewModel
    {

        /// <summary>
        /// Propietat que retorna el dia i hora actual
        /// </summary>
        #region ViewModelProperty: Now
        public DateTime Now { get { return DateTime.Now; } }
        #endregion


        /// <summary>
        /// Propietat que registra els recursos del sistema per poder assignar a una factura
        /// </summary>
        #region ViewModelProperty: Recursos
        private ObservableCollection<Recurs> recursos;
        public ObservableCollection<Recurs> Recursos { get { return recursos; } set { recursos = value; OnPropertyChanged("Recursos"); } }
        #endregion

        /// <summary>
        /// Propietat que registra les ofertes del sistema per poder assignar a una factura
        /// </summary>
        #region ViewModelProperty: Ofertes
        private ObservableCollection<Oferta> ofertes;
        public ObservableCollection<Oferta> Ofertes { get { return ofertes; } set { ofertes = value; OnPropertyChanged("Ofertes"); } }
        #endregion

        /// <summary>
        /// Propietat que registra els tipus de cobrament del sistema per poder assignar a una factura
        /// </summary>
         #region ViewModelProperty: Cobraments
        private ObservableCollection<Cobrament> cobraments;
        public ObservableCollection<Cobrament> Cobraments { get { return cobraments; } set { cobraments = value; OnPropertyChanged("Cobraments"); } }
        #endregion

        /// <summary>
        /// Propietat que registra els usuaris del sistema per poder assignar a una factura
        /// </summary> 
        #region ViewModelProperty: Usuaris
        private ObservableCollection<Usuari> usuaris;
        public ObservableCollection<Usuari> Usuaris
        {
            get
            {
                if (usuaris == null ) usuaris=Usuari.GetInstancies(); return usuaris;
            }
            set { usuaris = value; OnPropertyChanged("Usuaris"); }
        }
        #endregion

        /// <summary>
        /// Propietat que registra un recurs facturat que s'està modificant
        /// </summary>
        #region ViewModelProperty: Recurs_Facturat
        private Recurs_facturat recurs_Facturat;
        public Recurs_facturat Recurs_Facturat
        {
            get
            {
                return recurs_Facturat;
            }
            set
            {
                recurs_Facturat = value;
                OnPropertyChanged("Recurs_Facturat");
            }
        }
        #endregion
        
        /// <summary>
        /// Propietat que registra una oferta facturada que s'està modificant
        /// </summary>
        #region ViewModelProperty: Oferta
        private Oferta oferta;
        public Oferta Oferta
        {
            get
            {
                return oferta;
            }
            set
            {
                oferta = value;
                this.Recurs_Facturat.Id_recurs = oferta.Id_recurs;
                this.Recurs_Facturat.Import = oferta.Import_unitari;
                OnPropertyChanged("Oferta");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra la factura que s'ha de modificar
        /// </summary>
        #region ViewModelProperty: FacturaAModificar
        private Factura facturaAModificar;
        public Factura FacturaAModificar
        {
            get
            {
                return facturaAModificar;
            }
            set
            {
                facturaAModificar = value;
                OnPropertyChanged("FacturaAModificar");
            }
        }
        #endregion

        /// <summary>
        /// Per tenir un accés directe des d'un View a nom d'usuari d'una factura
        /// </summary>
        #region ViewModelProperty: nomUsuariFacturaAModificar
        public String nomUsuariFacturaAModificar { 
            get 
            {
                string retorn = "";
                try
                {
                    retorn = Usuari.GetInstancia(FacturaAModificar.Id_usuari_factura).Nom_usuari;
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_obtenir_usuari + FacturaAModificar.Id_usuari_factura, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                }
                return retorn;
            } 
        }
        #endregion

        /// <summary>
        /// Propietat que registra les factures mostrades
        /// </summary>
        #region ViewModelProperty: Factures
        private ObservableCollection<Factura> _factures;
        public ObservableCollection<Factura> Factures
        {
            get
            {
                try
                {
                    //Obtinc les factures en funció dels filtres activats, aquí només comprovo Usuari i Cobrament
                    if (FiltreUsuari != null)
                        if (FiltreCobrament != null)
                            _factures = Factura.GestInstanciesByUsuariAndCobrament(FiltreUsuari.Id_usuari, FiltreCobrament.Estat_cobrament);
                        else
                            _factures = Factura.GestInstanciesByUsuari(FiltreUsuari.Id_usuari);
                    else
                        if (FiltreCobrament != null)
                            _factures = Factura.GestInstanciesByCobrament(FiltreCobrament.Estat_cobrament);
                        else _factures = Factura.GetInstancies();
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_obtenir_factures + e.Message, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                }

                try
                {
                    //Un cop filtrats Usuari i Cobrament, només deixo passar les factures que inclouen el tipus de Recurs filtrat
                    if (FiltreRecurs != null)
                    {
                        //Colecció buida que emmagatzemarà les factures que coincideixen a contindre el recurs filtrat
                        ObservableCollection<Factura> facturesPont = new ObservableCollection<Factura>();
                        //Recorrem totes les factures comprovant si contenen el recurs
                        foreach (Factura it in _factures)
                        {
                            //Obtinc si està facturat aquest recurs a la factura, si ho està, es llença una excepció, pero
                            //continuo l'execució, si no es llença, afegeixo la factura a la llista
                            try
                            {
                                Recurs_facturat.GetInstancia(it.Id_factura, FiltreRecurs.Id_recurs);
                                facturesPont.Add(it);
                            }
                            catch (Exception e)
                            {
                                //Si no hi es le recurs, es llença un aexcepcio, simplement no afegeixo la factura a la llista
                                
                            }
                                
                        }
                        _factures = facturesPont;
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_obtenir_recurs_facturat + FiltreRecurs.Id_recurs + e.Message, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                }
                return _factures;
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el usuari seleccionat que filtra les factures
        /// </summary>
        #region ViewModelProperty: FiltreUsuari
        private Usuari _filtreUsuari;
        public Usuari FiltreUsuari
        {
            get
            {
                return _filtreUsuari;
            }
            set
            {
                _filtreUsuari = value;
                OnPropertyChanged("Factures");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el cobrament seleccionat que filtra les factures
        /// </summary>
        #region ViewModelProperty: FiltreCobrament
        private Cobrament _filtreCobrament;
        public Cobrament FiltreCobrament
        {
            get
            {
                return _filtreCobrament;
            }
            set
            {
                _filtreCobrament = value;
                OnPropertyChanged("Factures");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el recurs seleccionat que filtra les factures
        /// </summary>
        #region ViewModelProperty: FiltreRecurs
        private Recurs _filtreRecurs;
        public Recurs FiltreRecurs
        {
            get
            {
                return _filtreRecurs;
            }
            set
            {
                _filtreRecurs = value;
                OnPropertyChanged("Factures");
            }
        }
        #endregion

        /// <summary>
        /// Constructor del ViewModel que inicialititza els DelegateCommand i els llistats
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarFacturesViewModel(IEventAggregator messageBus)
        {
            try
            {
               
                    Recursos = Recurs.GetInstancies();
                    Ofertes = Oferta.GetInstancies();
                    Cobraments = Cobrament.GetInstancies();
               
            }
            catch (FaultException e)
            {
                MessageBox.Show(e.Message, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            
            Recurs_Facturat = Nou_recurs_facturat();
            
            RetornLlistaFacturesCommand = new DelegateCommand<object>(RetornLlistaFactures);
            RetornCreaFacturaCommand = new DelegateCommand<Factura>(RetornCreaFactura);
            RetornCreaRecursFacturatCommand = new DelegateCommand(RetornCreaRecursFacturat);
            RetornModificaFacturaCommand = new DelegateCommand<Factura>(RetornModificaFactura);
            ConfirmaElimRecursFacturatCommand = new DelegateCommand(ConfirmaElimRecurs, CanElimRecurs);
            ImprimirCommand = new DelegateCommand(Imprimir);
            ElimRecurs_facturatCommand = new DelegateCommand<Recurs_facturat>(ElimRecurs_facturat, CanModifyFactura);
            this.messageBus = messageBus;
        }


        /// <summary>
        /// Funció que s'executa al triar crear o modificar una factura. La factura s'ha d'haver assignat previament.
        /// </summary>
        #region DelegateCommand: RetornModificaFactura
        public ICommand RetornModificaFacturaCommand { get; protected set; }

        protected void RetornModificaFactura(Factura obj)
        {
            if (obj != null)
            {
                this.FacturaAModificar = obj;
                Uri newUri = new Uri("/Views/Modifica_Factura.xaml", UriKind.Relative);
                this.SelectedPage = newUri;
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
        }
        #endregion        
        
        /// <summary>
        /// Funció que s'executa quan un usuari accepta la finestra de confirmació de modificació o creació d'una factura.
        /// Controla que l'usuari tingui els dreta adients i crida a una funció que la modificarà al sistema
        /// </summary>
        #region DelegateCommand: ModifyFactura
        private DelegateCommand modifyFacturaCommand;
        public ICommand ModifyFacturaCommand
        {
            get
            {
                if (modifyFacturaCommand == null)
                {
                    //Si la crida es fa desde l'apartat principal, no tenim inicialitzat FacturaAModificar, en creem una de nova
                    if (FacturaAModificar == null) {
                        facturaAModificar = Nova_factura();
                    }
                    if (FacturaAModificar.Id_factura == null)
                        modifyFacturaCommand = new DelegateCommand(ModifyFactura, CanCreateFactura);
                    else
                        modifyFacturaCommand = new DelegateCommand(ModifyFactura, CanModifyFactura);
                }
                return modifyFacturaCommand;
            }
        }

        /// <summary>
        /// Funció que que modifica una factura (this.FacturaAModificar) al sistema.
        /// Un cop modificada, reinicialitza els paràmetres del View Model
        /// </summary>
        private void ModifyFactura()
        {
            //Si l'usuari no ha omplert tots els camps, llenço un error
            if (FacturaAModificar.Estat_cobrament == null || FacturaAModificar.Id_usuari_factura == null)
            {
                MessageBox.Show(Strings.Hi_ha_algun_camp_sense_identificar, Strings.Error_camps, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            else
            {
                //Si la factura té id (es automàtic i ho fa el sistema d'entitats) la modifico, si no, la inserto
                if (FacturaAModificar.Id_factura == null)
                {
                    try
                    {
                        //La funció Insert retorna l'identificador automàtic assignat
                        FacturaAModificar.Id_factura = Factura.Insert(FacturaAModificar);
                        Factures.Add(FacturaAModificar);
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(Strings.No_es_possible_crear_factura+ e.Message, Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                    }
                }
                else
                    try{
                        Factura.Update(FacturaAModificar);  
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(Strings.No_es_possible_modificar_factura +e.Message, Strings.Error_de_modificacio, MessageBoxButton.OK, MessageBoxImage.Error);
                    }


                facturaAModificar = null;
                Recurs_Facturat = Nou_recurs_facturat();
                OnPropertyChanged("Factures");
                Uri newUri = new Uri("/Views/Llista_Factures.xaml", UriKind.Relative);
                this.SelectedPage = newUri;
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
        }

        /// <summary>
        /// Funció que comprova si el usuari registrat té els permissos de modificar una factura (sigui propia o forana)
        /// </summary>
        /// <returns>True si es tenen els drets de modificació de factura propia o forana</returns>
        private bool CanModifyFactura()
        {
            bool modifica = false;
            try
            {
                //Si es factura propia, comprovo els drets de factura propia, sino, comprovo els drets de factura forana
                if (FacturaAModificar.Id_usuari_factura.Equals(SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
                    modifica = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Factura.modifica;
                else modifica = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Factura_forana.modifica;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            return modifica;
        }

        /// <summary>
        /// Funció que comprova si el usuari registrat té els permissos de modificar una factura (sigui propia o forana)
        /// Es crea per compatibilitat en la eliminació i creació d'un recurs facturat
        /// </summary>
        /// <returns>True si es tenen els drets de modificació de factura propia o forana</returns>
        private bool CanModifyFactura(Recurs_facturat obj)
        {
            return CanModifyFactura();
        }

        /// <summary>
        /// Funció que comprova si el usuari registrat té els permissos de crear una factura (sigui propia o forana)
        /// </summary>
        /// <returns></returns>
        private bool CanCreateFactura()
        {
            bool retorn = false;
            try
            {
                //Si es factura propia, comprovo els drets de factura propia, sino, comprovo els drets de factura forana
                if (FacturaAModificar.Id_usuari_factura.Equals(SingletonLoggedUser.Instance.LoggedUser.Id_usuari))
                    retorn = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Factura.crea;
                else retorn = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Factura_forana.crea;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return retorn;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta la finestra de creació d'un recurs facturat.
        /// Controla que l'usuari tingui els dreta adients i crida a una funció que l'afegirà al sistema
        /// </summary>
        #region DelegateCommand: AfegeixRecurs_facturatCommand
        private DelegateCommand afegeixRecurs_facturatCommand;

        public ICommand AfegeixRecurs_facturatCommand
        {
            get
            {
                if (afegeixRecurs_facturatCommand == null)
                {
                    if (this.Recurs_Facturat.Id_factura == null)
                        afegeixRecurs_facturatCommand = new DelegateCommand(AfegeixRecurs, CanCreateFactura);
                    else
                        afegeixRecurs_facturatCommand = new DelegateCommand(AfegeixRecurs, CanModifyFactura);
                }
                return afegeixRecurs_facturatCommand;
            }
        }

        /// <summary>
        /// Funció que que crea un recurs faturat (this.Recurs_Facturat) al sistema.
        /// Un cop modificat, reconfigura els paràmetres del View Model
        /// </summary>
        private void AfegeixRecurs()
        {
            //Si l'usuari no ha omplert tots els camps, llenço un error
            if (Recurs_Facturat.Id_recurs == null || Recurs_Facturat.Quantitat == 0 )
            {
                MessageBox.Show(Strings.Hi_ha_algun_camp_sense_identificar, Strings.Error_camps, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            else
            {
                Recurs_Facturat.Id_factura = FacturaAModificar.Id_factura;
                try
                {
                    Recurs_facturat.Insert(Recurs_Facturat);
                    FacturaAModificar.Recursos_facturats.Add(this.Recurs_Facturat);
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.No_es_possible_crear_recurs_facturat + Recurs_Facturat + Strings.Revisi_dades, Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                    //System.Console.WriteLine(e.InnerException);
                }

                Recurs_Facturat = Nou_recurs_facturat();
                Uri newUri = new Uri("/Views/Modifica_Factura.xaml", UriKind.Relative);
                this.SelectedPage = newUri;
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari selecciona la opció de eliminació d'un recurs facturat.
        /// Controla que l'usuari tingui els dreta adients i crida a una funció que mostrarà una finestra de confirmació
        /// </summary>
        #region DelegateCommand: ElimRecurs_facturatCommand
        public ICommand ElimRecurs_facturatCommand { get; private set; }

        private void ElimRecurs_facturat(Recurs_facturat obj)
       {
           if (obj != null)
           {
               Recurs_Facturat = obj;
               ActiveWindow = false;
               finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarRecursFacturat();
               finestra.Show();
           }
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta la finestra de eliminació d'un recurs facturat.
        /// Insereix a la factura el recurs i a la persistència. Reinicia els paràmetres
        /// </summary>
        #region DelegateCommand: ConfirmaElimRecursFacturat
        public ICommand ConfirmaElimRecursFacturatCommand {get;set;}
        
        private void ConfirmaElimRecurs()
        {
            //Si polsem el botó d'eliminar però no hi ha cap recurs seleccionat, no fem res
            if (Recurs_Facturat.Id_recurs != null)
            {
                try
                {
                    Recurs_facturat.Remove(Recurs_Facturat.Id_factura, Recurs_Facturat.Id_recurs);
                    FacturaAModificar.Recursos_facturats.Remove(Recurs_Facturat);
                }
                catch (FaultException e) {
                    MessageBox.Show(Strings.No_es_possible_esborrar_recurs_facturat + Recurs_Facturat + e.Message, Strings.Error_de_esborrat, MessageBoxButton.OK, MessageBoxImage.Error);
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error_de_esborrat, MessageBoxButton.OK, MessageBoxImage.Error);
                }
                //Desassigno el recurs facturat
                Recurs_Facturat = Nou_recurs_facturat();
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimRecurs()
        {
            bool esborra = false;
            try
            {
                esborra = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Recurs.esborra;
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_rol + SingletonLoggedUser.Instance.LoggedUser.Nom_rol, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return esborra;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al voler crear una nova factura
        /// </summary>
        #region DelegateCommand: RetornCreaFactura
        public ICommand RetornCreaFacturaCommand { get; set; }
        private void RetornCreaFactura(Factura factura)
        {
            facturaAModificar = Nova_factura();
            Uri newUri = new Uri("/Views/Modifica_Factura.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al voler afegir un nou recurs a la factura
        /// </summary>
        #region DelegateCommand: RetornCreaRecursFacturat
        public ICommand RetornCreaRecursFacturatCommand { get; set; }
        private void RetornCreaRecursFacturat()
        {
            Recurs_facturat recursFacturat = Nou_recurs_facturat();
            Uri newUri = new Uri("/Views/Afegeix_Recurs_Facturat.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa per imprimir la factura this.FacturaAModificar
        /// </summary>
        #region DelegateCommand: Imprimir
        public ICommand ImprimirCommand { get; private set; }
        private void Imprimir()
        {
            try
            {
                XiquiparcsReunits_RDA.Views.Impressio_factura impressio = new XiquiparcsReunits_RDA.Views.Impressio_factura();
                new XiquiparcsReunits_RDA.Tools.ControlPrinter(impressio, Strings.Factura);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_impressio, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
        }
        #endregion

        /// <summary>
        /// Funció 
        /// </summary>
        #region DelegateCommand: RadioButtonChanged
        public ICommand RadioButtonChangedCommand { get; private set; }
        private void RadioButtonChanged()
        {
            Recurs_Facturat.Data_inici_servei = DateTime.Now;
            if (Oferta.Id_oferta != null)
            {
                //Afegint 3 anys, forço que les dates a elegir tinguin com a màxim 3 anys de marge
                oferta = Oferta.CreateOferta(null, 0, null, 0, DateTime.Now, DateTime.Now.AddYears(3), "");
                OnPropertyChanged("Oferta");
            }
            OnPropertyChanged("Recurs_Facturat");
        }
        #endregion

        /// <summary>
        /// Funció interna que crea un recurs facturat "en blanc"
        /// </summary>
        /// <returns>Recurs facturat inicialitzat</returns>
        private Recurs_facturat Nou_recurs_facturat()
        {
            Recurs_facturat retorn = null;
            try {
                retorn = Recurs_facturat.CreateRecurs_facturat(null, null, 0, 0, 0, DateTime.Now, DateTime.Now.AddHours(1.00)); 
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_crear_recurs_facturat + "null", Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return retorn;
        }

         /// <summary>
        /// Funció interna que crea una factura "en blanc"
        /// </summary>
        /// <returns>Factura inicialitzada</returns>
        private Factura Nova_factura()
        {
            Factura retorn = null;
            try
            {
                retorn = Factura.CreateFactura(null, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, null, true, 0, DateTime.Now, "");
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_crear_factura + "null", Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return retorn;
        }
        
        /// <summary>
        /// Funció interna que crea una oferta "en blanc"
        /// </summary>
        /// <returns>Oferta inicialitzada</returns>
        private Oferta Nova_oferta()
        {
            Oferta retorn = null;
            try
            {
                retorn = Oferta.CreateOferta(null, 0, null, 0, DateTime.Now, DateTime.Now.AddYears(3), "");
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.No_es_possible_crear_oferta + "null", Strings.Error_de_insercio, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            return retorn;
        }
        
    }

}