﻿using System.Windows.Input;
using XiquiparcsReunits_RDA.Model;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Tools;
using System.Windows.Navigation;
using System;


namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarOfertesViewModel : ViewModel
    {

        /// <summary>
        ///Propietat que registra la oferta que s'ha de modificar
        /// </summary>
        #region ViewModelProperty: OfertaAModificar
        private Oferta ofertaAModificar;
        public Oferta OfertaAModificar
        {
            get
            {
                return ofertaAModificar;
            }
            set
            {
                ofertaAModificar = value;
                OnPropertyChanged("OfertaAModificar");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el recurs que filtra les factures
        /// </summary>
        #region ViewModelProperty: FiltreRecurs
        private Recurs _filtreRecurs;
        public Recurs FiltreRecurs
        {
            get
            {
                return _filtreRecurs;
            }
            set
            {
                _filtreRecurs = value;
                OnPropertyChanged("Ofertes");
            }
        }
        #endregion

        /// <summary>
        /// Llistat de ofertes del sistema, filtrades per recurs si correspon
        /// </summary>
        public ObservableCollection<Oferta> Ofertes 
        { 
            get 
            {
                if (FiltreRecurs == null)
                    return Oferta.GetInstancies();
                else return Oferta.GetInstancies(FiltreRecurs.Id_recurs);
            } 
        }

        /// <summary>
        /// Llistat de recursos del sistema
        /// </summary>
        public ObservableCollection<Recurs> Recursos { get { return Recurs.GetInstancies(); } }
        
        /// <summary>
        /// Constructor del ViewModel que inicialitza les comandes delegades
        /// </summary>
        /// <param name="messageBus">Bus de missatges entre els Views i els ViewModels</param>
        public GestionarOfertesViewModel(IEventAggregator messageBus)
        { 
            AssignaElimCommand = new DelegateCommand<Oferta>(AssignaEliminacio);
            RetornLlistaOfertesCommand = new DelegateCommand<object>(RetornLlistaOfertes);
            RetornCreaOfertaCommand = new DelegateCommand<Oferta>(RetornCreaOferta);
            ConfirmaElimCommand = new DelegateCommand(ConfirmaElim);
            RetornModificaOfertaCommand = new DelegateCommand<Oferta>(RetornModificaOferta);
            this.messageBus = messageBus;
        }
        
        /// <summary>
        ///Funció que s'executa al triar una oferta i voler modificar-la
        /// </summary>
        #region DelegateCommand: RetornModificaOferta
        public ICommand RetornModificaOfertaCommand { get; set; }
        private void RetornModificaOferta(Oferta oferta)
        {
            if (oferta != null)
            {
                ofertaAModificar = oferta;
                Uri newUri = new Uri("/Views/Modifica_Oferta.xaml", UriKind.Relative);
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan una oferta accepta la finestra de confirmació de modificació 
        /// </summary>
        #region DelegateCommand: ModifyOferta
        private DelegateCommand modifyOfertaCommand;

        public ICommand ModifyOfertaCommand
        {
            get
            {
                if (modifyOfertaCommand == null)
                {
                    if (OfertaAModificar.Id_oferta == null)
                        modifyOfertaCommand = new DelegateCommand(ModifyOferta, CanCreateOferta);
                    else
                        modifyOfertaCommand = new DelegateCommand(ModifyOferta, CanModifyOferta);
                }
                return modifyOfertaCommand;
            }
        }
        private void ModifyOferta()
        {
            if (OfertaAModificar.Id_oferta == null)
            {
                OfertaAModificar.Id_oferta=Oferta.Insert(OfertaAModificar);
                Ofertes.Add(OfertaAModificar);
            }
            else
                Oferta.Update(OfertaAModificar);
            modifyOfertaCommand = null;
            OnPropertyChanged("Ofertes");
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifyOferta()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Oferta.modifica;
        }
        private bool CanCreateOferta()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Oferta.crea;
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al voler crear una nova oferta
        /// </summary>
        #region DelegateCommand: RetornCreaOferta
        public ICommand RetornCreaOfertaCommand { get; set; }
        private void RetornCreaOferta(Oferta oferta)
        {
            ofertaAModificar = Oferta.CreateOferta(null, 0, "", 0, DateTime.Now, new DateTime(DateTime.Now.Year+1, DateTime.Now.Month, DateTime.Now.Day), "");
            Uri newUri = new Uri("/Views/Modifica_Oferta.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Conjunt de funcions que assignen la oferta a eliminar, mostren una finestra de confirmació i l'esborren
        /// </summary>
        #region DelegateCommand AssignaElim 
        public ICommand AssignaElimCommand { get; private set; }

        /// <summary>
        /// Funció que marca el paràmetre com a objecte a esborrar, i mostra una finestra de confirmació,
        /// agregant un manegador d'events pel tancament
        /// </summary>
        /// <param name="obj">Objecte a esborrar</param>
        private void AssignaEliminacio(Oferta obj)
        {
            if (obj != null)
            {
                OfertaAModificar = obj;
                ActiveWindow = false;
                finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarOferta();
                finestra.Show();
            }
        }
     
        public ICommand ConfirmaElimCommand { get; private set; }

        /// <summary>
        /// Funció que s'executa quan s'accepta a la finestra d'esborrat i esborra del llistat i de la persistència
        /// </summary>
        /// <param name="obj"></param>
        private void ConfirmaElim()
        {  
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (OfertaAModificar != null)
            {
                Oferta.Remove(OfertaAModificar.Id_oferta);
                Ofertes.Remove(OfertaAModificar); 
                OfertaAModificar = null;
                modifyOfertaCommand = null;
            }
            finestra.Close();
            ActiveWindow = true;
        }
        #endregion

    }
}