﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Tools;
using XiquiparcsReunits_RDA.Model;
using System.Windows.Navigation;
using System;


namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarRecursosViewModel : ViewModel
    {

        //Propietat que registra la secció que filtra els recursos
        #region ViewModelProperty: FiltreSeccio
        private Seccio _filtreSeccio;
        public Seccio FiltreSeccio
        {
            get
            {
                return _filtreSeccio;
            }
            set
            {
                _filtreSeccio = value;
                OnPropertyChanged("Recursos");
            }
        }
        #endregion

        public ObservableCollection<Recurs> Recursos 
        { 
            get 
            {
                if (FiltreSeccio == null)
                    return Recurs.GetInstancies();
                else return Recurs.GetInstancies(FiltreSeccio.Id_seccio);
            } 
        }

        public ObservableCollection<Seccio> Seccions { get { return Seccio.GetInstancies(); } }
        public ObservableCollection<TipusIVA> TipusIVAs { get { return TipusIVA.GetInstancies(); } }
        
        
        public GestionarRecursosViewModel(IEventAggregator messageBus)
        {
            ElimRecursCommand = new DelegateCommand<Recurs>(ElimRecurs);
            RetornLlistaRecursosCommand = new DelegateCommand<object>(RetornLlistaRecursos);
            RetornModificaRecursCommand = new DelegateCommand<Recurs>(RetornModificaRecurs);
            RetornCreaRecursCommand = new DelegateCommand<Recurs>(RetornCreaRecurs);
            this.messageBus = messageBus;
        }



        //Propietat que registra el recurs que s'ha de modificar
        #region ViewModelProperty: RecursAModificar
        private Recurs recursAModificar;
        public Recurs RecursAModificar
        {
            get
            {
                return recursAModificar;
            }
            set
            {
                recursAModificar = value;
                OnPropertyChanged("RecursAModificar");
            }
        }
        #endregion


        //Funció que s'executa al triar un recurs i voler modificar-lo
        #region DelegateCommand: RetornModificaRecurs
        public ICommand RetornModificaRecursCommand { get; set; }
        private void RetornModificaRecurs(Recurs recurs)
        {
            if (recurs != null)
            {
                recursAModificar = recurs;
                Uri newUri = new Uri("/Views/Modifica_Recurs.xaml", UriKind.Relative);
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
        }
        #endregion

        //Funció que s'executa quan un recurs accepta la finestra de confirmació de modificació 
        #region DelegateCommand: ModifyRecurs
        private DelegateCommand modifyRecursCommand;

        public ICommand ModifyRecursCommand
        {
            get
            {
                if (modifyRecursCommand == null)
                {

                    if (RecursAModificar.Id_recurs == null) 
                        modifyRecursCommand = new DelegateCommand(ModifyRecurs, CanCreateRecurs);
                    else
                        modifyRecursCommand = new DelegateCommand(ModifyRecurs, CanModifyRecurs);
                }
                return modifyRecursCommand;
            }
        }

        private bool isNew;

        private void ModifyRecurs()
        {
            if (isNew)
            {
                Recurs.Insert(recursAModificar);
                Recursos.Add(RecursAModificar);
            }
            else
                Recurs.Update(RecursAModificar);
            modifyRecursCommand = null;
            OnPropertyChanged("Recursos");
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifyRecurs()
        {
            isNew = false;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Recurs.modifica;
        }
        private bool CanCreateRecurs()
        {
            isNew = true;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Recurs.crea;
        }
        #endregion

        //Funció que s'executa al voler crear un nou recurs
        #region DelegateCommand: RetornCreaRecurs
        public ICommand RetornCreaRecursCommand { get; set; }
        private void RetornCreaRecurs(Recurs recurs)
        {
            recursAModificar = Recurs.CreateRecurs(null, null, 0, null, 0, true, "");
            Uri newUri = new Uri("/Views/Modifica_Recurs.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        #region DelegateCommand: ElimRecurs
        public ICommand ElimRecursCommand { get; private set; }

        private void ElimRecurs(Recurs obj)
        {
            if (obj != null)
            {
                RecursAModificar = obj;
                ActiveWindow = false;
                finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarRecurs();
                finestra.Show();
            }
        }

        public ICommand ConfirmaElimRecursCommand
        {
            get
            {
                if (modifyRecursCommand == null)
                {
                    modifyRecursCommand = new DelegateCommand(ConfirmaElimRecurs, CanElimRecurs);
                }
                return modifyRecursCommand;
            }
        }
        private void ConfirmaElimRecurs()
        {
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (RecursAModificar != null)
            {
                Recurs.Remove(RecursAModificar.Id_recurs);
                Recursos.Remove(RecursAModificar);
                OnPropertyChanged("Recursos");
                RecursAModificar = null;
                modifyRecursCommand = null;
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimRecurs()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Recurs.esborra;
        }
        #endregion

    }
}