﻿using System;
using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using XiquiparcsReunits_RDA.Model;
using Microsoft.Practices.Prism.Events;
using System.Windows.Navigation;
using XiquiparcsReunits_RDA.Tools;

using System.Windows;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarRolsViewModel : ViewModel
    {
        
        public ObservableCollection<Rol> Rols {get; set;}

        public GestionarRolsViewModel(IEventAggregator messageBus)
        {

            rolAModificar = Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol);
            Rols = Rol.GetInstancies();

            ElimRolCommand = new DelegateCommand<Rol>(ElimRol);
            RetornCreaRolCommand = new DelegateCommand<Rol>(RetornCreaRol);
            RetornLlistaRolsCommand = new DelegateCommand<object>(RetornLlistaRols);
            RetornModificaRolCommand = new DelegateCommand<Rol>(RetornModificaRol);
			RetornCreaRolCommand = new DelegateCommand<Rol>(RetornCreaRol);
            this.messageBus = messageBus;
        }

        //Propietat que registra el rol que s'ha de modificar
        #region ViewModelProperty: RolAModificar
        private Rol rolAModificar;
        public Rol RolAModificar
        {
            get
            {
                return rolAModificar;
            }
            set
            {
                rolAModificar = value;
                OnPropertyChanged("RolAModificar");
            }
        }
        #endregion

        //Funció que s'executa al triar un rol i voler modificar-lo
        #region DelegateCommand: RetornModificaRol
        public ICommand RetornModificaRolCommand { get; set; }
        private void RetornModificaRol(Rol rol)
        {
            rolAModificar = rol;
            Uri newUri = new Uri("/Views/Modifica_Rol.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        //Funció que s'executa quan un rol accepta la finestra de confirmació de modificació 
        #region DelegateCommand: ModifyRol
        private DelegateCommand modifyRolCommand;

        public ICommand ModifyRolCommand
        {
            get
            {
                if (modifyRolCommand == null)
                {
                    if (isNewRol) 
                        modifyRolCommand = new DelegateCommand(ModifyRol, CanCreateRol);
                    else
                        modifyRolCommand = new DelegateCommand(ModifyRol, CanModifyRol);
                }
                return modifyRolCommand;
            }
        }
        private void ModifyRol()
        {
            if (isNewRol)
            {
                Rol.Insert(RolAModificar);
                Rols.Add(RolAModificar);
                isNewRol = false;
            }
            else
                Rol.Update(RolAModificar);
            
            modifyRolCommand = null;
            OnPropertyChanged("Rols");
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool isNewRol=false;
        
        private bool CanModifyRol()
        {
            isNewRol = false;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Modifica_Rol.modifica;
        }

        private bool CanCreateRol()
        {
            isNewRol = true;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Modifica_Rol.crea;
        }
        #endregion


        //Funció que s'executa al voler crear un nou rol
        #region DelegateCommand: RetornCreaRol
        public ICommand RetornCreaRolCommand { get; set; }
        private void RetornCreaRol(Rol rol)
        {
            rolAModificar = Rol.CreateRol(null, false, "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", false, "000");
            isNewRol=true;
            Uri newUri = new Uri("/Views/Modifica_Rol.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        #region DelegateCommand: ElimRol
        public ICommand ElimRolCommand { get; private set; }

        private void ElimRol(Rol obj)
        {
            RolAModificar = obj;
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarRol();
            finestra.Show();
        }
       
        public ICommand ConfirmaElimRolCommand
        {
            get
            {
                if (modifyRolCommand == null)
                {
                    modifyRolCommand = new DelegateCommand(ConfirmaElimRol, CanElimRol);
                }
                return modifyRolCommand;
            }
        }
        private void ConfirmaElimRol()
        {
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (RolAModificar != null)
            {
                Rol.Remove(RolAModificar.Nom_rol);
                Rols.Remove(RolAModificar);
                modifyRolCommand = null;
                RolAModificar = null;
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimRol()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Modifica_Rol.esborra;
        }
        #endregion
    }
}