﻿using System.Windows.Input;
using XiquiparcsReunits_RDA.Model;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Tools;
using System.Windows.Navigation;
using System;


namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarSeccionsViewModel : ViewModel
    {
        public ObservableCollection<Seccio> Seccions{get;set;}

        public GestionarSeccionsViewModel(IEventAggregator messageBus)
        {
            
            //navigationHelper = new NavigationHelper();
            Seccions = Seccio.GetInstancies();
            ElimSeccioCommand = new DelegateCommand<Seccio>(ElimSeccio);
            RetornLlistaSeccionsCommand = new DelegateCommand<object>(RetornLlistaSeccions);
            RetornCreaSeccioCommand = new DelegateCommand<Seccio>(RetornCreaSeccio);
            RetornModificaSeccioCommand = new DelegateCommand<Seccio>(RetornModificaSeccio);
            this.messageBus = messageBus;
        }


        //Propietat que registra el seccio que s'ha de modificar
        #region ViewModelProperty: SeccioAModificar
        private Seccio seccioAModificar;
        public Seccio SeccioAModificar
        {
            get
            {
                return seccioAModificar;
            }
            set
            {
                seccioAModificar = value;
                OnPropertyChanged("SeccioAModificar");
            }
        }
        #endregion


        //Funció que s'executa al triar un seccio i voler modificar-lo
        #region DelegateCommand: RetornModificaSeccio
        public ICommand RetornModificaSeccioCommand { get; set; }
        private void RetornModificaSeccio(Seccio seccio)
        {
            seccioAModificar = seccio;
            Uri newUri = new Uri("/Views/Modifica_Seccio.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        //Funció que s'executa quan un seccio accepta la finestra de confirmació de modificació 
        #region DelegateCommand: ModifySeccio
        private DelegateCommand modifySeccioCommand;

        public ICommand ModifySeccioCommand
        {
            get
            {
                if (modifySeccioCommand == null)
                {
                    if (SeccioAModificar.Id_seccio == null)
                        modifySeccioCommand = new DelegateCommand(ModifySeccio, CanCreateSeccio);
                    else
                        modifySeccioCommand = new DelegateCommand(ModifySeccio, CanModifySeccio);
                   
                }
                return modifySeccioCommand;
            }
        }

        private bool isNew;
        private void ModifySeccio()
        {
                if (isNew)
                {
                    Seccio.Insert(SeccioAModificar);
                    Seccions.Add(SeccioAModificar);
                }
                else
                    Seccio.Update(SeccioAModificar);
                modifySeccioCommand = null;
                OnPropertyChanged("Seccions");
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifySeccio()
        {
            isNew = false;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Seccio.modifica;
        }
        private bool CanCreateSeccio()
        {
            isNew = true;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Seccio.crea;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al voler crear un nou seccio
        /// </summary>
        #region DelegateCommand: RetornCreaSeccio
        public ICommand RetornCreaSeccioCommand { get; set; }
        private void RetornCreaSeccio(Seccio seccio)
        {
            seccioAModificar = Seccio.CreateSeccio(null,"");
            Uri newUri = new Uri("/Views/Modifica_Seccio.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion



        #region DelegateCommand: ElimSeccio
        public ICommand ElimSeccioCommand { get; private set; }

        private void ElimSeccio(Seccio obj)
        {
            SeccioAModificar = obj;
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarSeccio();
            finestra.Show();
        }

        public ICommand ConfirmaElimSeccioCommand
        {
            get
            {
                if (modifySeccioCommand == null)
                {
                    modifySeccioCommand = new DelegateCommand(ConfirmaElimSeccio, CanElimSeccio);
                }
                return modifySeccioCommand;
            }
        }
        private void ConfirmaElimSeccio()
        {
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (SeccioAModificar != null)
            {
                Seccio.Remove(SeccioAModificar.Id_seccio);
                Seccions.Remove(seccioAModificar);
                modifySeccioCommand = null;
                seccioAModificar = null;
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimSeccio()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Seccio.esborra;
        }
        #endregion

        
    }
}