﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Model;
using XiquiparcsReunits_RDA.Tools;
using System.Windows.Navigation;
using System;
using System.Windows;


namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarTipusIVAViewModel : ViewModel
    {
        public ObservableCollection<TipusIVA> TipusIVAs { get; set; }
           
        public GestionarTipusIVAViewModel(IEventAggregator messageBus)
        {
            TipusIVAs =TipusIVA.GetInstancies();
            ElimTipusIVACommand = new DelegateCommand<TipusIVA>(ElimTipusIVA);
            RetornModificaTipusIVACommand = new DelegateCommand<TipusIVA>(RetornModificaTipusIVA);
            RetornCreaTipusIVACommand = new DelegateCommand<TipusIVA>(RetornCreaTipusIVA);
            this.messageBus = messageBus;
        }

        //Propietat que registra el tipusIVA que s'ha de modificar
        #region ViewModelProperty: TipusIVAAModificar
        private TipusIVA tipusIVAAModificar;
        public TipusIVA TipusIVAAModificar
        {
            get
            {
                return tipusIVAAModificar;
            }
            set
            {
                tipusIVAAModificar = value;
                OnPropertyChanged("TipusIVAAModificar");
            }
        }
        #endregion


        //Funció que s'executa al triar un tipusIVA i voler modificar-lo
        #region DelegateCommand: RetornModificaTipusIVA
        public ICommand RetornModificaTipusIVACommand { get; set; }
        private void RetornModificaTipusIVA(TipusIVA tipusIVA)
        {
            tipusIVAAModificar = tipusIVA;
            Uri newUri = new Uri("/Views/Modifica_Tipus_IVA.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        //Funció que s'executa quan un tipusIVA accepta la finestra de confirmació de modificació 
        #region DelegateCommand: ModifyTipusIVA
        private DelegateCommand modifyTipusIVACommand;

        public ICommand ModifyTipusIVACommand
        {
            get
            {
                if (modifyTipusIVACommand == null)
                {
                    if (TipusIVAAModificar.Nom_IVA == null)
                        modifyTipusIVACommand = new DelegateCommand(ModifyTipusIVA, CanCreateTipusIVA);
                    else
                        modifyTipusIVACommand = new DelegateCommand(ModifyTipusIVA, CanModifyTipusIVA);
                    
                }
                return modifyTipusIVACommand;
            }
        }
        private bool isNew;
        private void ModifyTipusIVA()
        {
            bool found=false;
            foreach (TipusIVA iva in TipusIVAs){
                if (iva.Nom_IVA.Equals(TipusIVAAModificar.Nom_IVA)) found = true;
            }
            if (0 < TipusIVAAModificar.Percentatge && TipusIVAAModificar.Percentatge < 1 && !found)
            {
                if (isNew)
                {
                    TipusIVA.Insert(TipusIVAAModificar);
                    TipusIVAs.Add(TipusIVAAModificar);
                }
                else
                    TipusIVA.Update(TipusIVAAModificar);
                modifyTipusIVACommand = null;
                OnPropertyChanged("TipusIVAs");
                Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
                this.SelectedPage = newUri;
                messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
            }
            else MessageBox.Show(Strings.Error_camps + Strings.Revisi_percetatge, Strings.Error, MessageBoxButton.OK);
        }

        private bool CanModifyTipusIVA()
        {
            isNew = false;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Tipus_IVA.modifica;
        }
        private bool CanCreateTipusIVA()
        {
            isNew = true;
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Tipus_IVA.crea;
        }
        #endregion

        //Funció que s'executa al voler crear un nou tipusIVA
        #region DelegateCommand: RetornCreaTipusIVA
        public ICommand RetornCreaTipusIVACommand { get; set; }
        private void RetornCreaTipusIVA(TipusIVA tipusIVA)
        {
            tipusIVAAModificar = TipusIVA.CreateTipusIVA(null, 0, "");
            Uri newUri = new Uri("/Views/Modifica_Tipus_IVA.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion


        #region DelegateCommand: ElimTipusIVA
        public ICommand ElimTipusIVACommand { get; private set; }

        private void ElimTipusIVA(TipusIVA obj)
        {
            TipusIVAAModificar = obj;
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarTipusIVA();
            finestra.Show();
        }

        public ICommand ConfirmaElimTipusIVACommand
        {
            get
            {
                if (modifyTipusIVACommand == null)
                {
                    modifyTipusIVACommand = new DelegateCommand(ConfirmaElimTipusIVA, CanElimTipusIVA);
                }
                return modifyTipusIVACommand;
            }
        }
        private void ConfirmaElimTipusIVA()
        {
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (tipusIVAAModificar != null)
            {
                TipusIVA.Remove(TipusIVAAModificar.Nom_IVA);
                TipusIVAs.Remove(tipusIVAAModificar);
                tipusIVAAModificar = null;
                modifyTipusIVACommand = null;
            }
            finestra.Close();
            ActiveWindow = true;
        }
        private bool CanElimTipusIVA()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Tipus_IVA.esborra;
        }
        #endregion

    }
}