﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Model;
using System;
using System.Windows.Data;
using System.Data.Services.Client;
using XiquiparcsReunits_RDA.Tools;
using System.Windows;


namespace XiquiparcsReunits_RDA.ViewModels
{
    public class GestionarUsuarisViewModel : ViewModel 
    {                
        /// <summary>
        /// Propietat que registra el rol que filtra els usuaris
        /// </summary>
        #region ViewModelProperty: FiltreRol
        private Rol _filtreRol;
        public Rol FiltreRol
        {
            get
            {
                return _filtreRol;
            }
            set
            {
                _filtreRol = value;
                usuaris = Usuari.GetInstancies(_filtreRol);
                OnPropertyChanged("Usuaris");
            }
        }
        #endregion

        /// <summary>
        /// Obté o assigna els usuaris.
        /// </summary>
        /// <value>Usuaris</value>
        public ObservableCollection<Usuari> Usuaris{
            get
            {
                if (usuaris == null) usuaris = Usuari.GetInstancies();
                return usuaris;
            }
        }
        /// <summary>
        /// Propietat que registra el nou password en MD5 en un canvi
        /// </summary>
        #region ViewModelProperty: NouPassword
        private string nouPassword="";
        public string NouPassword
        {
            get
            {
                return nouPassword;
            }
            set
            {
                nouPassword = MD5Core.GetHashString(value);
            }
        }
        private string confirmaNouPassword="";
        public string ConfirmaNouPassword
        {
            get
            {
                return confirmaNouPassword;
            }
            set
            {
                confirmaNouPassword = MD5Core.GetHashString(value);
            }
        }
        #endregion


        
        private ObservableCollection<Usuari> usuaris;


        public ObservableCollection<Rol> Rols { get { if (rols == null) rols = Rol.GetInstancies(); return rols; } }
        private ObservableCollection<Rol> rols;

        /// <summary>
        /// Constructor del View Model
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarUsuarisViewModel(IEventAggregator messageBus)
        {
            RetornLlistaUsuarisCommand = new DelegateCommand<object>(RetornLlistaUsuaris);
            RetornModificaUsuariCommand = new DelegateCommand<Usuari>(RetornModificaUsuari);
            RetornCreaUsuariCommand = new DelegateCommand<Usuari>(RetornCreaUsuari);
            ElimUsuariCommand = new DelegateCommand<Usuari>(ElimUsuari);
            ChangePasswordCommand = new DelegateCommand(ChangePassword);
            ConfirmaChangePasswordCommand = new DelegateCommand(ConfirmaChangePassword);
            CancelaChangePasswordCommand = new DelegateCommand(CancelaChangePassword);
           
            this.messageBus = messageBus;
        }

        /// <summary>
        /// Propietat que registra el usuari que s'ha de modificar
        /// </summary>
        #region ViewModelProperty: UsuariAModificar
        private Usuari usuariAModificar;
        public Usuari UsuariAModificar
        {
            get
            {
                return usuariAModificar;
            }
            set
            {
                usuariAModificar = value;
                OnPropertyChanged("UsuariAModificar");
            }
        }
        #endregion
       
        /// <summary>
        /// Funció que s'executa quan un usuari accepta la finestra de confirmació de modificació 
        /// </summary>
        #region DelegateCommand: ModifyTheUser
        private DelegateCommand modifyTheUserCommand;
        public ICommand ModifyTheUserCommand
        {
            get
            {
                if (modifyTheUserCommand == null)
                {
                    if (UsuariAModificar.Id_usuari == null)
                        modifyTheUserCommand = new DelegateCommand(ModifyTheUser, CanCreateTheUser);
                    else
                        modifyTheUserCommand = new DelegateCommand(ModifyTheUser, CanModifyTheUser);
                }
                return modifyTheUserCommand;
            }
        }

        private void ModifyTheUser()
        {
                //Si accedim des de crear usuari, UsuariAModificar tindrà el id NULL, l'inserirem i assignarem el nou id
                if (UsuariAModificar.Id_usuari == null)
                {
                    UsuariAModificar.Id_usuari = Usuari.Insert(UsuariAModificar);
                    Usuaris.Add(UsuariAModificar);
                }
                else
                {
                    Usuari.Update(UsuariAModificar);
                    //Actualitzo la llista d'usuaris
                    usuaris = Usuari.GetInstancies();
                }

                UsuariAModificar = null;
            modifyTheUserCommand = null;
            Uri newUri = new Uri("/Views/Logo.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }

        private bool CanModifyTheUser()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Usuari.modifica;
        }
        private bool CanCreateTheUser()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Usuari.crea;
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al triar modificar un usuari i voler modificar-lo. L'usuari s'ha d'haver assignat previament.
        /// </summary>
        #region DelegateCommand: RetornModificaUsuari
        public ICommand RetornModificaUsuariCommand { get; set; }
        protected void RetornModificaUsuari(Usuari obj)
        {
            //Si l'objecte es null, es que es vol modificar l'usuari registrat
            if (obj != null)
                UsuariAModificar = obj;
            else UsuariAModificar = SingletonLoggedUser.Instance.LoggedUser;
            Uri newUri = new Uri("/Views/Modifica_Usuari.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al voler crear un usuari
        /// </summary>
        #region DelegateCommand: RetornCreaUsuari
        public ICommand RetornCreaUsuariCommand { get; set; }
        private void RetornCreaUsuari(Usuari usuari)
        {
            //El password per defecte es "password"
            usuariAModificar = Usuari.CreateUsuari("", "", "", "", "", "", "", "", 0, "", "", "5F4DCC3B5AA765D61D8327DEB882CF99", "", "");
            Uri newUri = new Uri("/Views/Modifica_Usuari.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al voler eliminar un usuari
        /// </summary>
        #region DelegateCommand: ElimUsuari
        public ICommand ElimUsuariCommand { get; private set; }
        
        private void ElimUsuari(Usuari obj)
        {
            UsuariAModificar = obj;
            ActiveWindow = false;
            finestra= new XiquiparcsReunits_RDA.Views.ChildWindows.EsborrarUsuari();
            finestra.Show();
        }

        public ICommand ConfirmaElimUsuariCommand
        {
            get
            {
                if (modifyTheUserCommand == null)
                {
                        modifyTheUserCommand = new DelegateCommand(ElimTheUser, CanElimTheUser);
                }
                return modifyTheUserCommand;
            }
        }
        private void ElimTheUser()
        {            
            //Si polsem el botó d'eliminar però no hi ha cap element seleccionat, no fem res
            if (UsuariAModificar != null)
            {
                Usuari.Remove(UsuariAModificar.Id_usuari);
                usuaris.Remove(UsuariAModificar);
                UsuariAModificar = null;
                modifyTheUserCommand = null;
            }
            ActiveWindow = true;
            finestra.Close();
        }
        private bool CanElimTheUser()
        {
            return Rol.GetInstancia(SingletonLoggedUser.Instance.LoggedUser.Nom_rol).Usuari.esborra;
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al voler canviar el password de l'usuari actual
        /// </summary>
        #region DelegateCommand: ChangePassword
        public ICommand ChangePasswordCommand { get; private set; }        
        private void ChangePassword()
        {
            ActiveWindow = false;
            finestra= new XiquiparcsReunits_RDA.Views.ChildWindows.Canvi_Contrasenya();
            finestra.Show();
        }

        public ICommand ConfirmaChangePasswordCommand{ get; private set; }
        private void ConfirmaChangePassword()
        {
            //Si polsem el botó de canvi de contrasenya canviem el password de l'usuari
            if (UsuariAModificar != null)
            {
                if (ConfirmaNouPassword.Equals(NouPassword)){
                    UsuariAModificar.SetPwd(NouPassword);
                } else
                    MessageBox.Show(Strings.No_es_possible_canviar_password, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            ActiveWindow = true;
            finestra.Close();
        }
        
        public ICommand CancelaChangePasswordCommand{ get; private set; }
        private void CancelaChangePassword()
        {
            ConfirmaNouPassword = "";
            NouPassword = "";
            ActiveWindow = true;
            finestra.Close();
        }
        #endregion
    }
}