﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_RDA.Tools;
using Microsoft.Practices.Prism.Commands;
using XiquiparcsReunits_RDA.Model;
using System.ServiceModel;
using System.Collections.ObjectModel;

using System.Windows.Navigation;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public class PrincipalViewModel : ViewModel
    {

        /// <summary>
        /// Propietat que defineix si es mostra els menús quan un usuari es registra
        /// </summary>
        #region ViewModelProperty: IsLoggedUser
        private Boolean _isLoggedUser = false;

        public Boolean IsLoggedUser
        {
            get
            {
                return _isLoggedUser;
            }
            set
            {
                _isLoggedUser = value;
                OnPropertyChanged("IsLoggedUser");
                OnPropertyChanged("NotIsLoggedUser");
            }
        }
        #endregion
        
        /// <summary>
        /// Propietat que retorna si mostra menús quan no hi ha un usuari registrat
        /// </summary>
        #region ViewModelProperty: NotIsLoggedUser
        public bool NotIsLoggedUser
        {
            get
            {
                return !_isLoggedUser;
            }
        }
        #endregion NotIsLoggedUser

        
        /// <summary>
        /// Propietat que controla en la identificació si s'ha introduït un nom d'usuari
        /// </summary>
        #region ViewModelProperty: Login
        private string _login;
        public string Login
        {
            get
            {
                return _login;
            }

            set
            {
                _login = value;
                OnPropertyChanged("Login");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que controla en la identificació si s'ha introduït un codi d'accés, en format MD5
        /// </summary>
        #region ViewModelProperty: Password
        private string _password;
        public string Password
        {
            get
            {
                return _password;
            }

            set
            {
                _password = MD5Core.GetHashString(value);
                OnPropertyChanged("Password");
            }
        }
        #endregion

        /// <summary>
        /// Constructor principal del View Model. S'encarrega de inicialitzar el menú, i les comandes d'aquest.
        /// A més gestiona les comandes de loggin.
        /// </summary>
        /// <param name="messageBus">Bus de missatges entre els Views i els ViewModel</param>
        public PrincipalViewModel(IEventAggregator messageBus)
        {
            this.messageBus = messageBus;
            
            // página por defecto
            this.SelectedPage = new Uri("/Views/Logo.xaml", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Subscribe(ChangePage);

            // Conjunt de comandes del menú
            RetornIdentificacioCommand = new DelegateCommand<object>(RetornIdentificacio);
            RetornDesidentificacioCommand = new DelegateCommand<object>(RetornDesidentificacio);
            SortirCommand = new DelegateCommand<object>(Sortir);
            RetornLlistaUsuarisCommand = new DelegateCommand<object>(RetornLlistaUsuaris);
            RetornLlistaRolsCommand = new DelegateCommand<object>(RetornLlistaRols);
            RetornLlistaFacturesCommand = new DelegateCommand<object>(RetornLlistaFactures);
            RetornLlistaRecursosCommand = new DelegateCommand<object>(RetornLlistaRecursos);
            RetornLlistaSeccionsCommand = new DelegateCommand<object>(RetornLlistaSeccions);
            RetornLlistaOfertesCommand = new DelegateCommand<object>(RetornLlistaOfertes);
            RetornSobreNosaltresCommand = new DelegateCommand<object>(RetornSobreNosaltres);
            RetornSobreAplicacioCommand = new DelegateCommand<object>(RetornSobreAplicacio);
            RetornLlistaCobramentsCommand = new DelegateCommand<object>(RetornLlistaCobraments);
            RetornLlistaTipusIVACommand = new DelegateCommand<object>(RetornLlistaTipusIVA);
        }


        public void ChangeLoggedUser(Usuari newUsuari)
        {
            SingletonLoggedUser.Instance.LoggedUser = newUsuari;
        }

        /// <summary>
        /// Funcions que desidentifiquen a l'usuari del sistema
        /// </summary>
        #region DelegateCommand: RetornDesidentificacio
        public ICommand RetornDesidentificacioCommand { get; private set; }

        private void RetornDesidentificacio(object obj)
        {
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.Desidentificacio();
            finestra.Show();
        }

        public ICommand DeloginTheUserCommand
        {
            get
            {             
                return new DelegateCommand(DeloginTheUser, CanLoginTheUser);
            }
        }

        /// <summary>
        /// Funció que des-identifica a un usuari, tornant les variables al punt inicial.
        /// </summary>
        private void DeloginTheUser(){
               SingletonLoggedUser.Instance.LoggedUser = null;
               finestra.Close();
               ActiveWindow = true;
               IsLoggedUser = false;
               this.Login = "";
               this.Password = "";
               this.SelectedPage = new Uri("/Views/Logo.xaml", UriKind.Relative);
               // página por defecto
               messageBus.GetEvent<CompositePresentationEvent<Uri>>().Subscribe(ChangePage);               
        }

        #endregion

        /// <summary>
        /// Conjunt de funcions i comandes delegades que permeten identificar a un usuari
        /// </summary>
        #region DelegateCommand: LoginTheUser
        private DelegateCommand loginTheUserCommand;

        public ICommand LoginTheUserCommand
        {
            get
            {
                if (loginTheUserCommand == null)
                {
                    loginTheUserCommand = new DelegateCommand(LoginTheUser, CanLoginTheUser);
                }
                return loginTheUserCommand;
            }
        }

        private void LoginTheUser()
        {
            Usuari ddbbUser;
            try
            {
                GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient(); 
                OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
                MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

                messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", Login));
                messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", Password));
            
                //Per poder fer la lectura del usuari, inicialitzo el SingletonLoggedUser
                SingletonLoggedUser.Instance.LoggedUser = Usuari.CreateUsuari(Login, "", "", "", "", "", "", "", 0, "", "", Password, "", "");
                SingletonLoggedUser.Instance.LoggedUser.Id_usuari = Usuari.GetIdUsuari(Login);
            
                ddbbUser = Usuari.GetInstanciaByNom(Login);

                if (!ddbbUser.TestPwd(Password))
                    MessageBox.Show(Strings.Usuari_incorrecte, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
                else
                {
                    if (Rol.GetInstancia(ddbbUser.Nom_rol).Acces_escriptori)
                    {
                
                        SingletonLoggedUser.Instance.LoggedUser = ddbbUser;
                        finestra.Close();
                        ActiveWindow = true;
                        IsLoggedUser = true;
                    }
                    else MessageBox.Show(Strings.Usuari_no_te_permis, Strings.Error, MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.No_es_possible_obtenir_usuari + Login, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut, Strings.Error_de_cerca, MessageBoxButton.OK, MessageBoxImage.Error);
                //System.Console.WriteLine(e.InnerException);
            }
        }

        private bool CanLoginTheUser()
        {
            return true;
            //return !String.IsNullOrEmpty(Login) && !String.IsNullOrEmpty(Password);
        }

        public ICommand RetornIdentificacioCommand { get; private set; }
        
        private void RetornIdentificacio(object obj)
        {
            ActiveWindow = false;
            finestra = new XiquiparcsReunits_RDA.Views.ChildWindows.Identificacio();
            finestra.Closed += new EventHandler(IdentificacioWindow_Closed);
            finestra.Show();
        }

        private void IdentificacioWindow_Closed(object sender, EventArgs e)
        {
            if (finestra.DialogResult != false)
            {
                //Si hi ha hagut un error d'identificació es llança una excepció, si no, l'identificació es correcta
                _isLoggedUser = true;
                ActiveWindow = true;
            }
        }
        #endregion
        
        /// <summary>
        /// Funcio que tanca l'aplicació
        /// </summary>
        #region DelegateCommand: Sortir
        public ICommand SortirCommand { get; private set; }

        private void Sortir(object obj)
        {
            Application.Current.Shutdown();
        }
        #endregion


        /// <summary>
        /// Funció que navega i mostra la informació sobre la empresa
        /// </summary>
        #region DelegateCommand: RetornSobreNosaltres
        public ICommand RetornSobreNosaltresCommand { get; private set; }

        private void RetornSobreNosaltres(object obj)
        {
            Uri newUri = new Uri("/Views/Sobre_Nosaltres.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
         }
        #endregion


        /// <summary>
        /// Funció que navega i mostra la informació sobre la aplicació
        /// </summary>
        #region DelegateCommand: RetornSobreAplicacio
        public ICommand RetornSobreAplicacioCommand { get; private set; }

        private void RetornSobreAplicacio(object obj)
        {
            Uri newUri = new Uri("/Views/Sobre_Aplicacio.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion
    }
}
