﻿using System.ComponentModel;
using XiquiparcsReunits_RDA.Tools;
using Microsoft.Practices.Prism.Events;
using System;
using XiquiparcsReunits_RDA.Model;
using System.Windows.Input;
using Microsoft.Practices.Prism.Commands;
using System.Windows;
using System.Collections.ObjectModel;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_RDA.ViewModels
{
    public abstract class ViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Propietat que registra el usuari registrat
        /// </summary>
        #region ViewModelProperty: LoggedUser
        public Usuari LoggedUser
        {
            get { return SingletonLoggedUser.Instance.LoggedUser; }
        }
        #endregion


        /// <summary>
        /// Llistat de les provincies que es mostrarn en un combo
        /// </summary>
        private string[] provincies = { "Álava", "Albacete", "Alicante", "Almería", "Asturias", "Avila", "Badajoz", "Barcelona", "Burgos", "Cáceres", "Cádiz", "Cantabria", "Castellón", "Ceuta", "Ciudad Real", "Córdoba", "Cuenca", "Gerona", "Granada", "Guadalajara", "Guipúzcoa", "Huelva", "Huesca", "Islas Baleares", "Jaén", "La Coruña", "La Rioja", "Las Palmas", "León", "Lérida", "Lugo", "Madrid", "Málaga", "Melilla", "Murcia", "Navarra", "Orense", "Palencia", "Pontevedra", "S.C. de Tenerife", "Salamanca", "Segovia", "Sevilla", "Soria", "Tarragona", "Teruel", "Toledo", "Valencia", "Valladolid", "Vizcaya", "Zamora", "Zaragoza" };
        public ObservableCollection<String> Provincies { get { return new ObservableCollection<string>(provincies); } private set { } }
	
        /// <summary>
        /// Propietat que registra la pàgina visualitzada actual
        /// </summary>
        #region ViewModelProperty: SelectedPage   
        private Uri selectedPage;
        public Uri SelectedPage
        {
            get { return selectedPage; }
            set { selectedPage = value; OnPropertyChanged("SelectedPage"); }
        }
        #endregion



        /// <summary>
        /// Propietats que afecten al comportament principal de la finestar contenidora      
        /// </summary>
        #region ViewModelProperty: ActiveWindow
        private Boolean _activeWindow = true;
        public Boolean ActiveWindow { get { return _activeWindow; } set { _activeWindow = value; OnPropertyChanged("ActiveWindow"); } }
        #endregion

        /// <summary>
        /// Funció que permet navegar a una nova pàgina
        /// </summary>
        /// <param name="obj">Uri de la pàgina a mostrar</param>
        public void ChangePage(Uri obj)
        {
            SelectedPage = obj;
        }
        
        /// <summary>
        /// Bus de missatges (que gestina les comandes delegades) entre els Views i els ViewModels
        /// </summary>
        public IEventAggregator messageBus{get;set;}
        
        /// <summary>
        /// Gestor que controla els esdeveniments i comunica als Views qualsevol canvi en els objectes
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Funció que mostra la finestra modal pasada com a paràmetre
        /// </summary>
        /// <param name="finestra">Finestra modal a mostrar</param>
       protected void ShowWindow(Window finestra)
        {
            ActiveWindow = false;
            this.finestra = finestra;
            finestra.Show();
        }

        /// <summary>
        /// Funció que tanca la finestra modal
        /// </summary>
        #region DelegateCommand: CloseChildWindowCommand
        protected Window finestra;
        private DelegateCommand closeChildWindowCommand;
        public ICommand CloseChildWindowCommand  {
            get
            {
                closeChildWindowCommand = new DelegateCommand(CloseChildWindow);
                return closeChildWindowCommand;
            }
        }

        /// <summary>
        /// Funció que tanca una finestra modal
        /// </summary>
        private void CloseChildWindow()
        {
            finestra.Close();
        }
        #endregion
        
        /// <summary>
        /// Funció que s'executa al triar llistar els usuaris
        /// </summary>
        #region DelegateCommand: RetornLlistaUsuaris
        public ICommand RetornLlistaUsuarisCommand { get; protected set; }

        protected void RetornLlistaUsuaris(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Usuaris.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);

        }
        #endregion

        /// <summary>
        /// Funció que s'executa al triar llistar els rols
        /// </summary>
        #region DelegateCommand: RetornLlistaRols
        public ICommand RetornLlistaRolsCommand { get; protected set; }

        protected void RetornLlistaRols(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Rols.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al triar llistar les factures
        /// </summary>
        #region DelegateCommand: RetornLlistaFactures
        public ICommand RetornLlistaFacturesCommand { get; protected set; }

        protected void RetornLlistaFactures(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Factures.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al triar llistar els recursos
        /// </summary>
        #region DelegateCommand: RetornLlistaRecursos
        public ICommand RetornLlistaRecursosCommand { get; protected set; }

        protected void RetornLlistaRecursos(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Recursos.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al triar llistar les seccions
        /// </summary>
        #region DelegateCommand: RetornLlistaSeccions
        public ICommand RetornLlistaSeccionsCommand { get; protected set; }

        protected void RetornLlistaSeccions(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Seccions.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion


        /// <summary>
        /// Funció que s'executa al triar llistar les ofertes
        /// </summary>
        #region DelegateCommand: RetornLlistaOfertes
        public ICommand RetornLlistaOfertesCommand { get; protected set; }

        protected void RetornLlistaOfertes(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Ofertes.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al triar llistar els tipus d'IVA
        /// </summary>
        #region DelegateCommand: RetornLlistaTipusIVA
        public ICommand RetornLlistaTipusIVACommand { get; protected set; }

        protected void RetornLlistaTipusIVA(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Tipus_IVA.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al triar llistar els tipus de cobrament
        /// </summary>
        #region DelegateCommand: RetornLlistaCobraments
        public ICommand RetornLlistaCobramentsCommand { get; protected set; }

        protected void RetornLlistaCobraments(object obj)
        {
            Uri newUri = new Uri("/Views/Llista_Tipus_Cobrament.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa al tornar a la pantalla inicial
        /// </summary>
        #region DelegateCommand: RetornPrincipal
        private DelegateCommand retornPrincipalCommand;
        public ICommand RetornPrincipalCommand {
            get 
            { if (retornPrincipalCommand == null)
                {
                    retornPrincipalCommand=new DelegateCommand(RetornPrincipal);
                }
                return retornPrincipalCommand; 
            } 
        }

        private void RetornPrincipal()
        {
            Uri newUri = new Uri("/Views/Principal.xaml", UriKind.Relative);
            this.SelectedPage = newUri;
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que prepara una connexió amb el servidor de client, preparant la capcelera SOAP amb el usuari i password
        /// del sistema
        /// </summary>
        /// <returns>Client creat</returns>
        protected GestorXiquiparc.GestorXiquiparcClient InitClient()
        {
            GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
            if (SingletonLoggedUser.Instance.LoggedUser != null)
            {
                OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
                MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

                messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", SingletonLoggedUser.Instance.LoggedUser.Id_usuari));
                messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", SingletonLoggedUser.Instance.LoggedUser.Pwd));
            }
            return client;
        }
    }
}
