﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using XiquiparcsReunits_Silverlight.Tools;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;

namespace XiquiparcsReunits_Silverlight.Model
{
    /// <summary>
    /// Objecte que representa un tipus d'estat de cobrament d'una factura, per exemple, al comptat, pendent de cobrament...
    /// </summary>
	public class Cobrament: INotifyPropertyChanged
	{
		/// <summary>
		/// Propietat que definei un estat de cobrament
		/// </summary>
        private string estat_cobrament;
        public string Estat_cobrament { get { return estat_cobrament; } set { this.estat_cobrament = value; OnPropertyChanged("Estat_cobrament"); } }
        
        /// <summary>
        /// Propietat de la descripció de l'estat de cobrament
        /// </summary>
        private string descripcio;
        public string Descripcio { get { return descripcio; } set { this.descripcio = value; OnPropertyChanged("Descripcio"); } }

		public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Constructor privat que crea una instancia sense paràmetres
        /// </summary>
        private Cobrament() {
        }


        /// <summary>
        /// Constructor que crea una instancia de l'objecte pero no es realitza lectura de la persistència
        /// </summary>
        /// <param name="estat_cobrament">Estat del cobrament que l'identifica</param>
        /// <param name="descripcio">Descripció de l'estat de cobrament</param>
        public Cobrament(String estat_cobrament, string descripcio)
        {
            this.Estat_cobrament = estat_cobrament;
            this.Descripcio = descripcio;
        }

		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

       
        /// <summary>
        /// Constructor estàtic que retorna un objecte a partir dels seus paràmetres
        /// </summary>
        /// <param name="estat_cobrament">Estat identificador del cobrament</param>
        /// <param name="descripcio">Descripcio del cobrament</param>
        /// <returns>Objecte creat</returns>
	    public static Cobrament CreateCobrament(string estat_cobrament, string descripcio)
        {
            Cobrament cobrament = new Cobrament();
            cobrament.Estat_cobrament = estat_cobrament;
            cobrament.Descripcio = descripcio;
            return cobrament;
        }
		
		public override string ToString(){
			return this.Estat_cobrament;
		}

	}
}