﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using System.Collections.Generic;
using System.Windows;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;

namespace XiquiparcsReunits_Silverlight.Model
{   
    /// <summary>
    /// Objecte que registra els continents de la part de la web autoeditable. Es definirà una alçada i una amplada, i un codi
    /// xaml que definirà el continent. Ha de ser de tipus template per poder vincular el contingut.
    /// </summary>
	public class Continent: INotifyPropertyChanged
	{  

        
        /// <summary>
        /// Identificador únic del continent
        /// </summary>
		#region ViewModelProperty: Id_continent
        private string id_continent  { get; set;}
        public string Id_continent
        {
            get {
                return id_continent;
                }
            set
            {
                id_continent = value;
                OnPropertyChanged("Id_continent");
            }
        }
        #endregion


        /// <summary>
        /// Codi xaml que formarà el continent basat en un UserControl Template
        /// </summary>
        #region ViewModelProperty: Xaml
        private string xaml  { get; set;}
        public string Xaml
        {
            get {
                return xaml;
                }
            set
            {
                xaml = value;
                OnPropertyChanged("Xaml");
            }
        }
        #endregion

		/// <summary>
		/// Alçada del contenidor en píxels
		/// </summary>
        #region ViewModelProperty: Alcada
        private int alcada;
        public int Alcada {
			get {return alcada;}
			set {
				alcada=value;
				OnPropertyChanged("Alcada");
            }
		}
        #endregion

        /// <summary>
        /// Alçada del contenidor en píxels
        /// </summary>
        #region ViewModelProperty: Amplada
        private int amplada;
        public int Amplada
        {
            get { return amplada; }
            set
            {
                amplada = value;
                OnPropertyChanged("Amplada");
            }
        }
        #endregion

        /// <summary>
        /// Observacions sobre el continent
        /// </summary>
        #region ViewModelProperty: Observacions
        private string observacions;
        public string Observacions
        {
            get { return observacions; }
            set
            {
                observacions= value;
                OnPropertyChanged("Observacions");
            }
        }
        #endregion

        /// <summary>
        /// Constructor sense paràmetres
        /// </summary>
        private Continent() {
        }

				
		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

        /// <summary>
        /// Funció que retorna un continent inicialitzant els mínims paràmetres
        /// </summary>
        /// <param name="id">Identificador del continent (no es comprova)</param>
        /// <param name="xaml">Codi xaml que defineix el continent de tipus template</param>
        /// <param name="alcada">Alçada en píxels de l'element</param>
        /// <param name="amplada">Amplada en píxels de l'element</param>
        /// <param name="observacions">Observacions sobre el continent</param>
        /// <returns>Continent creat</returns>
        public static Continent CreateContinent(string id, String xaml, int alcada, int amplada, string observacions)
        {
            Continent continent = new Continent();
            continent.Id_continent = id;
            continent.Xaml = xaml;
            continent.Amplada = amplada;
            continent.Alcada = alcada;
            continent.Observacions = observacions;
            return continent;
        }
		
		/// <summary>
		/// Funció que retorna el codi del continent
		/// </summary>
		/// <returns>Cadena que representa el codi del continent</returns>
		public  override string ToString(){
			return this.Id_continent;
		}
	}
}