﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using System.Collections.Generic;
using System.Windows;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;
using System.ServiceModel;

namespace XiquiparcsReunits_Silverlight.Model
{
    /// <summary>
    /// Objecte que registra els continguts de la part de la web autoeditable. Es definirà un continent i una pàgina, i un codi
    /// xaml que definirà el contingut. Ha de ser de tipus binding per vincular al continent.
    /// </summary>
    public class Contingut : INotifyPropertyChanged
    {
    
        /// <summary>
        /// Identificador únic del contingut
        /// </summary>
        #region ViewModelProperty: Id_contingut
        private int id_contingut { get; set; }
        public int Id_contingut
        {
            get
            {
                return id_contingut;
            }
            set
            {
                id_contingut = value; OnPropertyChanged("Id_contingut");
            }
        }
        #endregion

        /// <summary>
        /// Codi xaml que formarà el contingut de tipus binding
        /// </summary>
        #region ViewModelProperty: Xaml
        private string xaml { get; set; }
        public string Xaml
        {
            get
            {
                return xaml;
            }
            set
            {
                xaml = value;
                OnPropertyChanged("Xaml");
            }
        }
        #endregion

        /// <summary>
        /// Identificador de la pàgina que contindrà l'element
        /// </summary>
        #region ViewModelProperty: Pagina
        private int pagina;
        public int Pagina
        {
            get { return pagina; }
            set
            {
                pagina = value;
                OnPropertyChanged("Pagina");
            }
        }
        #endregion

        /// <summary>
        /// Identificador del continent
        /// </summary>
        #region ViewModelProperty: Continent
        private string continent;
        public string Continent
        {
            get { return continent; }
            set
            {
                continent = value;
                OnPropertyChanged("Continent");
            }
        }
        #endregion


        /// <summary>
        /// Observacions sobre el contingut
        /// </summary>
        #region ViewModelProperty: Observacions
        private string observacions;
        public string Observacions
        {
            get { return observacions; }
            set
            {
                observacions = value;
                OnPropertyChanged("Observacions");
            }
        }
        #endregion
        
        /// <summary>
        /// Constructor sense paràmetres, es declara públic ja que s'utilitza des de XAML
        /// </summary>
        public Contingut()
        {
        }


        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        /// <summary>
        /// Funció que retorna un contingut inicialitzant els mínims paràmetres
        /// </summary>
        /// <param name="id_contingut">Identificador del contingut</param>
        /// <param name="xaml">Codi xaml que defineix el contingut de tipus Binding</param>
        /// <param name="pagina">Pàgina on està inscrit el contingut</param>
        /// <param name="continent">Continent del contingut</param>
        /// <param name="observacions">Observacions sobre el contingut</param>
        /// <returns>Continent creat</returns>
        public static Contingut CreateContingut(int id_contingut, String xaml, int pagina, string continent, string observacions)
        {
            Contingut contingut = new Contingut();
            contingut.Id_contingut = id_contingut;
            contingut.Xaml = xaml;
            contingut.Pagina = pagina;
            contingut.Continent = continent;
            contingut.Observacions = observacions;

            return contingut;
        }



        /// <summary>
        /// Funció que retorna el codi del contingut
        /// </summary>
        /// <returns>Cadena que representa el codi del contingut</returns>
        public override string ToString()
        {
            return this.Xaml;
        }
    }
}