﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace XiquiparcsReunits_Silverlight.Model
{
    /// <summary>
    /// Classe que registra cadascun dels drets definits a un objecte Rol, els identifica en funció de crear, modificar o esborrar
    /// </summary>
    public class Dret
    {
        public bool crea { get; set; }
        public bool modifica { get; set; }
        public bool esborra { get; set; }

        /// <summary>
        /// Permet crear un nou dret amb tots els permisos a 1
        /// </summary>
        public Dret()
        {
            this.crea = true;
            this.modifica = true;
            this.esborra = true;
        }

        /// <summary>
        /// Permet crear un nou dret a partir d'una cadena de booleans
        /// </summary>
        /// <param name="drets">Tres booleans, sobre si es permet crear, modificar o esborarra respectivament</param>
        public Dret(bool[] drets)
        {
            if (drets.Length!=3){
                throw new FormatException("La llargada de la cadena ha de ser 3 elements");
            }
            this.crea=drets[0];
            this.modifica=drets[1];
            this.esborra=drets[2];
        }
      
        /// <summary>
        /// Permet crear un nou dret a partir d'una cadena de caràcters
        /// </summary>
        /// <param name="drets">Tres caràcters '0' o '1', sobre si es permet crear, modificar o esborarra respectivament</param>
        public Dret(char[] drets)
        {
            if (drets.Length!=3){
                throw new FormatException("La llargada de la cadena ha de ser 3 elements");
            }
           if (drets[0]=='0') {
                this.crea=false;
            } else if  (drets[0]=='1'){
                this.crea=true;
           } else throw new FormatException("Els caràcters han de ser '0' o '1'");
           if (drets[1]=='0') {
                this.modifica=false;
           } else if  (drets[1]=='1'){
                this.modifica=true;
           } else throw new FormatException("Els caràcters han de ser '0' o '1'");
           if (drets[2]=='0') {
                this.esborra=false;
           } else if  (drets[2]=='1'){
                this.esborra=true;
           } else throw new FormatException("Els caràcters han de ser '0' o '1'");
            
        }

        /// <summary>
        /// Permet crear un nou dret a partir d'una cadena
        /// </summary>
        /// <param name="drets">String que representa els drets</param>
        public Dret(string drets)
        {
            Dret dret = new Dret(drets.ToArray());
            this.crea = dret.crea;
            this.modifica = dret.modifica;
            this.esborra = dret.esborra;
        }

        /// <summary>
        /// Converteix a string un dret, per exemple "111" significa que l'usuari té dret de crear, modificar i esborrar l'element
        /// </summary>
        /// <returns>Tres caràcters '0' o '1', sobre si es permet crear, modificar o esborarra respectivament</returns>
        public override string ToString()
        {
            String result = "";
            if (this.crea) result = "1"; else result = "0";
            if (this.modifica) result = String.Concat(result, "1"); else result = String.Concat(result, "0");
            if (this.esborra) result = String.Concat(result, "1"); else result = String.Concat(result, "0");
            return result;
        }
    }
}
