﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;

namespace XiquiparcsReunits_Silverlight.Model
{
    /// <summary>
    /// Objecte que representa una promoció sobre un recurs. S'oferirà un determinat recurs en un tram de temps a un preu
    /// determinat, i en una quantitat limitada.
    /// </summary>
	public class Oferta: INotifyPropertyChanged
	{
		public string Id_oferta {get;set;}
		public string Id_recurs {get;set;}
		public Decimal Import_unitari {get;set;}
        public String Descripcio { get; set; }
		public int Quantitat_ofertada {get;set;}
		public DateTime Data_inici_oferta{get;set;}
		public DateTime Data_fi_oferta{get;set;}
        			
		public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Constructor privat per evitar la seva crida
        /// </summary>
        private Oferta() { }

		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
		
        /// <summary>
        /// Crea una oferta (fàbrica) a partir de tots els paràmetres. No la inserta a la BBDD.
        /// </summary>
        /// <param name="id_oferta">Identificador de la oferta</param>
        /// <param name="import_unitari">Import unitari del recurs en oferta</param>
        /// <param name="id_recurs">Identificador del recurs en oferta</param>
        /// <param name="quantitat_ofertada">Quantitat de recurs ofertat</param>
        /// <param name="data_inici_oferta">Data i hora d'inici de l'oferta</param>
        /// <param name="data_fi_oferta">Data i hora de fi de l'oferta</param>
        /// <param name="descripcio">Descripció de l'oferta</param>
        /// <returns>Oferta creada</returns>
		public static Oferta CreateOferta(String id_oferta, Decimal import_unitari, String id_recurs, Int32 quantitat_ofertada, DateTime data_inici_oferta, DateTime data_fi_oferta, string descripcio)
        {
            Oferta oferta = new Oferta();
            oferta.Id_oferta = id_oferta;
            oferta.Descripcio = descripcio;
            oferta.Import_unitari = import_unitari;
            oferta.Id_recurs = id_recurs;
            oferta.Quantitat_ofertada = quantitat_ofertada;
            oferta.Data_inici_oferta = data_inici_oferta;
            oferta.Data_fi_oferta = data_fi_oferta;
            return oferta;
        }
		
		/// <summary>
		/// Funció que retorna una cadena identificador de l'oferta
		/// </summary>
		/// <returns>Cadena identificadora de l'oferta</returns>
		public override  string ToString(){
			return this.Id_oferta;
		}
	}
}