﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using System.Collections.Generic;
using System.Windows;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;
using System.ServiceModel;

namespace XiquiparcsReunits_Silverlight.Model
{   
    /// <summary>
    /// Objecte que registra les pàgines de la part de la web autoeditable. Es definirà un identificador (int), un titol i una opció del menú,
    /// També s'inclouen observacions internes per la gestió
    /// </summary>
	public class Pagina : INotifyPropertyChanged
	{       

        /// <summary>
        /// Llistat de continguts de la pàgina
        /// </summary>
        #region ViewModelProperty: Continguts
        private ObservableCollection<Contingut> continguts;
        public ObservableCollection<Contingut> Continguts
        {
            get { return continguts; }
            set { continguts= value; OnPropertyChanged("Continguts");}
        }
        #endregion


        /// <summary>
        /// Identificador únic de la pàgina. Quan s'assigna, es llegeix de la persistencia els continguts i continents que li corresponen
        /// </summary>
		#region ViewModelProperty: Id_pagina
        private int id_pagina  { get; set;}
        public int Id_pagina
        {
            get {
                return id_pagina;
                }
            set
            {
                id_pagina = value;
                OnPropertyChanged("Id_pagina");
            }
        }
        #endregion

        /// <summary>
        /// Titol que apareixerà a la barra del explorador al veure la pàgina
        /// </summary>
		#region ViewModelProperty: Titol
        private string titol  { get; set;}
        public string Titol
        {
            get {
                return titol;
                }
            set
            {
                titol = value;
                OnPropertyChanged("Titol");
            }
        }
        #endregion

        /// <summary>
        /// Opció al menú per veure la pàgina
        /// </summary>
        #region ViewModelProperty: Menu
        private string menu;
        public string Menu
        {
            get { return menu; }
            set
            {
                menu = value;
                OnPropertyChanged("Menu");
            }
        }
        #endregion

        /// <summary>
        /// Observacions de la pàgina
        /// </summary>
        #region ViewModelProperty: Observacions
        private string observacions;
        public string Observacions
        {
            get { return observacions; }
            set
            {
                observacions = value;
                OnPropertyChanged("Observacions");
            }
        }
        #endregion

        /// <summary>
        /// Constructor sense paràmetres
        /// </summary>
        private Pagina() {
            Continguts = new ObservableCollection<Contingut>();
        }
        		
		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

        /// <summary>
        /// Funció que retorna una pàgina inicialitzant els mínims paràmetres
        /// </summary>
        /// <param name="id">Identificador de la pàgina (no es comprova)</param>
        /// <param name="titol">Titol a mostrar al navegador</param>
        /// <param name="menu">Titol a mostrar al menú</param>
        /// <param name="observacions">Observacions sobre la pàgina</param>
        /// <returns>Pagina creada</returns>
        public static Pagina CreatePagina(int id, String titol, String menu, string observacions)
        {
            Pagina pagina = new Pagina();
            pagina.Id_pagina = id;
            pagina.Titol = titol;
            pagina.Menu = menu;
            pagina.Observacions = observacions;


            return pagina;
        }
		
		/// <summary>
		/// Funció que retorna el identificador
		/// </summary>
		/// <returns>Cadena que representa el identificador (posició) de la pàgina</returns>
		public  override string ToString(){
            return this.Id_pagina.ToString();
		}
	}
}