﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using System.Windows;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;

namespace XiquiparcsReunits_Silverlight.Model
{
    /// <summary>
    /// Objecte que representa la utilització d'un recurs en una factura però en un preu promocional
    /// </summary>
	public class Recurs_facturat: INotifyPropertyChanged
	{
		public String Id_factura {get;set;}

        private string id_recurs { get; set; }

        /// <summary>
        /// Paràmetre que inicialitzas el identificador del recurs, en l'assignació inicialitza el recurs i el tipus d'IVA
        /// </summary>
        public String Id_recurs
        {
            get
            {
                    return id_recurs;
            }
            set
            {
                id_recurs = value; OnPropertyChanged("Id_recurs");
            }
        }

        private Decimal import;
        public Decimal Import 
        { 
            get 
            { 
                return import; 
            }
            set 
            { 
                import=value; OnPropertyChanged("Import");
            } 
        }

        private int quantitat=0;
        public Int32 Quantitat
        {
            get
            {
                return quantitat;
            }
            set
            {
                quantitat = value; OnPropertyChanged("Quantitat"); OnPropertyChanged("Import_total");
            }
        }

        /// <summary>
        /// Propietat que llegeix i escriu el IVA en percentatge, això es en "18"
        /// </summary>
        public Decimal Percentatge_IVA
        {
            get
            {
                return IVA_recurs * 100;
            }
            set
            {
                IVA_recurs = value / 100; OnPropertyChanged("Percentatge_IVA");
            }
        }
		public DateTime Data_inici_servei{get;set;}
		public DateTime Data_fi_servei{get;set;}

        private string descripcio_recurs;
        public string Descripcio_recurs { get { return descripcio_recurs; } set { descripcio_recurs = value; OnPropertyChanged("Descripcio_recurs"); } }

        /// <summary>
        /// Propietat que llegeix i escriu el IVA en decimal, això es en "0,18"
        /// </summary>
        private decimal iva;
        public Decimal IVA_recurs { get { return iva; } set { iva = value; OnPropertyChanged("IVA_recurs"); } }

        public Decimal Import_total { get { return Import * Quantitat * (1 + Percentatge_IVA/100); } }

        private bool recurs_controlable;
        public bool Recurs_controlable { get { return recurs_controlable; } set { recurs_controlable = value; OnPropertyChanged("Recurs_controlable"); } }

		public event PropertyChangedEventHandler PropertyChanged;
		
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
		
        		
        
        /// <summary>
        /// Funció que retorna un nou recurs facturat a partir dels paràmetres sense inscriure'l a la persistència
        /// </summary>
        /// <param name="id_factura">Identificador de la factura</param>
        /// <param name="id_recurs">Identificador del recurs</param>
        /// <param name="import">Import del recurs en el moment de la facturació</param>
        /// <param name="quantitat">Quantitat de recurs facturada</param>
        /// <param name="percentatge_IVA">Percentatge d'IVA del recurs en el moment de la facturació</param>
        /// <param name="data_inici_servei">Data d'inici de la facturació</param>
        /// <param name="data_fi_servei">Data de fi de la facturació</param>
        /// <returns></returns>
		public static Recurs_facturat CreateRecurs_facturat(String id_factura, String id_recurs, Decimal import, Int32 quantitat, Decimal percentatge_IVA, DateTime data_inici_servei, DateTime data_fi_servei)
        {
            Recurs_facturat recurs_facturat = new Recurs_facturat();
            recurs_facturat.Id_factura = id_factura;
            recurs_facturat.Id_recurs = id_recurs;
            recurs_facturat.Import = import;
            recurs_facturat.Quantitat = quantitat;
            recurs_facturat.Percentatge_IVA = percentatge_IVA;
            recurs_facturat.Data_inici_servei = data_inici_servei;
            recurs_facturat.Data_fi_servei = data_fi_servei;
            return recurs_facturat;
        }
		
		/// <summary>
		/// Retorna una cadena que representa l'objecte
		/// </summary>
		/// <returns>Cadena que representa l'objecte</returns>
		public override  string ToString(){
			return this.Id_recurs + " a " + this.Id_factura;
		}
		
	}
}