﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using XiquiparcsReunits_Silverlight.Tools;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;

namespace XiquiparcsReunits_Silverlight.Model
{
	public class Rol: INotifyPropertyChanged
	{
		public string Nom_rol {get;set;}
		public bool Acces_escriptori{get;set;}
		public bool Modifica_empresa{get;set;}
		public Dret Usuari {get;set;}
		public Dret Modifica_Rol {get;set;}
		public Dret Reserva {get;set;}
        public Dret Reserva_forana { get; set; }
        public Dret Recurs { get; set; }
        public Dret Seccio { get; set; }
        public Dret Oferta { get; set; }
        public Dret Factura { get; set; }
        public Dret Factura_forana { get; set; }
        public Dret Cobrament { get; set; }
		public Dret Tipus_IVA {get;set;}
        public Dret Pagina { get; set; }
        public Dret Continent { get; set; }
        public Dret Contingut { get; set; }
        	
		public event PropertyChangedEventHandler PropertyChanged;
		

		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}

        /// <summary>
        /// Constructor que crea una nova instancia de Rol, amb tots els drets en denegat
        /// </summary>
        public Rol()
        {
            this.Nom_rol = null;
            this.Modifica_empresa = false;
            this.Usuari = new Dret("000");
            this.Reserva_forana = new Dret("000");
            this.Reserva = new Dret("000");
            this.Recurs = new Dret("000");
            this.Seccio = new Dret("000");
            this.Oferta = new Dret("000");
            this.Factura = new Dret("000");
            this.Factura_forana = new Dret("000");
            this.Cobrament = new Dret("000");
            this.Tipus_IVA = new Dret("000");
            this.Pagina = new Dret("000");
            this.Continent = new Dret("000");
            this.Contingut = new Dret("000");
            this.Acces_escriptori = false;
            this.Modifica_Rol = new Dret("000");
        }

		
        /// <summary>
        /// Fàbrica que retorna un rol amb els tots paràmetres iniciliatizats 
        /// </summary>
        /// <param name="nom_rol">Nom del nou rol</param>
        /// <param name="modifica_empresa">Booleà que representa si l'usauri té dret a modificar les dades de l'empresa</param>
        /// <param name="usuari">Drets de CUD usuaris</param>
        /// <param name="reserva_forana">Drets de CUD de factures del futur no propies</param>
        /// <param name="reserva">Drets de CUD de factures del futur propies</param>
        /// <param name="recurs">Drets de CUD de recursos</param>
        /// <param name="seccio">Drets de CUD de seccions</param>
        /// <param name="oferta">Drets de CUD de ofertes</param>
        /// <param name="factura">Drets de CUD de factures propies</param>
        /// <param name="factura_forana">Drets de CUD de factures no propies</param>
        /// <param name="cobrament">Drets de CUD de tipus de cobrament</param>
        /// <param name="tipus_IVA">Drets de CUD de tipus d'iva</param>
        /// <param name="pagina">Drets de CUD de pagines</param>
        /// <param name="continent">Drets de CUD de continents</param>
        /// <param name="contingut">Drets de CUD de continguts</param>
        /// <param name="acces_escriptori">Booleà que representa si l'usauri té accés a l'aplicació d'escriptori o a la web</param>
        /// <param name="rol">Drets de CUD de rols</param>
        /// <returns>Rol creat a partir dels paràmetres</returns>
		 public static Rol CreateRol(String nom_rol, Boolean modifica_empresa, String usuari, String reserva_forana, String reserva, String recurs, String seccio, String oferta, String factura, String factura_forana, String cobrament, String tipus_IVA, String pagina, String continent, String contingut, Boolean acces_escriptori, String rol)
        {
            Rol resultat = new Rol();
            resultat.Nom_rol = nom_rol;
            resultat.Modifica_empresa = modifica_empresa;
            resultat.Usuari = new Dret(usuari);
            resultat.Reserva_forana = new Dret(reserva_forana);
            resultat.Reserva = new Dret(reserva);
            resultat.Recurs = new Dret(recurs);
            resultat.Seccio = new Dret(seccio);
            resultat.Oferta = new Dret(oferta);
            resultat.Factura = new Dret(factura);
            resultat.Factura_forana = new Dret(factura_forana);
            resultat.Cobrament = new Dret(cobrament);
            resultat.Tipus_IVA = new Dret(tipus_IVA);
            resultat.Pagina = new Dret(pagina);
            resultat.Continent = new Dret(continent);
            resultat.Contingut = new Dret(contingut);
            resultat.Acces_escriptori = acces_escriptori;
            resultat.Modifica_Rol = new Dret(rol);
            return resultat;
        }

		/// <summary>
		/// Retorna una cadena de text que representa l'objecte
		/// </summary>
		/// <returns>Cadena que representa l'objecte</returns>
		public  override string ToString(){
			return this.Nom_rol;
		}
		
	}
}