﻿using System;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Windows;
using XiquiparcsReunits_Silverlight.GestorXiquiparc;
using XiquiparcsReunits_Silverlight.Tools;

namespace XiquiparcsReunits_Silverlight.Model
{
	public class Usuari: INotifyPropertyChanged
	{
        private string id_usuari;
        public string Id_usuari
        {
            get
            { return id_usuari; }
            set
            { id_usuari = value; OnPropertyChanged("Id_usuari"); }
        }

        private string nom_usuari;
        public string Nom_usuari
        {
            get
            { return nom_usuari; }
            set
            { nom_usuari = value; OnPropertyChanged("Nom_usuari"); }
        }

        private string nif_usuari;
        public string NIF_usuari
        {
            get
            { return nif_usuari; }
            set
            { nif_usuari = value; OnPropertyChanged("NIF_usuari"); }
        }

        private string nom_fiscal_usuari;
        public string Nom_fiscal_usuari
        {
            get
            { return nom_fiscal_usuari; }
            set
            { nom_fiscal_usuari = value; OnPropertyChanged("Nom_fiscal_usuari"); }
        }
        
        public string Telefon1 { get; set; }
		public string Telefon2 {get;set;}
		public string Movil {get;set;}
		public string Fax {get;set;}
		public string Direccio {get;set;}

        private string poblacio;
        public string Poblacio
        {
            get
            { return poblacio; }
            set
            { poblacio = value; OnPropertyChanged("Poblacio"); }
        }

		public int Codpos {get;set;}

        private string provincia;
        public string Provincia
        {
            get
            { return provincia; }
            set
            { provincia = value; OnPropertyChanged("Provincia"); }
        }

		public string Estat {get;set;}

        private string MD5Pwd;
        /// <summary>
        /// Propietat que registra el password de l'usuari, sempre es guarda en format MD5
        /// </summary>
        public string Pwd { get { return MD5Pwd; } set { MD5Pwd= MD5Core.GetHashString(value);  } }

        private string observacions;
        public string Observacions
        {
            get
            { return observacions; }
            set
            { observacions = value; OnPropertyChanged("Observacions"); }
        }

        private string nom_rol;
        public string Nom_rol
        {
            get
            { return nom_rol; }
            set
            { nom_rol = value; OnPropertyChanged("Nom_rol"); }
        }
        			
		public event PropertyChangedEventHandler PropertyChanged;
        
        /// <summary>
        /// Event captat en la modificació d'una propietat de l'usuari
        /// </summary>
        /// <param name="name">Nom de la propietat modificada</param>
		public void OnPropertyChanged(string name){
			if (PropertyChanged !=null)
					PropertyChanged(this, new PropertyChangedEventArgs(name));
		}
       		

        /// <summary>
        /// Constructor (fàbrica) d'usuaris amb tots els paràmetres. No l'inscriu a la BBDD
        /// </summary>
        /// <param name="nom_usuari">Nom d'usuari</param>
        /// <param name="nIF_usuari">NIF de l'usuari</param>
        /// <param name="nom_fiscal">Nom fiscal (real) de l'usuari</param>
        /// <param name="tel1">Telèfon 1 de l'usuari</param>
        /// <param name="tel2">Telèfon 2 de l'usuari</param>
        /// <param name="movil">Movil de l'usuari</param>
        /// <param name="direccio">Direcció postal de l'usuari</param>
        /// <param name="poblacio">Població postal de l'usuari</param>
        /// <param name="codpos">Codi postal de l'usuari</param>
        /// <param name="provincia">Provincia postal de l'usuari</param>
        /// <param name="estat">Estat postal de l'usuari</param>
        /// <param name="pwd">Codi d'accés de l'usuari</param>
        /// <param name="observacions">Observacions sobre l'usuari</param>
        /// <param name="nom_rol">Nom del rol de l'usuari</param>
        /// <returns>Usuari creat a partir dels paràmetres</returns>
		public static Usuari CreateUsuari(String nom_usuari, String nIF_usuari, String nom_fiscal, string tel1, string tel2, String movil, String direccio, String poblacio, Int32 codpos, String provincia, String estat, String pwd, string observacions, String nom_rol)
        {
            Usuari usuari = new Usuari();
            usuari.Nom_usuari = nom_usuari;
            usuari.NIF_usuari = nIF_usuari;
            usuari.Nom_fiscal_usuari = nom_fiscal;
            usuari.Movil = movil;
            usuari.Telefon1 = tel1;
            usuari.Telefon2 = tel2;
            usuari.Direccio = direccio;
            usuari.Poblacio = poblacio;
            usuari.Codpos = codpos;
            usuari.Provincia = provincia;
            usuari.Estat = estat;
            usuari.Pwd = pwd;
            usuari.Observacions = observacions;
            usuari.Nom_rol = nom_rol;
            return usuari;
        }

        /// <summary>
        /// Funció constructora de usuaris a partir de tots els paràmetres (fàbrica d'usuaris). No l'inscriu a la BBDD
        /// </summary>
        /// <param name="id_usuari">Identificador de l'usuari</param>
        /// <param name="nom_usuari">Nom de l'usuari</param>
        /// <param name="nIF_usuari">NIF de l'usuari</param>
        /// <param name="nom_fiscal">Nom fiscal (real) de l'usuari</param>
        /// <param name="tel1">Telèfon 1 del client</param>
        /// <param name="tel2">Telèfon 2 del client</param>
        /// <param name="movil">Telèfon movil del client</param>
        /// <param name="direccio">Direcció postal del client</param>
        /// <param name="poblacio">Població postal del client</param>
        /// <param name="codpos">Codi postal del client</param>
        /// <param name="provincia">Provincia postal del client</param>
        /// <param name="estat">Estat postal del client</param>
        /// <param name="pwd">Codi d'accés del client</param>
        /// <param name="observacions">Observacions sobre el client</param>
        /// <param name="nom_rol">Rol del client</param>
        /// <returns>Usuari creat a partir dels paràmetres</returns>
        public static Usuari CreateUsuari(String id_usuari, String nom_usuari, String nIF_usuari, String nom_fiscal, String tel1, String tel2, String movil, String direccio, String poblacio, Int32 codpos, String provincia, String estat, String pwd, string observacions, String nom_rol)
        {
            Usuari usuari = CreateUsuari(nom_usuari, nIF_usuari, nom_fiscal, tel1, tel2, movil, direccio, poblacio, codpos, provincia, estat, pwd, observacions, nom_rol);
            usuari.Id_usuari = id_usuari;
            return usuari;
        }

		
        /// <summary>
        /// Funció que retorna el nom d'usuari de l'usuari
        /// </summary>
        /// <returns>Text amb el nom de l'usuari</returns>
		public override  string ToString(){
			return this.Nom_usuari;
		}
        
        /// <summary>
        /// Funció que comprova un codi d'accés comparant-lo amb el del usuari actual (tots dos en format MD5)
        /// </summary>
        /// <param name="password">Codi a comparar</param>
        /// <returns>Cert si ambdós codis coincideixen</returns>
        public bool TestPwd(string password)
        {
            return this.Pwd.Equals(MD5Core.GetHashString(password));
        }
	}
}