﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text.RegularExpressions;


//Codi obtingut de http://www.codeproject.com/Articles/294603/Taming-Silverlight-Async-Calls

namespace XiquiparcsReunits_Silverlight.Tools
{

    public class AsyncCoordinator
    {
        public void addClient(AsyncRunnerBase client)
        {
            clients.Add(client);
        }
        List<AsyncRunnerBase> clients = new List<AsyncRunnerBase>();
        public void initiate()
        {
            foreach (AsyncRunnerBase client in clients)
            {
                client.initiateCall();
            }
        }
    }
    public delegate void InvokerDelegate();

    public class AsyncRunnerBase
    {
        public enum RunStatus { NotStarted, InProgress, Finished, Failed };
        List<AsyncRunnerBase> dependencies = new List<AsyncRunnerBase>();
        public RunStatus status = RunStatus.NotStarted;
        protected AsyncCoordinator coordinator = null;
        public InvokerDelegate invoke;
        public void dependsOn(AsyncRunnerBase dependency)
        {
            dependencies.Add(dependency);
        }
        public void initiateCall()
        {
            if (status != RunStatus.NotStarted) return;
            foreach (AsyncRunnerBase dependency in dependencies)
            {
                if (dependency.status != RunStatus.Finished) return;
            }
            // all dependencies have finished, so we can start
            status = RunStatus.InProgress;
            invoke();
        }
        public AsyncRunnerBase(AsyncCoordinator c)
        {
            coordinator = c;
            coordinator.addClient(this);
            status = RunStatus.NotStarted;
        }
        public AsyncRunnerBase()
        {
            coordinator = null;
            status = RunStatus.NotStarted;
        }
    }
    public class AsyncRunner<T> : AsyncRunnerBase where T : System.ComponentModel.AsyncCompletedEventArgs
    {
        EventHandler<T> callback;

        public AsyncRunner()
        {
        }
        public AsyncRunner(AsyncCoordinator c)
            : base(c)
        {
        }
        protected static Regex nameFromClass = new Regex(@".*\.([A-Za-z0-9\-]+)CompletedEventArgs.*");

        public void registerCallback(Object client, EventHandler<T> d)
        {
            callback = d;
            String callName = "";
            Match mx = nameFromClass.Match(d.GetType().FullName);
            if (mx.Success)
            {
                callName = mx.Groups[1].Value;
            }
            else
            {
                // can't happen......
            }
            //client.slowcall1Completed += new EventHandler<slowcall1CompletedEventArgs>();
            EventInfo eventInfo = client.GetType().GetEvent(callName + "Completed");
            MethodInfo addMethod = eventInfo.GetAddMethod();
            MethodInfo removeMethod = eventInfo.GetRemoveMethod();
            EventHandler<T> mycallback = new EventHandler<T>((o, e) =>
            {
                if (e.UserState != this) return;
                this.status = RunStatus.InProgress;
                Object[] args = { callback };
                removeMethod.Invoke(client, args);
                // now do the real work
                callback(o, e);
                if (this.status != RunStatus.Failed) this.status = RunStatus.Finished;
                // now see if any more children are ready to run
                if (coordinator != null) coordinator.initiate();
            });

            Object[] args1 = { mycallback };
            addMethod.Invoke(client, args1);
        }
    }
}