﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using XiquiparcsReunits_Silverlight.Model;
using System.ComponentModel;
using System.ServiceModel;

namespace XiquiparcsReunits_Silverlight.Tools
{

    /// <summary>
    /// Patter de Singleton que permet accedir des de XAML, ja que té un constructor que crea un getter
    /// que retorna la instancia únit
    /// </summary>
    public class SingletonLoggedUser
    {
        /// <summary>
        /// Propietats que registren un usuari registrat, un cop identificat, llegeix de la persistencia
        /// els drets d'aquest usuari de manera asíncrona
        /// </summary>
        private Usuari loggedUser { get; set; }
        public Usuari LoggedUser
        {
            get
            {
                return loggedUser;
            }
            set
            {
                loggedUser = value;
            }
        }

        /// <summary>
        /// Paràmetres de drets continguts en un rol del usuari registrat
        /// </summary>
        public Rol RolLoggedUser { get; set; }

        /// <summary>
        /// Propietat estàtica que registra el Singleton
        /// </summary>
        private static SingletonLoggedUser instance;

        /// <summary>
        /// Constructor declarat privat per evitar la crida (no compliria el model Singleton)
        /// Inicialitza els paràmetres a un usuari fictici sense cap dret de modificació
        /// </summary>
        private SingletonLoggedUser() {
            this.loggedUser = Usuari.CreateUsuari("", "", "", "", "", "", "", "", 0, "", "", "", "", "default");
            this.RolLoggedUser = Rol.CreateRol("default", false, "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", "000", false, "000");
            }

        /// <summary>
        /// Constructor estàndard del Singleton
        /// </summary>
        public static SingletonLoggedUser Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new SingletonLoggedUser();
                }
                return instance;
            }
        }

    }
}



