﻿using System.Windows.Input;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_Silverlight.Model;
using System;
using System.Collections;
using System.Windows.Controls;
using System.Windows;
using XiquiparcsReunits_Silverlight.Tools;
using System.ServiceModel;
using System.ComponentModel;
using XiquiparcsReunits_Silverlight.Views.ChildWindows;

namespace XiquiparcsReunits_Silverlight.ViewModels
{
    public class GestionarFacturesViewModel : ViewModel
    {

        //Variable estàtica que marca el nombre màxim de reintents de lectura
        private static int maxreintents = Int32.Parse(Configure.Max_async_retries);

        GestorXiquiparc.GestorXiquiparcClient client;

        private ChildWindow finestra;
       
        /// <summary>
        /// Propietat que registra un recurs facturat que s'està modificant
        /// </summary>
        #region ViewModelProperty: Recurs_Facturat
        private Recurs_facturat recurs_Facturat;
        public Recurs_facturat Recurs_Facturat
        {
            get
            {
                return recurs_Facturat;
            }
            set
            {
                recurs_Facturat = value;
                OnPropertyChanged("Recurs_Facturat");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra les factures i les reserves de l'usuari registrat
        /// </summary>
        #region ViewModelProperty: Factures
        private ObservableCollection<Factura> factures;
        public ObservableCollection<Factura> Factures
        {
            get
            {
                InitFactures();
                return factures;
            }
            private set
            {
                factures = value; OnPropertyChanged("Factures");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra les reserves (factures amb recursos del futur)
        /// </summary>
        #region ViewModelProperty: Reserves
        private ObservableCollection<Factura> reserves;
        public ObservableCollection<Factura> Reserves
        {
            get
            {
                
                return reserves;
            }
            private set{
                reserves = value; OnPropertyChanged("Reserves");
            }
        }
        #endregion


        /// <summary>
        /// Propietat que registra el dia d'inici del calendari de consulta de disponibilitat
        /// Per defecte es avui
        /// </summary>
        #region ViewModelProperty: DataInici
        private DateTime dataInici=DateTime.Now;
        public DateTime DataInici { get { return dataInici; }
            set { dataInici = value; OnPropertyChanged("DataInici"); }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el dia de fi del calendari de consulta de disponibilitat
        /// Per defecte es avui + 3 anys
        /// </summary>
        #region ViewModelProperty: DataFi
        private DateTime dataFi = DateTime.Now.AddYears(Int32.Parse(Configure.Rang_anys));
        public DateTime DataFi { get { return dataFi; }
            set { dataFi = value; OnPropertyChanged("DataFi"); }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el dia dels recursos reservats (de factures amb recursos del futur)
        /// </summary>
        #region ViewModelProperty: DiaRecursosReservats
        private DateTime diarecursos_reservats=DateTime.Now;
        public DateTime DiaRecursos_reservats
        {
            get
            {
                return diarecursos_reservats;
            }
            set
            {
                diarecursos_reservats = value; OnPropertyChanged("Recursos_reservats");
            }
        }
        #endregion

        

        /// <summary>
        /// Propietat que registra els recursos reservats (de factures amb recursos del futur)
        /// </summary>
        #region ViewModelProperty: RecursosReservats
        public Recurs RecursAConsultarDisponibilitat {get; set;}
        public ObservableCollection<Recurs_facturat> Recursos_reservats      
        {
         get
            {
                return this.Recursos_facturats_dia(RecursAConsultarDisponibilitat, DiaRecursos_reservats);
            }
        }
        #endregion

        

        /// <summary>
        /// Constructor del ViewModel que inicialititza els DelegateCommand i els llistats
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarFacturesViewModel(IEventAggregator messageBus)
        {
            this.messageBus = messageBus;
                        
            Factures = new ObservableCollection<Factura>();
            Reserves = new ObservableCollection<Factura>();
            
            DisponibilitatRecursCommand = new DelegateCommand<Recurs>(DisponibilitatRecurs);
            DisponibilitatOfertaCommand = new DelegateCommand<Oferta>(DisponibilitatOferta);
            
            //InitFactures();
                        
        }

        /// <summary>
        /// Funció que inicialitza els variables factures i reserves amb les lectures asíncrones de la persistencia,
        /// es reintentarà tants cops com el paràmetre de maxreintents
        /// </summary>
        private void InitFactures()
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
            {
                client = InitClient();
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                // Contador intern dels reintents de lectura
                int reintents = 0;
                //Inicialitzo la lectura de les factures de manera asíncrona (es reintenta en el nombre d'ocasions 
                //definit a l'arxiu Configure abans de llençar l'error)
                try
                {
                    reintents++;
                    llegeix_factures();
                }
                catch (FaultException err)
                {
                    if (reintents < maxreintents)
                    {
                        client.Abort();
                        llegeix_factures();
                    }
                    else
                    {
                        MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    }
                }
                catch (Exception err)
                {
                    if (reintents < maxreintents)
                    {
                        client.Abort();
                        llegeix_factures();
                    }
                    else
                    {
                        MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    }
                }

            }
        }

        /// <summary>
        /// Funció que llegeix les factures de l'usuari registrat i les afegeix a la llista. També afegeix les reserves (factures del futur)
        /// </summary>
        private void llegeix_factures()
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
            {
                //client = InitClient();
                //Inicialitzo la lectura dels contenidors de manera asíncrona
                client.GetFacturesByUsuariCompleted += (sender, e) =>
                {
                    if (e.Result != null)
                    {
                        //En aquest punt estem en un moment asíncron i s'han obtingut totes les factures de l'usuari
                        foreach (GestorXiquiparc.Factura it in e.Result)
                        {
                            Factura factura = Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio);
                            factures.Add(factura);

                            // A partir d'aquí, afegim asíncronament els recursos facturats d'aquesta factura a un llistat
                            // A més, es comprova si es reserva (té recursos facturats al futur) i els afegeixo a la llista de reserves
                            try
                            {
                                client.GetRecursosFacturatsByFacturaCompleted += (sender2, elem) =>
                                {
                                    try
                                    {
                                        if (elem.Result != null)
                                            foreach (GestorXiquiparc.Recurs_facturat item in elem.Result)
                                            {
                                                factura.Recursos_facturats.Add(Recurs_facturat.CreateRecurs_facturat(item.id_factura, item.id_recurs, item.import, item.quantitat, item.percentatge_IVA, item.data_inici_servei, item.data_fi_servei));
                                                if (item.data_inici_servei > DateTime.Now)
                                                {
                                                    factura.EsReserva = true;
                                                    reserves.Add(factura);
                                                }
                                            }
                                        client.CloseAsync();
                                    }
                                    catch (FaultException error)
                                    {
                                        MessageBox.Show(Strings.Error_de_cerca + error.Message);
                                    }
                                    catch (Exception error)
                                    {
                                        MessageBox.Show(Strings.Error_desconegut + error.Message);
                                    }
                                }; client.GetRecursosFacturatsByFacturaAsync(factura.Id_factura);
                            }
                            catch (FaultException error)
                            {
                                MessageBox.Show(Strings.Error_de_cerca + error.Message);
                            }
                            catch (Exception error)
                            {
                                MessageBox.Show(Strings.Error_desconegut + error.Message);
                            }
                        }
                    }
                    client.CloseAsync();
                };
                client.GetFacturesByUsuariAsync(SingletonLoggedUser.Instance.LoggedUser.Id_usuari, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            }
        }

        /// <summary>
        /// Comanda que s'executa al voler conèixer la disponibilitat d'un recurs
        /// </summary>
        #region DelegateCommand: DisponibilitatRecurs
        public ICommand DisponibilitatRecursCommand { get; protected set; }

        /// <summary>
        /// Fa apareixer una finestra amb un calendari on el client pot consultar un llistat de les reserves
        /// del recurs triat, dia a dia
        /// </summary>
        /// <param name="objecte">Recurs o oferta a inserir a la factura</param>
        private void DisponibilitatRecurs(Recurs recurs)
        {
            DataInici = DateTime.Now;
            DataFi=DataInici.AddYears(Int32.Parse(Configure.Rang_anys));
            RecursAConsultarDisponibilitat = recurs;
            finestra = new Disponibilitat();
            finestra.Show();
        }
        #endregion

        /// <summary>
        /// Comanda que s'executa al voler conèixer la disponibilitat d'una oferta
        /// </summary>
        #region DelegateCommand: DisponibilitatOferta
        public ICommand DisponibilitatOfertaCommand { get; protected set; }

        /// <summary>
        /// Mostra una finestra on el client pot veure quines reserves hi ha fetes d'un oferta, en qualsevol dia
        /// Aquesta funció transforma l'oferta en un recurs nou i el passa a la funció DisponibilitatRecurs
        /// </summary>
        /// <param name="objecte">Oferta a consultar la disponibilitat</param>
        private void DisponibilitatOferta(Oferta oferta)
        {
            try
            {
                //Activo que la finestra principal passi a mode "pensant"
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                GestorXiquiparc.GestorXiquiparcClient client = InitClient();
                AsyncRunner<GestorXiquiparc.GetRecursCompletedEventArgs> actRecurs = new AsyncRunner<GestorXiquiparc.GetRecursCompletedEventArgs>();
                actRecurs.invoke = () =>
                {
                    client.GetRecursAsync(oferta.Id_recurs, actRecurs);
                };
                actRecurs.registerCallback(client, (o, e) =>
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    if (e.Result != null)
                    {
                        GestorXiquiparc.Recurs it = e.Result;
                        //En aquest punt estem en un moment asíncron i s'han obtingut el recurs de l'oferta
                        DataFi = oferta.Data_fi_oferta;
                        DataInici = oferta.Data_inici_oferta;
                        RecursAConsultarDisponibilitat=Recurs.CreateRecurs(it.id_recurs, it.id_seccio, it.preu, it.nom_IVA, it.quantitat, it.disponibilitat_controlable, it.descripcio);
                        finestra = new Disponibilitat();
                        finestra.Show();
                    }
                    client.CloseAsync();
                });
                actRecurs.initiateCall();
            }
            catch (FaultException e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }
            
        }
        #endregion

        /// <summary>
        /// Funció que retorna un llistat dels recursos facturats d'un recurs, en un dia concret
        /// cercats del total. Aquesta funció serveix per mostrar la disponibilitat d'un recurs en 
        /// un dia complert
        /// </summary>
        public ObservableCollection<Recurs_facturat> Recursos_facturats_dia(Recurs recurs, DateTime data)
        {
            ObservableCollection<Recurs_facturat> recursos = new ObservableCollection<Recurs_facturat>();
            //Llegeixo asíncronament tots els recursos facturats del recurs en concret i comprovo la data
            try
            {
                //Activo que la finestra principal passi a mode "pensant"
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                GestorXiquiparc.GestorXiquiparcClient client = InitClient();
                AsyncRunner<GestorXiquiparc.GetRecursosFacturatsByRecursCompletedEventArgs> actFactUsu = new AsyncRunner<GestorXiquiparc.GetRecursosFacturatsByRecursCompletedEventArgs>();
                actFactUsu.invoke = () =>
                {
                    client.GetRecursosFacturatsByRecursAsync(recurs.Id_recurs, actFactUsu);
                };
                actFactUsu.registerCallback(client, (o, e) =>
                {
                    if (e.Result != null)
                    {
                        //En aquest punt estem en un moment asíncron i s'han obtingut tots els recursos facturats del recurs
                        foreach (GestorXiquiparc.Recurs_facturat it in e.Result)
                            //Comprovo si el recurs facturat inclou el dia buscat (dona igual l'inici pot ser al 
                            //passat, pero acabar avui o en el futur; pot ser d'avui pero acabar avui o al futur o del futur
                            if (it.data_inici_servei.Date  <=data.Date && it.data_fi_servei.Date>=data.Date)
                                recursos.Add(Recurs_facturat.CreateRecurs_facturat(it.id_factura, it.id_recurs,it.import, it.quantitat, it.percentatge_IVA, it.data_inici_servei, it.data_fi_servei));
                    }
                    client.CloseAsync();
                });
                actFactUsu.initiateCall();
            }
            catch (FaultException e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }
            return recursos;
        }
       

    }

}