﻿using System;
using System.Windows;
using System.Windows.Input;
using XiquiparcsReunits_Silverlight.Model;
using XiquiparcsReunits_Silverlight.Tools;
using Microsoft.Practices.Prism.Events;
using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Commands;
using System.ServiceModel;
using System.Windows.Markup;
using System.ComponentModel;

namespace XiquiparcsReunits_Silverlight.ViewModels
{
    public class GestionarPaginesViewModel : ViewModel
    {
        //Variable estàtica que marca el nombre màxim de reintents de lectura
        private static int maxreintents = Int32.Parse(Configure.Max_async_retries);

        /// <summary>
        /// Propietat que registra la pàgina a ser Parsejada
        /// </summary>
        #region ViewModelProperty: Pagina
        private Pagina pagina;
        public Pagina Pagina
        {
            get
            {
                return pagina;
            }
            private set
            {
                pagina = value; OnPropertyChanged("Pagina");
            }
        }
        #endregion


        /// <summary>
        /// Propietat que registra les pàgines
        /// </summary>
        #region ViewModelProperty: Pagines
        private ObservableCollection<Pagina> pagines;
        public ObservableCollection<Pagina> Pagines
        {
            get
            {
                return pagines;
            }
            private set
            {
                pagines = value; OnPropertyChanged("Pagines");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra els continguts
        /// </summary>
        #region ViewModelProperty: Continguts
        private ObservableCollection<Contingut> continguts;
        public ObservableCollection<Contingut> Continguts
        {
            get
            {
                return continguts;
            }
            private set
            {
                continguts = value; OnPropertyChanged("Continguts");
                //Inicialitzo la variable XamlViewData amb els templates corresponents
                xamlViewData = "";
                foreach (Contingut it in continguts)
                {
                    Continent continent = null;
                    foreach (Continent cont in continents)
                        if (cont.Id_continent.Equals(it.Continent))
                            continent = cont;
                    xamlViewData += "<" + continent.Observacions + "\" Template=\"{StaticResource " + it.Id_contingut + "}\" ";
                    xamlViewData += it.Xaml;
                    xamlViewData += "></" + continent.Observacions + ">";
                }
            }
        }
        #endregion
        /// <summary>
        /// Propietat que registra els continents
        /// </summary>
        #region ViewModelProperty: Continents
        private ObservableCollection<Continent> continents;
        public ObservableCollection<Continent> Continents
        {
            get
            {
                return continents;
            }
            private set
            {
                continents = value; OnPropertyChanged("Continents");
            }
        }
        #endregion


        /// <summary>
        /// Constructor del ViewModel que inicialititza els DelegateCommand i els llistats
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarPaginesViewModel(IEventAggregator messageBus)
        {
            this.messageBus = messageBus;

            ModifyContingutCommand = new DelegateCommand<Contingut>(ModifyContingut, CanModifyContingut);
            DeleteContingutCommand = new DelegateCommand<Contingut>(DeleteContingut, CanDeleteContingut);
            CreateContingutCommand = new DelegateCommand<Pagina>(CreateContingut, CanCreateContingut);

            ModifyPaginaCommand = new DelegateCommand<Pagina>(ModifyPagina, CanModifyPagina);
            DeletePaginaCommand = new DelegateCommand<Pagina>(DeletePagina, CanDeletePagina);
            CreatePaginaCommand = new DelegateCommand(CreatePagina, CanCreatePagina);

            NavegarPaginaAutogestionadaCommand = new DelegateCommand<Pagina>(NavegarPaginaAutogestionada);

            Pagines = new ObservableCollection<Pagina>();
            continents = new ObservableCollection<Continent>();
            continguts = new ObservableCollection<Contingut>();
            InitInstancies();
        }

        private GestorXiquiparc.GestorXiquiparcClient client;
            

        /// <summary>
        /// Funció que inicialitza les variables pagines, continguts i continents amb les dades de la persistencia
        /// Es fa de manera asíncrona. A mesura que es llegeixen els continguts, s'afegeixen a la seva pàgina corresponent.
        /// També s'inicialitza el codi XAML de la pàgina actual
        /// </summary>
        private void InitInstancies()
        {
            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
            // Contador intern dels reintents de lectura
            int reintents = 0;
            //Inicialitzo la lectura de les pàgines de manera asíncrona (es reintenta en el nombre d'ocasions 
            //definit a l'arxiu Configure abans de llençar l'error)
            try
            {
                reintents++;
                llegeix_pagines();
            }
            catch (FaultException err)
            {
                if (reintents < maxreintents){
                    client.Abort();
                    llegeix_pagines();
                }
                else
                {
                    MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                }
            }
            catch (Exception err)
            {
                if (reintents < maxreintents)
                {
                    client.Abort();
                    llegeix_pagines();
                }
                else
                {
                    MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                }
            }
        }

        /// <summary>
        /// Funció privada que inicialitza les pàgines, els continguts i els continents
        /// de manera asíncrona. NO captura excepcions. Hauran de ser tractades en un
        /// nivell superior
        /// </summary>
        private void llegeix_pagines(){

            client = InitClient();
            
            // Contador intern dels reintents de lectura
            int reintents = 0;
            
            client.GetPaginesCompleted += (sender, e) =>
                {
                    //En aquest punt estem en un moment asíncron i s'han obtingut totes les pagines
                    foreach (GestorXiquiparc.Pagina it in e.Result)
                        Pagines.Add(Pagina.CreatePagina(it.id_pagina, it.nom_pagina, it.text_menu, it.observacions));
                    client.CloseAsync();

                    try
                    {
                        reintents++;
                        llegeix_continents();
                    }
                    catch (FaultException err)
                    {
                        if (reintents < maxreintents)
                        {
                            client.Abort();
                            llegeix_continents();
                        }
                        else
                        {
                            MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                    }
                    catch (Exception err)
                    {
                        if (reintents < maxreintents)
                        {
                            client.Abort();
                            llegeix_continents();
                        }
                        else
                        {
                            MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                    }
                };
            client.GetPaginesAsync();
        }



        /// <summary>
        /// Funció privada que inicialitza els continguts i els continents
        /// de manera asíncrona. NO captura excepcions. Hauran de ser tractades en un
        /// nivell superior
        /// </summary>
        private void llegeix_continents()
        {
            // Contador intern dels reintents de lectura
            int reintents = 0;
            client = InitClient();

            xamlResourcesData = "";
            
            //Inicialitzo la lectura dels contenidors de manera asíncrona
            client.GetContinentsCompleted += (sender2, e2) =>
                {
                    xamlResourcesData += "<"+Configure.TipusContenidor+".Resources>";
                    //En aquest punt estem en un moment asíncron i s'han obtingut tots els continents
                    foreach (GestorXiquiparc.Continent it in e2.Result)
                    {
                        Continent continent = Model.Continent.CreateContinent(it.id_continent, it.xaml, it.alcada, it.amplada, it.observacions);
                        Continents.Add(continent);
                        xamlResourcesData += "<ControlTemplate x:Key=\"" + it.id_continent + "\" TargetType=\"" + it.observacions + "\">";
                        xamlResourcesData += it.xaml;
                        xamlResourcesData += "</ControlTemplate>";
                    }
                    xamlResourcesData += "</" + Configure.TipusContenidor + ".Resources>";
                    client.CloseAsync();

                    try
                    {
                        reintents++;
                        llegeix_continguts();
                    }
                    catch (FaultException err)
                    {
                        if (reintents < maxreintents)
                        {
                            client.Abort();
                            llegeix_continguts();
                        }
                        else
                        {
                            MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                    }
                    catch (Exception err)
                    {
                        if (reintents < maxreintents)
                        {
                            client.Abort();
                            llegeix_continguts();
                        }
                        else
                        {
                            MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                    }                    
                };
                client.GetContinentsAsync();
        }

        /// <summary>
        /// Funció privada que inicialitza els continguts
        /// de manera asíncrona. NO captura excepcions. Hauran de ser tractades en un
        /// nivell superior
        /// </summary>
       private void llegeix_continguts()
        {
            //Passo a llegir els continguts             
            client = InitClient();

            //Inicialitzo la lectura dels contenidors de manera asíncrona
           
                client.GetContingutsCompleted += (sender3, e3) =>
                {
                    //En aquest punt estem en un moment asíncron i s'han obtingut tots els continents
                    foreach (GestorXiquiparc.Contingut it in e3.Result)
                    {
                        Contingut contingut = Model.Contingut.CreateContingut(it.id_contingut, it.xaml, it.id_pagina, it.id_continent, it.observacions);
                        Continguts.Add(contingut);
                        foreach (Pagina pagina in pagines)
                            if (contingut.Pagina == pagina.Id_pagina)
                                pagina.Continguts.Add(contingut);
                    }
                    client.CloseAsync();
                };
                client.GetContingutsAsync();            
        }

        /// <summary>
        /// Funció que s'executa quan un polsa visualitzar pàgina autogestionada des de el menú
        /// </summary>
        #region DelegateCommand: NavegarPaginaAutogestionada
        public ICommand NavegarPaginaAutogestionadaCommand { get; protected set; }
        /// <summary>
        /// Funció que en rebre un event de canvi de pàgina autogestinada, incialitza la variable
        /// que la interfície XAML llegirà, i mostra la pàgina a la finestra de navegació principal
        /// </summary>
        /// <param name="pagina">Pàgina a ser visualitzada</param>
        private void NavegarPaginaAutogestionada(Pagina pagina)
        {
            this.Pagina = pagina;
            Uri newUri = new Uri("Pagina", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Publish(newUri);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta modificar un contingut
        /// </summary>
        #region DelegateCommand: ModifyContingutCommand
        public ICommand ModifyContingutCommand { get; protected set; }


        private void ModifyContingut(Contingut contingut)
        {
            try
            {
                client = InitClient();
                client.UpdateContingutCompleted += (sender, e) =>
                {
                      MessageBox.Show(Strings.Actualitzacio_completada);
                      client.CloseAsync();
                };
                client.UpdateContingutAsync(contingut.Id_contingut, contingut.Pagina, contingut.Continent, contingut.Xaml, contingut.Observacions, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
          
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }
        /// <summary>
        /// Funció que retorna el dret si l'usuari pot modificar un contingut
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanModifyContingut(Contingut contingut)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
            {
                return SingletonLoggedUser.Instance.RolLoggedUser.Contingut.modifica;
            }
            else return false;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta esborrar un contingut
        /// </summary>
        #region DelegateCommand: DeleteContingut
        public ICommand DeleteContingutCommand { get; protected set; }

        private void DeleteContingut(Contingut contingut)
        {
            try
            {
                client = InitClient();
                client.DeleteContingutCompleted += (sender, e) =>
                {
                    Pagines[contingut.Pagina - 1].Continguts.Remove(contingut);
                    MessageBox.Show(Strings.Actualitzacio_completada);
                    client.CloseAsync();
                };
                client.DeleteContingutAsync(contingut.Id_contingut, contingut.Pagina, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }
        /// <summary>
        /// Funció que retorna el dret sobre si l'usuari pot esborrar un contingut
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanDeleteContingut(Contingut contingut)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Contingut.esborra;
            else return false;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta crear un contingut
        /// </summary>
        #region DelegateCommand: CreateContingut
        public ICommand CreateContingutCommand { get; protected set; }

        /// <summary>
        /// Crea un nou contingut i l'inserta a la persistencia, a partir d'aquell moment es modificable
        /// S'assigna el primer continent de la colecció, si no existeix cap es mostra un missatge 
        /// </summary>
        /// <param name="pagina">Pàgina a la qual s'assignarà el nou contingut</param>
        private void CreateContingut(Pagina pagina)
        {
            GestorXiquiparc.GestorXiquiparcClient client = InitClient();

            try
            {

                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                Contingut contingut = Contingut.CreateContingut(-1, Configure.Exemple_XAML, pagina.Id_pagina, "", "");

                if (continents.Count > 0)
                {
                    //Inicialitzo la inserció del contingut creat
                    AsyncRunner<GestorXiquiparc.CreateContingutCompletedEventArgs> act1 = new AsyncRunner<GestorXiquiparc.CreateContingutCompletedEventArgs>();
                    act1.invoke = () =>
                    {
                        client.CreateContingutAsync(contingut.Pagina, Continents[0].Id_continent, contingut.Xaml, contingut.Observacions, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, act1);
                    };
                    act1.registerCallback(client, (obj, elem) =>
                    {
                        try
                        {
                            contingut.Id_contingut = elem.Result.id_contingut;
                            pagina.Continguts.Add(contingut);
                            MessageBox.Show(Strings.Actualitzacio_completada);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            client.CloseAsync();
                        }
                        catch (FaultException e)
                        {
                            MessageBox.Show(Strings.Error_de_cerca + e.Message);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show(Strings.Error_desconegut + e.Message);
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                    });
                    act1.initiateCall();
                }
                else MessageBox.Show(Strings.Abans_insertar_contingut);
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message);
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
            }

        }
        /// <summary>
        /// Funció que retorna el dret si l'usuari pot crear un contingut
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanCreateContingut(Pagina pagina)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Contingut.crea;
            else return false;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta crear una pàgina
        /// </summary>
        #region DelegateCommand: CreatePagina
        public ICommand CreatePaginaCommand { get; protected set; }

        /// <summary>
        /// Crea una nova pàgina i l'inserta a la persistencia, a partir d'aquell moment es modificable
        /// S'assigna al següent punt de la colecció
        /// </summary>
        private void CreatePagina()
        {
            try
            {

                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                Pagina pagina = Pagina.CreatePagina(-1, Configure.Exemple_titol, Configure.Exemple_menu, "");

                GestorXiquiparc.GestorXiquiparcClient client = InitClient();

                //Inicialitzo la inserció de la pàgina creada
                AsyncRunner<GestorXiquiparc.CreatePaginaCompletedEventArgs> actCreaPag = new AsyncRunner<GestorXiquiparc.CreatePaginaCompletedEventArgs>();
                actCreaPag.invoke = () =>
                {
                    client.CreatePaginaAsync(pagina.Titol, pagina.Menu, pagina.Observacions, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, actCreaPag);
                };
                actCreaPag.registerCallback(client, (obj, elem) =>
                {
                    try
                    {
                        pagina.Id_pagina = elem.Result.id_pagina;
                        Pagines.Add(pagina);
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        client.CloseAsync();
                        MessageBox.Show(Strings.Actualitzacio_completada);
                    }
                    catch (FaultException e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_de_cerca + e.Message);
                    }
                    catch (Exception e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_desconegut + e.Message);
                    }
                });
                actCreaPag.initiateCall();
            }
            catch (FaultException e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }
        /// <summary>
        /// Funció que retorna el dret si l'usuari pot crear una pàgina
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanCreatePagina()
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Pagina.crea;
            else return false;
        }
        #endregion


        /// <summary>
        /// Funció que s'executa quan un usuari accepta esborrar una pàgina
        /// </summary>
        #region DelegateCommand: DeletePagina
        public ICommand DeletePaginaCommand { get; protected set; }

        private void DeletePagina(Pagina pagina)
        {
            try
            {
                client = InitClient();

                client.DeletePaginaCompleted += (sender, e) =>
                {
                    MessageBox.Show(Strings.Actualitzacio_completada);
                    Pagines.Remove(pagina);
                    client.CloseAsync();
                };
                client.DeletePaginaAsync(pagina.Id_pagina, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
           
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }
        /// <summary>
        /// Funció que retorna el dret sobre si l'usuari pot esborrar un contingut
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanDeletePagina(Pagina pagina)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Pagina.esborra;
            else return false;
        }
        #endregion


        /// <summary>
        /// Funció que s'executa quan un usuari accepta modificar una pàgina
        /// </summary>
        #region DelegateCommand: ModifyPaginaCommand
        public ICommand ModifyPaginaCommand { get; protected set; }

        private void ModifyPagina(Pagina pagina)
        {
            try
            {
                client = InitClient();
                
                client.UpdatePaginaCompleted += (sender, e) =>
                {
                    MessageBox.Show(Strings.Actualitzacio_completada);
                    client.CloseAsync();
                };
                client.UpdatePaginaAsync(pagina.Id_pagina, pagina.Titol, pagina.Menu, pagina.Observacions, SingletonLoggedUser.Instance.LoggedUser.Id_usuari);
            }
            catch (FaultException e)
            {
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }

        /// <summary>
        /// Funció que retorna el dret si l'usuari pot modificar una pàgina
        /// </summary>
        /// <returns>Cert si té el dret</returns>
        private bool CanModifyPagina(Pagina pagina)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
            {
                return SingletonLoggedUser.Instance.RolLoggedUser.Pagina.modifica;
            }
            else return false;
        }
        #endregion

        private string iniciXAML = "<"+Configure.TipusContenidor+" "+Configure.IniciXAML+">";
        private string fiXAML = "</"+Configure.TipusContenidor+">";
        private string xamlResourcesData = "";
        private string xamlViewData = "";
        public string XamlViewData 
        { 
            get 
            {
                xamlViewData = "";
                //Carrego tots els continguts de la pàgina actual a la varible xamlViewData
                foreach (Contingut it in Continguts)
                {
                    //Obtinc el continent d'aquest contingut
                    foreach (Continent continent in Continents)
                        if (it.Continent.Equals(continent.Id_continent))
                        {
                            if (it.Pagina == this.Pagina.Id_pagina)
                            {
                                xamlViewData += "<" + continent.Observacions + " Height=\"" + continent.Alcada + "\" Width=\"" + continent.Amplada + "\" Template=\"{StaticResource " + it.Continent + "}\" ";
                                xamlViewData += it.Xaml;
                                xamlViewData += "></" + continent.Observacions + ">";
                            }
                        }
                }
                //El codi XAML es compondrà d'una capcelera (iniciXAML) amb els includes, un camp de Resources (xamlResourcesData), 
                //els controls pròpiament dits (xamlViewData) i un tancament (fiXAML)
                return iniciXAML + xamlResourcesData + xamlViewData + fiXAML; 
            } 
        }
    }
}
