﻿using Microsoft.Practices.Prism.Commands;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_Silverlight.Tools;
using XiquiparcsReunits_Silverlight.Model;
using System;
using System.ServiceModel;
using XiquiparcsReunits_Silverlight.Views.ChildWindows;
using System.Windows.Controls;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using System.ComponentModel;

namespace XiquiparcsReunits_Silverlight.ViewModels
{
    public class GestionarRecursosViewModel : ViewModel
    {
        //Variable estàtica que marca el nombre màxim de reintents de lectura
        private static int maxreintents = 3;
        private ChildWindow finestra;
        private Recurs recurs_a_facturar;

        /// <summary>
        /// Propietat que registra una factura que s'està modificant
        /// </summary>
        #region ViewModelProperty: FacturaAModificar
        private Factura facturaAModificar;
        public Factura FacturaAModificar
        {
            get
            {
                return facturaAModificar;
            }
            private set
            {
                facturaAModificar = value; OnPropertyChanged("FacturaAModificar");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra un recurs que s'està afegint a una factura
        /// </summary>
        #region ViewModelProperty: RecursAInserir
        private Recurs recursAInserir;
        public Recurs RecursAInserir
        {
            get
            {
                return recursAInserir;
            }
            private set
            {
                recursAInserir = value; OnPropertyChanged("RecursAInserir");
            }
        }
        #endregion


        /// <summary>
        /// Propietat que registra la modificació de el recurs facturat que s'inserirà a la factura
        /// </summary>
        #region ViewModelProperty: Recurs_Facturat
        private Recurs_facturat recurs_facturat;
        public Recurs_facturat Recurs_Facturat
        {
            get
            {
                return recurs_facturat;
            }
            private set
            {
                recurs_facturat = value; OnPropertyChanged("Recurs_Facturat");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra la data marge d'inici que es permetrà afegir el recurs facturat
        /// En cas de ser una oferta, aquesta data correspondrà a la data d'inici de l'oferta
        /// </summary>
        #region ViewModelProperty: DataInici
        private DateTime dataInici = DateTime.Now;
        public DateTime DataInici
        {
            get
            {
                return dataInici;
            }
            set
            {
                dataInici = value; OnPropertyChanged("DataInici");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra la data marge de fi que es permetrà afegir el recurs facturat
        /// En cas de ser una oferta, aquesta data correspondrà a la data de fi de l'oferta
        /// </summary>
        #region ViewModelProperty: DataFi
        private DateTime dataFi = DateTime.Now.AddYears(1);
        public DateTime DataFi
        {
            get
            {
                return dataFi;
            }
            set
            {
                dataFi = value; OnPropertyChanged("DataFi");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra els recursos del sistema
        /// </summary>
        #region ViewModelProperty: Recursos
        private ObservableCollection<Recurs> recursos;
        public ObservableCollection<Recurs> Recursos
        {
            get
            {
                return recursos;
            }
            private set
            {
                recursos = value; OnPropertyChanged("Recursos");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra els recursos del sistema
        /// </summary>
        #region ViewModelProperty: Ofertes
        private ObservableCollection<Oferta> ofertes;
        public ObservableCollection<Oferta> Ofertes
        {
            get
            {
                return ofertes;
            }
            private set
            {
                ofertes = value; OnPropertyChanged("Ofertes");
            }
        }
        #endregion

        private GestorXiquiparc.GestorXiquiparcClient client;

        /// <summary>
        /// Constructor de la View Model
        /// </summary>
        /// <param name="messageBus"></param>
        public GestionarRecursosViewModel(IEventAggregator messageBus)
        {
            this.messageBus = messageBus;
            Ofertes = new ObservableCollection<Model.Oferta>();
            Recursos = new ObservableCollection<Model.Recurs>();

            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);

            InitRecursos();

            CancelCreateRecursCommand = new DelegateCommand(CancelCreateRecurs);
            AcceptaCreateRecursCommand = new DelegateCommand(AcceptaCreateRecurs, CanModifyFactura);
            ReservarRecursCommand = new DelegateCommand<Recurs>(ReservarRecurs, CanReservarRecurs);
            ReservarOfertaCommand = new DelegateCommand<Oferta>(ReservarOferta, CanReservarOferta);
        }

        /// <summary>
        /// Funció privada que inicialitza els recursos i les ofertes de manera asíncrona, primer els recursos, despres les ofertes
        /// </summary>
        private void InitRecursos()
        {// Contador intern dels reintents de lectura
            int reintents = 0;
            //Inicialitzo la lectura de manera asíncrona (es reintenta en 3 ocasions abans de llençar l'error)
            try
            {
                reintents++;
                llegeix_recursos();
            }
            catch (FaultException err)
            {
                if (reintents < maxreintents)
                {
                    client.Abort();
                    llegeix_recursos();
                }
                else
                {
                    MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                }
            }
            catch (Exception err)
            {
                if (reintents < maxreintents)
                {
                    client.Abort();
                    llegeix_recursos();
                }
                else
                {
                    MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                }
            }
           
        }

        /// <summary>
        /// Funció privada que inicialitza els recursos
        /// de manera asíncrona. NO captura excepcions. Hauran de ser tractades en un
        /// nivell superior
        /// </summary>
        private void llegeix_recursos()
        {
            //Llegeixo els recursos de manera asíncrona
            client = InitClient();
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);

                AsyncRunner<GestorXiquiparc.GetRecursosCompletedEventArgs> actRecursos = new AsyncRunner<GestorXiquiparc.GetRecursosCompletedEventArgs>();
                actRecursos.invoke = () =>
                {
                    client.GetRecursosAsync(actRecursos);
                };
                actRecursos.registerCallback(client, (o, e) =>
                {
                        foreach (GestorXiquiparc.Recurs it in e.Result)
                        {
                            Recursos.Add(Recurs.CreateRecurs(it.id_recurs, it.id_seccio, it.preu, it.nom_IVA, it.quantitat, it.disponibilitat_controlable, it.descripcio));
                        }
                        client.CloseAsync();

                        int reintents = 0;
                        //Inicialitzo la lectura de manera asíncrona (es reintenta en 3 ocasions abans de llençar l'error)
                        try
                        {
                            reintents++;
                            llegeix_ofertes();
                        }
                        catch (FaultException err)
                        {
                            if (reintents < maxreintents)
                            {
                                client.Abort();
                                llegeix_ofertes();
                            }
                            else
                            {
                                MessageBox.Show(Strings.Error_de_cerca + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            }
                        }
                        catch (Exception err)
                        {
                            if (reintents < maxreintents)
                            {
                                client.Abort();
                                llegeix_ofertes();
                            }
                            else
                            {
                                MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error + reintents.ToString(), MessageBoxButton.OK);
                                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            }
                        }
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                });
                actRecursos.initiateCall();
        }

        private void llegeix_ofertes()
        {
            client = InitClient();
            
            //Llegeixo les ofertes de manera asíncrona
           
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);

                AsyncRunner<GestorXiquiparc.GetOfertesCompletedEventArgs> actOfertes = new AsyncRunner<GestorXiquiparc.GetOfertesCompletedEventArgs>();
                actOfertes.invoke = () =>
                {
                    client.GetOfertesAsync(actOfertes);
                };
                actOfertes.registerCallback(client, (o, e) =>
                {
                    try
                    {
                        foreach (GestorXiquiparc.Oferta it in e.Result)
                        {
                            Ofertes.Add(Oferta.CreateOferta(it.id_oferta, it.import_unitari, it.id_recurs, it.quantitat_ofertada, it.data_inici_oferta, it.data_fi_oferta, it.descripcio));
                        }
                        client.CloseAsync();
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    }
                    catch (FaultException err)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Actualizacio_fallida + err.Message);
                    }
                    catch (Exception err)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_desconegut + err.Message);
                    }
                });
                actOfertes.initiateCall();
        }

        /// <summary>
        /// Funció que s'executa quan un usuari tria reservar un recurs o una oferta
        /// Apareix una finestra on el usuari introdueix els paràmetres
        /// </summary>
        #region DelegateCommand: ReservarRecurs
        public ICommand ReservarRecursCommand { get; protected set; }

        /// <summary>
        /// Afegeix a una nova factura o a una factura en estat modificable de l'usuari un nou Recurs o Oferta i 
        /// fa apareixer una finestra on pot modificar els paràmetres
        /// </summary>
        /// <param name="objecte">Recurs o oferta a inserir a la factura</param>
        private void ReservarRecurs(Recurs objecte)
        {
            recurs_a_facturar = objecte;
            GestorXiquiparc.GestorXiquiparcClient client = InitClient();
            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
            try
            {
                //Comprovo si un usuari té alguna factura en estat modificable
                ObservableCollection<Factura> factures = new ObservableCollection<Factura>();
                AsyncRunner<GestorXiquiparc.GetFacturesByUsuariCompletedEventArgs> act1 = new AsyncRunner<GestorXiquiparc.GetFacturesByUsuariCompletedEventArgs>();
                act1.invoke = () =>
                {
                    client.GetFacturesByUsuariAsync(SingletonLoggedUser.Instance.LoggedUser.Id_usuari, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, act1);
                };
                act1.registerCallback(client, (obj, elem) =>
                {
                    try
                    {
                        //Estem en un punt asíncron amb la lectura de les factures disponibles
                        foreach (GestorXiquiparc.Factura it in elem.Result)
                            factures.Add(Factura.CreateFactura(it.id_factura, it.usuari_factura, it.estat_cobrament, it.modificable, it.a_compte, it.data_cobrament, it.descripcio));

                        client.CloseAsync();

                        //Un cop tenim les factures, comprovo si hi ha alguna que sigui modificable
                        Factura factura = null;
                        bool found = false;
                        for (int i = 0; i < factures.Count; i++)
                        {
                            if (factures[i].Modificable)
                            {
                                factura = factures[i]; found = true; break;
                            }
                        }
                        if (!found)
                        {
                            //Si no hi ha cap factura modificable, passo a crear-ne una, en estat comptat, de manera asíncrona
                            factura = Factura.CreateFactura("", SingletonLoggedUser.Instance.LoggedUser.Id_usuari, "Comptat", true, 0, DateTime.Now, "");

                            try
                            {
                                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
                                GestorXiquiparc.GestorXiquiparcClient client2 = InitClient();
                                AsyncRunner<GestorXiquiparc.CreateFacturaCompletedEventArgs> actFactUsu = new AsyncRunner<GestorXiquiparc.CreateFacturaCompletedEventArgs>();
                                actFactUsu.invoke = () =>
                                {
                                    client2.CreateFacturaAsync(factura.Id_usuari_factura, factura.Estat_cobrament, factura.Data_cobrament, factura.Descripcio, factura.A_compte, factura.Modificable, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, actFactUsu);
                                };
                                actFactUsu.registerCallback(client2, (o, e) =>
                                {
                                    if (e.Result != null)
                                    {
                                        
                                        FacturaAModificar = Factura.CreateFactura(e.Result.id_factura, e.Result.usuari_factura, e.Result.estat_cobrament, e.Result.modificable, e.Result.a_compte, e.Result.data_cobrament, e.Result.descripcio);


                                        //Si es tracta d'un recurs normal, l'assigno per a que hi tingui accés el codi XAML
                                        //Les dates d'inici i fi, les deixo per defecte
                                        RecursAInserir = objecte;
                                        Recurs_Facturat = Recurs_facturat.CreateRecurs_facturat(factura.Id_factura, RecursAInserir.Id_recurs, RecursAInserir.Preu, 0, RecursAInserir.Percentatge_IVA, DateTime.Now, DateTime.Now.AddHours(1));

                                        //Mostro una finestra de modificació del recurs o la oferta a afegir a la factura
                                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                                        finestra = new ReservarServeiWindow();

                                        finestra.Show();
                                    }
                                    client.CloseAsync();
                                });
                                actFactUsu.initiateCall();
                            }
                            catch (FaultException e)
                            {
                                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
                            }
                            catch (Exception e)
                            {
                                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                                MessageBox.Show(Strings.Error_desconegut + e.Message);
                            }
                        }
                        else
                        {
                            //En aquest punt l'objecte factura refereix una factura modificable anterior llegida de la persistencia 
                            FacturaAModificar = factura;

                            //Si es tracta d'un recurs normal, l'assigno per a que hi tingui accés el codi XAML
                            //Les dates d'inici i fi, les deixo per defecte
                            RecursAInserir = objecte;
                            Recurs_Facturat = Recurs_facturat.CreateRecurs_facturat(factura.Id_factura, RecursAInserir.Id_recurs, RecursAInserir.Preu, 0, RecursAInserir.Percentatge_IVA, DateTime.Now, DateTime.Now.AddHours(1));

                            //Mostro una finestra de modificació del recurs o la oferta a afegir a la factura
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            finestra = new ReservarServeiWindow();

                            finestra.Show();
                        }
                    }
                    catch (FaultException e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_de_cerca + e.Message);
                        if (finestra != null) finestra.Close();
                    }
                    catch (Exception e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_desconegut + e.Message);
                        if (finestra != null) finestra.Close();
                    }
                });
                act1.initiateCall();
            }
            catch (FaultException e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }
        }
        /// <summary>
        /// Funció que retorna el dret si l'usuari pot crear (la modificació es comprova després a la finestra de confirmació)
        /// </summary>
        /// <param name="objecte">Recurs a inserir</param>
        /// <returns>Cert si té el dret</returns>
        private bool CanReservarRecurs(Recurs objecte)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Factura.crea;
            else return false;
        }
        #endregion

        /// <summary>
        /// Commanda que permet reservar una oferta, fent apareixer una finestra
        /// </summary>
        #region DelegateCommand: ReservarOferta
        public ICommand ReservarOfertaCommand { get; protected set; }

        /// <summary>
        /// Afegeix a una nova factura o a una factura en estat modificable de l'usuari una nova Oferta i 
        /// fa apareixer una finestra on pot modificar els paràmetres
        /// Aquesta funció transforma l'oferta en un recurs nou i el passa a la funció ReservaRecurs
        /// </summary>
        /// <param name="objecte">Recurs o oferta a inserir a la factura</param>
        private void ReservarOferta(Oferta objecte)
        {
            GestorXiquiparc.GestorXiquiparcClient client = InitClient();
            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);
            try
            {
                AsyncRunner<GestorXiquiparc.GetRecursCompletedEventArgs> act1 = new AsyncRunner<GestorXiquiparc.GetRecursCompletedEventArgs>();
                act1.invoke = () =>
                {
                    client.GetRecursAsync(objecte.Id_recurs, act1);
                };
                act1.registerCallback(client, (obj, elem) =>
                {
                    try
                    {
                        GestorXiquiparc.Recurs recurs = elem.Result;
                        DataFi = objecte.Data_fi_oferta;
                        DataInici = objecte.Data_inici_oferta;
                        client.CloseAsync();
                        ReservarRecurs(Recurs.CreateRecurs(recurs.id_recurs, recurs.id_seccio, objecte.Import_unitari, recurs.nom_IVA, objecte.Quantitat_ofertada, recurs.disponibilitat_controlable, objecte.Descripcio));
                    }
                    catch (FaultException e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
                    }
                    catch (Exception e)
                    {
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        MessageBox.Show(Strings.Error_desconegut + e.Message);
                    }
                });
                act1.initiateCall();
            }
            catch (FaultException e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Actualizacio_fallida + e.Message);
            }
            catch (Exception e)
            {
                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                MessageBox.Show(Strings.Error_desconegut + e.Message);
            }

        }


        /// <summary>
        /// Funció que retorna el dret si l'usuari pot crear (la modificació es comprova després a la finestra de confirmació)
        /// </summary>
        /// <param name="objecte">Oferta a inserir</param>
        /// <returns>Cert si té el dret</returns>
        private bool CanReservarOferta(Oferta objecte)
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Factura.crea;
            else return false;
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari cancela la creació d'una reserva (recurs en factura)
        /// </summary>
        #region DelegateCommand: CancelCreateRecurs
        public ICommand CancelCreateRecursCommand { get; protected set; }

        private void CancelCreateRecurs()
        {
            finestra.Close();
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta la creació d'una reserva (recurs en factura)
        /// </summary>
        #region DelegateCommand: AcceptaCreateRecurs
        public ICommand AcceptaCreateRecursCommand { get; protected set; }

        private void AcceptaCreateRecurs()
        {
            if (Recurs_Facturat.Quantitat > 0 && Recurs_Facturat.Quantitat <= recurs_a_facturar.Quantitat && Recurs_Facturat.Data_inici_servei <= Recurs_Facturat.Data_fi_servei)
            {
                //Recurs_facturat.Insert(Recurs_Facturat);
                GestorXiquiparc.GestorXiquiparcClient client = InitClient();
                try
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);

                    AsyncRunner<GestorXiquiparc.CreateRecursFacturatCompletedEventArgs> actCreaRecurs = new AsyncRunner<GestorXiquiparc.CreateRecursFacturatCompletedEventArgs>();
                    actCreaRecurs.invoke = () =>
                    {
                        client.CreateRecursFacturatAsync(Recurs_Facturat.Id_factura, Recurs_Facturat.Id_recurs, Recurs_Facturat.Quantitat, Recurs_Facturat.Data_inici_servei, Recurs_Facturat.Data_fi_servei, SingletonLoggedUser.Instance.LoggedUser.Id_usuari, actCreaRecurs);
                    };
                    actCreaRecurs.registerCallback(client, (o, e) =>
                    {
                        try
                        {
                            GestorXiquiparc.Recurs_facturat resultat = e.Result;
                            client.CloseAsync();
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                        }
                        catch (FaultException err)
                        {
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            MessageBox.Show(Strings.Actualizacio_fallida + err.Message);
                        }
                        catch (Exception err)
                        {
                            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                            MessageBox.Show(Strings.Error_desconegut + err.Message);
                        }
                    });
                    actCreaRecurs.initiateCall();
                }
                catch (FaultException e)
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK);
                }
                catch (Exception e)
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK);
                }

                finestra.Close();
            }
            else MessageBox.Show(Strings.Revisi_dates_quantitat, Strings.Error, MessageBoxButton.OK);
        }

        /// <summary>
        /// Funció que retorna el dret si l'usuari pot crear i modificar una factura propia
        /// </summary>
        /// <param name="recurs">Recurs a inserir</param>
        /// <returns>Cert si té el dret</returns>
        private bool CanModifyFactura()
        {
            if (SingletonLoggedUser.Instance.LoggedUser != null)
                return SingletonLoggedUser.Instance.RolLoggedUser.Factura.modifica;
            else return false;
        }
        #endregion
    }
}