﻿using System.Collections.ObjectModel;
using Microsoft.Practices.Prism.Events;
using XiquiparcsReunits_Silverlight.Model;
using System.Windows.Input;
using XiquiparcsReunits_Silverlight.Tools;
using System;
using System.Windows;
using System.ServiceModel;
using XiquiparcsReunits_Silverlight.Views.ChildWindows;
using System.Windows.Controls;
using Microsoft.Practices.Prism.Commands;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_Silverlight.ViewModels
{
    public class GestionarUsuarisViewModel : ViewModel
    {

        /// <summary>
        /// Propietat que registra el usuari que s'està modificant
        /// </summary>
        private Usuari usuariAModificar;
        public Usuari UsuariAModificar
        {
            get { return usuariAModificar; }
            set { usuariAModificar = value; OnPropertyChanged("UsuariAModificar"); }
        }


        public GestionarUsuarisViewModel(IEventAggregator messageBus)
        {
            //usuaris= Usuari.GetInstancies();
            //rols = Rol.GetInstancies();
            CreateTheUserCommand = new DelegateCommand(CreateTheUser);
            AddTheUserCommand = new DelegateCommand(AddTheUser);
            ExitWindowCommand = new DelegateCommand(ExitWindow);
            RecordatoriCommand = new DelegateCommand(Recordatori);
            this.messageBus = messageBus;
        }

        private ChildWindow finestra;

        /// <summary>
        /// Funció que s'executa quan un usuari vol recordar la contrasenya
        /// </summary>
        #region DelegateCommand: Recordatori
        public ICommand RecordatoriCommand { get; protected set; }

        private void Recordatori()
        {
            MessageBox.Show(Strings.Recordatori_Info, Strings.Recordatori_contrasenya, MessageBoxButton.OK);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari vol crear un nou usuari
        /// </summary>
        #region DelegateCommand: CreateTheUser
        public ICommand CreateTheUserCommand { get; protected set; }

        private void CreateTheUser()
        {
            //Mostro una finestra de modificació del usuari
            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
            UsuariAModificar = Usuari.CreateUsuari("", "", "", "", "", "", "", "", 0, "", "", "", "test", Configure.Rol_usuari_web);
            finestra = new AltaUsuariPage();

            finestra.Show();
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari accepta la creació d'un nou usuari. S'inserta el usuari a la persistència i es
        /// tanca la finestra.
        /// </summary>
        #region DelegateCommand: AddTheUser
        public ICommand AddTheUserCommand { get; protected set; }

        private void AddTheUser()
        {
            if (UsuariAModificar.Nom_usuari != "" && UsuariAModificar.Nom_fiscal_usuari != "" && UsuariAModificar.NIF_usuari != "" && UsuariAModificar.Codpos > 0 && UsuariAModificar.Codpos <= 99999 && UsuariAModificar.Direccio != "" && UsuariAModificar.Estat != "" && UsuariAModificar.Movil != "" && UsuariAModificar.Poblacio != "" && UsuariAModificar.Provincia != "" && UsuariAModificar.Pwd != "" && UsuariAModificar.Telefon1 != "")
            {

                GestorXiquiparc.GestorXiquiparcClient client = new GestorXiquiparc.GestorXiquiparcClient();
                OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
                MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

                messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", Configure.Usuari_web));
                messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", Configure.Password_usuari_web));

                messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(true);

                try
                {
                    client.CreateUsuariCompleted += (sender, e) =>
                    {
                        UsuariAModificar.Id_usuari = e.Result.id_usuari;
                        client.CloseAsync();
                        finestra.Close();
                        messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    };
                    client.CreateUsuariAsync(UsuariAModificar.Nom_usuari, UsuariAModificar.NIF_usuari, UsuariAModificar.Nom_fiscal_usuari, UsuariAModificar.Telefon1, UsuariAModificar.Telefon2, UsuariAModificar.Movil, UsuariAModificar.Direccio, UsuariAModificar.Poblacio, UsuariAModificar.Codpos, UsuariAModificar.Provincia, UsuariAModificar.Estat, UsuariAModificar.Pwd, UsuariAModificar.Observacions, UsuariAModificar.Nom_rol, Configure.Usuari_web);

                }
                catch (FaultException e)
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK);
                }
                catch (Exception e)
                {
                    messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Publish(false);
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK);
                }

            }
            else
                MessageBox.Show(Strings.No_es_posible_registrar, Strings.Error, MessageBoxButton.OK);
        }
        #endregion

        /// <summary>
        /// Funció que s'executa quan un usuari cancela la creació d'un nou usuari
        /// </summary>
        #region DelegateCommand: ExitWindow
        public ICommand ExitWindowCommand { get; protected set; }

        private void ExitWindow()
        {
            finestra.Close();
        }
        #endregion
    }
}