﻿using System;
using System.Windows;
using System.Windows.Input;
using Microsoft.Practices.Prism.Events;
using Microsoft.Practices.Prism.Commands;
using XiquiparcsReunits_Silverlight.Model;
using System.ServiceModel;
using System.Threading;
using XiquiparcsReunits_Silverlight.Tools;
using System.Windows.Threading;
using System.ComponentModel;
using System.ServiceModel.Channels;

namespace XiquiparcsReunits_Silverlight.ViewModels
{
    public class PrincipalViewModel : ViewModel
    {
        /// <summary>
        /// Propietat que registra si hi ha accions asíncrones realitzant-se. La finestra principal es bloqueja
        /// </summary>
        #region ViewModelProperty: IsBusy
        private Boolean isBusy = false;
        public Boolean IsBusy
        {
            get
            {
                return isBusy;
            }

            private set
            {
                isBusy = value; OnPropertyChanged("IsBusy");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra si el usuari actualment identificat té permis d'accedir a escriptori i web
        /// </summary>
        #region ViewModelProperty: Acces_web
        private bool acces_web;
        public bool Acces_web
        {
            get
            {
                return acces_web;
            }

            private set
            {
                acces_web = value; OnPropertyChanged("Acces_web");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el usuari actualment identificat, i llegeix asíncronament el seu rol
        /// </summary>
        #region ViewModelProperty: LoggedUser
        public Usuari LoggedUser
        {
            get
            {
                return SingletonLoggedUser.Instance.LoggedUser;
            }
            set
            {
                if (value != null)
                {
                    SingletonLoggedUser.Instance.LoggedUser = value;
                    OnPropertyChanged("LoggedUser");
                    IsLoggedUser = true;

                    IsBusy = true;

                    try
                    {
                        GestorXiquiparc.GestorXiquiparcClient client = InitClient();

                        AsyncRunner<GestorXiquiparc.GetRolCompletedEventArgs> act1 = new AsyncRunner<GestorXiquiparc.GetRolCompletedEventArgs>();
                        act1.invoke = () =>
                        {
                            client.GetRolAsync(LoggedUser.Nom_rol, act1);
                        };
                        act1.registerCallback(client, (o, e) =>
                        {
                            try
                            {
                                GestorXiquiparc.Rol it = e.Result;
                                RolLoggedUser = Rol.CreateRol(it.nom_rol, it.modifica_empresa, it.usuari, it.reserva_forana, it.reserva, it.recurs, it.seccio, it.oferta, it.factura, it.factura_forana, it.cobrament, it.tipus_IVA, it.pagina, it.continent, it.contingut, it.acces_escriptori, it.rol1);
                                SingletonLoggedUser.Instance.RolLoggedUser = RolLoggedUser;
                                client.CloseAsync();
                                Acces_web = SingletonLoggedUser.Instance.RolLoggedUser.Acces_escriptori;
                                IsBusy = false;
                            }
                            catch (FaultException err)
                            {
                                MessageBox.Show(Strings.Identificacio_fallida + err.Message); IsBusy = false;
                            }
                            catch (Exception err)
                            {
                                MessageBox.Show(Strings.Error_desconegut + err.Message); IsBusy = false;
                            }
                        });
                        act1.initiateCall();
                    }
                    catch (FaultException e)
                    {
                        MessageBox.Show(Strings.Identificacio_fallida + e.Message); IsBusy = false;
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(Strings.Error_desconegut + e.Message); IsBusy = false;
                    }
                }
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra el rol de l'usuari actualment identificat
        /// </summary>
        #region ViewModelProperty: RolLoggedUser
        public Rol RolLoggedUser
        {
            get
            {
                return SingletonLoggedUser.Instance.RolLoggedUser;
            }
            set
            {
                if (value != null)
                {
                    SingletonLoggedUser.Instance.RolLoggedUser = value;
                    OnPropertyChanged("RolLoggedUser");
                }
            }
        }
        #endregion

        /// <summary>
        /// Propietat que registra si hi ha un usuari registrat al sistema
        /// </summary>
        #region ViewModelProperty: IsLoggedUser
        private bool isLoggedUser;
        public bool IsLoggedUser
        {
            get
            {
                return isLoggedUser;
            }
            private set { isLoggedUser = value; OnPropertyChanged("IsLoggedUser"); }
        }
        #endregion


        /// <summary>
        /// Funció que serà cridada en cada event d'espera en altres ViewModel i que modificarà a IsBusy en aquest
        /// </summary>
        /// <param name="isBusy">Nou estat de IsBusy</param>
        public void ChangeIsBusy(Boolean IsBusy)
        {
            this.IsBusy = IsBusy;
        }

        /// <summary>
        /// Propietat que controla en la identificació si s'ha introduït un nom d'usuari
        /// </summary>
        #region ViewModelProperty: Login
        private string _login = "";
        public string Login
        {
            get
            {
                return _login;
            }

            set
            {
                _login = value;
                OnPropertyChanged("Login");
            }
        }
        #endregion

        /// <summary>
        /// Propietat que controla en la identificació si s'ha introduït un codi d'accés
        /// </summary>
        #region ViewModelProperty: Password


        private string _password = "";
        /// <summary>
        /// Propietat que manté en format MD5 el password introduits en el login
        /// </summary>
        public string Password
        {
            get
            {
                return _password;
            }

            set
            {
                if (value != null)
                    _password = MD5Core.GetHashString(value);
                else _password = null;
            }
        }
        #endregion


        /// <summary>
        /// Constructor per defecte del ViewModel
        /// </summary>
        /// <param name="messageBus">Bus de missatges amb els View</param>
        public PrincipalViewModel(IEventAggregator messageBus)
        {

            this.messageBus = messageBus;
            this.SelectedPage = new Uri("/Home", UriKind.Relative);
            // página por defecto
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Subscribe(ChangePage);

            //Missatge de comunicació que registra una lectura asíncrona on l'aplicació queda "pensant"
            messageBus.GetEvent<CompositePresentationEvent<Boolean>>().Subscribe(ChangeIsBusy);

        }

        /// <summary>
        /// Comanda assignada al botó del View per registar-se
        /// </summary>
        #region DelegateCommand: LoginTheUser
        private DelegateCommand loginTheUserCommand;

        public ICommand LoginTheUserCommand
        {
            get
            {
                if (loginTheUserCommand == null)
                {
                    loginTheUserCommand = new DelegateCommand(LoginTheUser);
                }
                return loginTheUserCommand;
            }
        }

        /// <summary>
        /// Funció que comprova el usuari a partir del id inscrit per l'usuari a Login, de manera asíncrona
        /// </summary>
        private void LoginTheUser()
        {
            if (Login != null || Login != "")
            {
                IsBusy = true;
                try
                {
                    //Comprovem si el codi i el password son correctes, si no ho son es llença una excepcio
                    TestLoggedUser(this.Login, this.Password);
                }
                catch (FaultException e)
                {
                    MessageBox.Show(Strings.Identificacio_fallida + e.Message, Strings.Error, MessageBoxButton.OK); IsBusy = false;
                }
                catch (Exception e)
                {
                    MessageBox.Show(Strings.Error_desconegut + e.Message, Strings.Error, MessageBoxButton.OK); IsBusy = false;
                }
            }
            else MessageBox.Show(Strings.Error_de_cerca, Strings.Error, MessageBoxButton.OK); IsBusy = false;
        }

        /// <summary>
        /// Funció que comprova el usuari i el password (en MD5), en cas de coincidir assigna l'usuari, en cas contrari, llença un error
        /// </summary>
        /// <param name="usuari">Usuari al qual comparar el camp Password</param>
        /// <param name="pwd">Password en format MD5 a comprovar</param>
        private void TestLoggedUser(String id_usuari, String pwd)
        {
            GestorXiquiparc.GestorXiquiparcClient client = InitClient();
            OperationContextScope scope = new OperationContextScope((IContextChannel)client.InnerChannel);
            MessageHeaders messageHeadersElement = OperationContext.Current.OutgoingMessageHeaders;

            messageHeadersElement.Add(MessageHeader.CreateHeader("username", "", id_usuari));
            messageHeadersElement.Add(MessageHeader.CreateHeader("password", "", pwd));

            AsyncRunner<GestorXiquiparc.GetUsuariByNomCompletedEventArgs> act1 = new AsyncRunner<GestorXiquiparc.GetUsuariByNomCompletedEventArgs>();
            act1.invoke = () =>
            {
                client.GetUsuariByNomAsync(Login, Login, act1);
            };
            act1.registerCallback(client, (o, elem) =>
            {
                try
                {
                    GestorXiquiparc.Usuari it = elem.Result;
                    Usuari usuari = Usuari.CreateUsuari(it.nom_usuari, it.NIF_usuari, it.nom_fiscal_usuari, it.telefon1, it.telefon2, it.movil, it.direccio, it.poblacio, it.codpos, it.provincia, it.estat, it.pwd, it.observacions, it.nom_rol);
                    usuari.Id_usuari = it.id_usuari;
                    LoggedUser = usuari;
                    client.CloseAsync();
                    IsBusy = false;
                }
                catch (FaultException err)
                {
                    MessageBox.Show(Strings.Identificacio_fallida + err.Message, Strings.Error, MessageBoxButton.OK); IsBusy = false;
                }
                catch (Exception err)
                {
                    MessageBox.Show(Strings.Error_desconegut + err.Message, Strings.Error, MessageBoxButton.OK); IsBusy = false;
                }
            });
            act1.initiateCall();
        }
        #endregion

        #region DelegateCommand: DeLoginTheUser
        private DelegateCommand deLoginTheUserCommand;
        public ICommand DeLoginTheUserCommand
        {
            get
            {
                if (deLoginTheUserCommand == null)
                {
                    deLoginTheUserCommand = new DelegateCommand(DeLoginTheUser);
                }
                return deLoginTheUserCommand;
            }
        }

        /// <summary>
        /// Funció que des-identifica a un usuari, tornant les variables al punt inicial.
        /// </summary>
        private void DeLoginTheUser()
        {
            this.LoggedUser = null;
            IsLoggedUser = false;
            Acces_web = false;
            this.RolLoggedUser = null;
            this.Login = null;
            this.Password = null;
            //Torno a la pàgina per defecte
            this.SelectedPage = new Uri("/Home", UriKind.Relative);
            messageBus.GetEvent<CompositePresentationEvent<Uri>>().Subscribe(ChangePage);

        }
        #endregion
    }
}
